# http://www.28lines.com/?p=8
import os
import re
from setuptools import setup, find_packages
from setuptools.command.test import test


class TestRunner(test):
    def run(self, *args, **kwargs):
        if self.distribution.install_requires:
            self.distribution.fetch_build_eggs(
                self.distribution.install_requires)
        if self.distribution.tests_require:
            self.distribution.fetch_build_eggs(
                self.distribution.tests_require)
        from waonline.tests.runtests import runtests
        runtests()


def get_data_files(*args, **kwargs):

    EXT_PATTERN = kwargs.get('ext') or '\.(html|js|txt|css|po|mo)'

    data_dict = {}
    for pkg_name in args:
        data_files = []
        for dirpath, dirnames, filenames in os.walk(pkg_name):
            rel_dirpath = re.sub("^" + pkg_name + '/', '',  dirpath)
            # Ignore dirnames that start with '.'
            for i, dirname in enumerate(dirnames):
                if dirname.startswith('.'): del dirnames[i]
            if filenames:
                data_files += [os.path.join(rel_dirpath, f) for f in filenames
                               if re.search(EXT_PATTERN, f)]
        data_dict[pkg_name] = data_files
    return data_dict

setup(
    name='webazienda',
    version = "1.2.3",
    description='Waonline: interfaccia per webazienda', 
    author='Essepaghe',
    author_email='dev@essepaghe.it',
    packages=find_packages('src', exclude=['tests', 'tests.*']),
    package_dir = {'': 'src'},
    platforms='any',
    zip_safe=False,
    classifiers=[
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Framework :: Django',
    ],
    test_suite = "waonline.tests",
    cmdclass={"test": TestRunner},
    install_requires = [
        'setuptools',
        'SQLAlchemy==0.6.8',
        'docutils',
        'nose',
        'psycopg2',
        'reportlab',
        'simplejson',
        'tornado==2.2',
        'django',
        'ipdbplugin',
        'mako',
        'python-daemon',
        'ipdb',
        'cElementTree',
        'fabric',
        'pyquery',
       ],
    package_data=get_data_files('src/waonline'),
    entry_points = {
      'console': [
         'wademon = waonline.wademon:main',
         ]
      }
    )

