# coding: latin-1
import string

from waonline.core.rbase             import RBase
from waonline.utils.utility           import formatData, framData, isDataOK
from waonline.apps.gestione.tabelle.tabelle_l import Comune

PARAMETRI = ['db', 'nome', 'cognome', 'sesso', 'data', 'comune']
COD_MESI = ['A', 'B', 'C', 'D', 'E', 'H', 'L', 'M', 'P', 'R', 'S', 'T']
VOCALI = ['A', 'E', 'I', 'O', 'U']
CONSONANTI = ['B', 'C', 'D', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'X', 'Y', 'Z']
ALFABETO = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U',
            'V', 'W', 'X', 'Y', 'Z']
CONV_CAR_PARI = {'A':0, '0':0, 'B':1, '1':1, 'C':2, '2':2, 'D':3, '3':3, 'E':4, '4':4, 'F':5, '5':5, 'G':6, '6':6,
                 'H':7, '7':7, 'I':8, '8':8, 'J':9, '9':9, 'K':10, 'L':11, 'M':12, 'N':13, 'O':14, 'P':15, 'Q':16,
                 'R':17, 'S':18, 'T':19, 'U':20, 'V':21, 'W':22, 'X':23, 'Y':24, 'Z':25}
CONV_CAR_DISPARI = {'A':1, '0':1, 'B':0, '1':0, 'C':5, '2':5, 'D':7, '3':7, 'E':9, '4':9, 'F':13, '5':13, 'G':15,
                    '6':15, 'H':17, '7':17, 'I':19, '8':19, 'J':21, '9':21, 'K':2, 'L':4, 'M':18, 'N':20, 'O':11,
                    'P':3, 'Q':6, 'R':8, 'S':12, 'T':14, 'U':16, 'V':10, 'W':22, 'X':25, 'Y':24, 'Z':23}


class CodiceFiscale():

    def __init__(self, **kwargs):
        self.errore = None
        for k in PARAMETRI:
            if k in kwargs and kwargs[k]:
                if k == "db":
                    v = kwargs[k]
                else:
                    v = str(kwargs[k]).translate(string.maketrans("", "aeeiou")).upper()
                setattr(self, k, v)

            
    def calcola(self):
        cod_fiscale = None

        if self._parametri_ok():
            # Cognome
            vocali = "".join([c for c in self.cognome if c in VOCALI])
            consonanti = "".join([c for c in self.cognome if c in CONSONANTI])
            cognome = consonanti[0:3]
            if len(cognome) < 3:
                cognome += vocali[0:3 - len(cognome)]
            if len(cognome) < 3:
                cognome += "X" * (3 - len(cognome))
    
            # Nome
            vocali = "".join([c for c in self.nome if c in VOCALI])
            consonanti = "".join([c for c in self.nome if c in CONSONANTI])
            if len(consonanti) > 3:
                nome = consonanti[0] + consonanti[2:4]
            else:
                nome = consonanti[0:3]
            if len(nome) < 3:
                nome += vocali[0:3 - len(nome)]
            if len(nome) < 3:
                nome += "X" * (3 - len(nome))
    
            # Anno, mese e giorno
            anno = framData(self.data, True)[0][2:4]
            mese = COD_MESI[framData(self.data)[1] - 1]
            giorno = "%02d" %(framData(self.data)[2] + (40 if self.sesso == "F" else 0))
    
            # parziale...
            cod_fiscale = cognome + nome + anno + mese + giorno + self.comune
    
            # Codice di controllo
            cod_fiscale += self._carattere_controllo(cod_fiscale)
        return cod_fiscale                


    def codice_ok(self, cod_fisc, formale=False):
        cod_fisc = cod_fisc.upper()
        if len(cod_fisc) <> 16:
            return False
        if formale:
            if cod_fisc[15] == self._carattere_controllo(cod_fisc[0:15]):
                return True
            else:
                return False
        else:
            if cod_fisc == self.calcola():
                return True
            else:
                return False
        

    def _carattere_controllo(self, cod_fisc):
        somma = 0
        for i in range(1, 16):
            if i % 2 == 0:
                somma += CONV_CAR_PARI[cod_fisc[i - 1]]
            else:
                somma += CONV_CAR_DISPARI[cod_fisc[i - 1]]
        return ALFABETO[(somma % 26)]


    def _parametri_ok(self):
        for k in PARAMETRI:
            try:
                v = getattr(self, k)
                if k == "sesso" and v not in ['F', 'M']:
                    self._errore(k, v)
                if k == "data" and not isDataOK(formatData(v, "int", "%Y%m%d")):
                    self._errore(k, v)
                if k == "comune" and not self.db.query(Comune).filter(Comune.cod_fiscale == v).first():
                    self._errore(k, v)
            except:
                self._errore(k)
            finally:
                if self.errore:
                    return False
        return True


    def _errore(self, k, v=None, errore=None):
        if errore:
            self.errore = errore
        else:            
            if v:
                self.errore = "Parametro '%s' errato: %s" %(k, v)
            else:
                self.errore = "Parametro '%s' non specificato" %(k)


