# -*- coding: utf-8 -*-
""" Modulo per la gestione dei dipendenti """
import json
from datetime           import date
from xml.etree          import cElementTree as ET

from sqlalchemy.sql     import and_, or_

from waonline.apps.gestione.gest_valoriLista   import carica_causali_in_session
from waonline.apps.gestione.gest_valoriLista   import GestioneListe
from waonline.core.rbase                       import RBase
from .tabelle.dipendenti import Dipendente, DipendenteOrario, \
        DipendenteOrarioSett, DipendenteOrarioSettGG, \
        DipendenteCarFam, DipendenteFest, DipendenteFestTab, \
        DipendenteQual, DipendenteRappPrec, DipendenteStraMagg
from .tabelle.ditte      import DittaSommDist
from .cod_fisc           import CodiceFiscale
from waonline.apps.presenze.tabelle.presenze   import Presenze
from waonline.apps.gestione.tabelle.tabelle_c  import Aliquota, Causale, Contratto, \
        ContrattoApprend, ContrattoLivello, ContrattoQual
from waonline.apps.gestione.tabelle.tabelle_l  import Comune
from waonline.apps.gestione.tabelle.tabelle_v  import CampoWA, Qualifica
from waonline.apps.gestione.tabelle.tabelle_p  import TabellaP
from waonline.utils.utility            import formatData, isDataOK, getObjInfo, \
        toJSON, to_dict
from waonline.core.wadecorators       import WaAuth
from waonline.core.wadecohelpers      import dbl_date
from waonline.core import sessions

class GestDipti(RBase):

    """ Classe per la gestione dei dipendenti  """
    def _post_init_(self):
        """ Init delle variabili per il modulo """
        self.funzione_id = self.ga("id_funzione", None)
        self.filtro_nome = self.ga("nominativo", None)
        self.escludi_cessati = self.ga("escludi_cessati", None)
        self.archivio = self.ga("archivio", None)
        self.stato = self.ga("stato", None)
        self.tipo = self.ga("tipo", None)
        if self._periodo:
            self.periodo = self._periodo
        else:
            self.periodo = date.today()
            self.periodo = (self.periodo.year * 100) + self.periodo.month
        self.nuovo_codice = None
        if self.ga("nuovo_codice"):
            self.nuovo_codice = self.ga("nuovo_codice").zfill(5)


    """
        G E S T I O N E  -  A N A G R A F I C A
    """
    @sessions.require_lock
    def gestione_anagrafiche(self):
        self._add_to_log()
        if self.tipo == "a": # anagrafica
            if self.stato == "lista":
                if self.archivio == "s":
                    self._elenca_somm_dist()
                else:
                    self._elenca_dipendenti()
                if self.archivio == 'p':
                    carica_causali_in_session(self)
            elif self.stato == "visualizza" or self.stato == "inserisci" or self.stato == "duplica":
                if self.archivio == "s":
                    self._modifica_somm_dist()
                else:
                    self._modifica_dipendente()
            elif self.stato == "salva":
                if self.archivio == "s":
                    self._salva_somm_dist()
                else:
                    self._salva_dipendente()
            elif self.stato == "elimina":
                if self.archivio == "s":
                    self._elimina_somm_dist()
                else:
                    self._elimina_dipendente()
            elif self.stato == "verifica":
                self._verifica_dipendente()
        elif self.tipo == "o": # orario
            if self.stato == "visualizza":
                self._modifica_orario()
            elif self.stato == "salva":
                self._salva_orario()
        elif self.tipo == "i": # info
            self._repHead('Content-Type', 'application/javascript')
            if self._dipte_id:
                if self.archivio == "s":
                    dipte = self._q(DittaSommDist).filter(DittaSommDist.id == self._dipte_id).first()
                    self.risposta.contenuto = toJSON({'info':
                        '[%s] %s %s' % (dipte.id, dipte.cognome, dipte.nome)})
                else:
                    dipte = self._dipti[0]
                    self.risposta.contenuto = toJSON({'info':
                        '[%s] %s - %s' % (dipte.id, dipte.codice, dipte.nominativo)})
            else:
                self.risposta.contenuto = toJSON({'info': 'Nuovo dipendente'})
                return
            #self._fresponse()
            #self._ftipoDati()
        else:
            # funzioni varie
            try:
                return getattr(self, "_%s" %(self.tipo))()
            except:
                pass

    def _has_periodo(self, periodo, ditta_id=None):
        try:
            if periodo == None:
                return False
            ditta_id = ditta_id or self._ditta.id
        except:
            return False
        return periodo in [c.periodo for c in \
                self._ditta.status_wa_rel[0].status_periodi_rel]

    def _get_ultimo_periodo(self, ditta_id=None):
        try:
            ditta_id = ditta_id or self._ditta.id
            return  self._ditta.status_wa_rel[0].status_periodi_rel[-1].periodo
        except:
            return None

    @WaAuth()
    def _elenca_dipendenti(self,
                           data_inizio={"tipo": dbl_date, "default": None},
                           data_fine={"tipo": dbl_date, "default": None},
                           seg=None
                           ):
        """ Restituisce l'elenco dei dipendenti """
        self._repHead("Content-Type", "application/javascript")
        if self.archivio == 'p':
            periodo = self._periodo
            if not self._has_periodo(periodo):
                periodo = self._get_ultimo_periodo()
                if not periodo:
                    periodo = formatData(date.today(), 'int', '%Y%m')
            periodo *= 100
        else:
            periodo = formatData(date.today(), 'int', '%Y%m%d')
        dass = periodo + 100
        squery = "SELECT * FROM ("\
        "SELECT "\
        "  dipendenti.id as id,"\
        "  dipendenti.codice as codice, "\
        "  dipendenti.matricola as matricola, "\
        "  (dipendenti.cognome||' '||dipendenti.nome) AS nominativo, "\
        "  dipendenti.data_ass as data_ass, "\
        "  COALESCE(dipendenti.data_cess, dipendenti.data_cess_fut) AS data_cess,"\
        "  dipendenti_qual.codice as qualifica, "\
        "  dipendenti_qual.centro_costo as centro_costo, "\
        "  dipendenti_qual.livello as livello, "\
        "  dipendenti_qual.rif_inail as inail "\
        "FROM "\
        "  dipendenti, "\
        "  dipendenti_qual "\
        "WHERE dipendenti_qual.dipendente_id = dipendenti.id "\
        "AND dipendenti.ditta_id = %(ditta_id)s "\
        "AND dipendenti_qual.is_qual_att = True "
        if self.archivio != "p":
            squery += " AND dipendenti.is_collab = %s" % (self.archivio == "c")
        squery += ") AS qq WHERE  1=1 "
        if self.archivio == 'p':
            squery += " AND data_ass <= %(dass)s "
        if self.escludi_cessati or self.archivio == 'p':
            squery +=" AND (data_cess >= %(periodo)s OR data_cess IS NULL) "
        if self.filtro_nome:
            self.filtro_nome = '%' + self.filtro_nome + '%'
            squery += " AND LOWER(nominativo) LIKE "\
            "LOWER('%s')" %  self.filtro_nome
        prefs = self._preferenze
        if "CODDIPE" in prefs:
            lsi = lno = ''
            for llista in prefs['CODDIPE']['si']:
                lsi += ", ".join(map(lambda x: "'%s'" % x, llista))
            for llista in prefs['CODDIPE']['no']:
                lno += ", ".join(map(lambda x: "'%s'" % x, llista))
            if lsi:
                squery += " AND codice IN (%s) " % lsi
            if lno:
                squery += " AND codice NOT IN (%s)" % lno
        squery += " ORDER BY codice ASC"
        try:
            squery = squery % {"ditta_id": self._ditta.id, "periodo": periodo,
                    "studio": self.user.studio_codice, 'dass': dass}
            info = to_dict(self._ex(squery).fetchall())
        except Exception, err:
            info = []
            self.errore(err)
        self.risposta.contenuto = toJSON(info)


    def _elenca_somm_dist(self):
        """ Restituisce l'elenco dei somministrati """
        info = []
        for dipte in self._ditta.somm_dist_rel:
            if self.filtro_nome:
                self.filtro_nome = '%' + self.filtro_nome + '%'
                dipte = dipte.filter(
                        "LOWER (cognome || ' ' || nome) like LOWER('%" + (
                            self.filtro_nome) + "%')")

            if self.escludi_cessati == True and dipte.data_uscita:
                dataChk = (self.periodo * 100) + 01
                if dipte.data_uscita < dataChk:
                    continue
            info.append(getObjInfo(dipte,
                includi=['id', 'tipo', 'qualifica', 'livello',
                    'data_ingresso', 'data_uscita', 'nominativo']
                ))
        self.risposta.contenuto = toJSON(info)

    def _modifica_dipendente(self):
        liste = GestioneListe(self)
        dipte = None
        if self.stato != "inserisci":
            dipte = self._dipti[0]
        if dipte:
            dipteQual = dipte.qualifica
        campiWA = self._q(CampoWA).filter(and_(CampoWA.is_solo_pres == True,
            or_(CampoWA.tipo_archivio == ("C" if self.archivio == "c" else "D"),
                    CampoWA.tipo_archivio == None))).order_by(CampoWA.id)
        if self.funzione_id:
            campiWA.filter(CampoWA.funzione == self.funzione_id)
        cod_qual = None
        root = ET.Element("campi")
        for campo in campiWA:
            tag = ET.SubElement(root, "campo")
            ET.SubElement(tag, "codice_campo").text = campo.codice
            ET.SubElement(tag, "descrizione").text = unicode(campo.descrizione)
            if campo.annotazione:
                ET.SubElement(tag, "annotazione").text = unicode(campo.annotazione)
            else:
                ET.SubElement(tag, "annotazione").text = ""
            ET.SubElement(tag, "formato").text = "%s%s" %(str(campo.formato), ("%" if campo.codice == "01236" else ""))
            ET.SubElement(tag, "caratteri").text = str(campo.num_caratteri)
            if campo.is_obbligatorio != None:
                ET.SubElement(tag, "required").text = str(campo.is_obbligatorio)
            if campo.formula:
                ET.SubElement(tag, "listafrom").text = campo.formula
            ET.SubElement(tag, "valida").text = "true" if campo.codice == "01230" else "false"
            subTag = ET.SubElement(tag, "valore")

            # estrazione del valore
            objectRif = None
            if dipte:
                if campo.classe_rif == "Dipendente":
                    objectRif = dipte
                if campo.classe_rif == "DipendenteQual":
                    objectRif = dipteQual
                if not objectRif:
                    continue

            # estrae il valore del campo
            if (self.stato == "duplica" or self.stato == "inserisci") and campo.codice == "00015":
                subTag.text = self.nuovo_codice
            else:
                if self.stato != "inserisci" and objectRif:
                    argomenti = campo.campo_rif.split(":")
                    funzione = None
                    parametro = None
                    indice = None
                    argomenti = campo.campo_rif.split(":")
                    if hasattr(self, argomenti[0]):
                        funzione = getattr(self, argomenti.pop(0))
                    try:
                        parametro = argomenti.pop(0)
                        try:
                            indice = int(argomenti.pop(0))
                        except: pass
                    except: pass
                    if funzione:
                        if parametro:
                            if parametro == "self":
                                if objectRif:
                                    returnVal = funzione(objectRif)
                                else:
                                    returnVal = ""
                            else:
                                if objectRif:
                                    returnVal = funzione(getattr(objectRif, parametro))
                                else:
                                    returnVal = ""
                        else:
                            returnVal = funzione()
                        if indice != None:
                            subTag.text = returnVal[indice]
                        else:
                            subTag.text = returnVal
                    else:
                        if objectRif:
                            subTag.text = unicode(getattr(objectRif, campo.campo_rif, ""))
                        else:
                            subTag.text = ""
                if subTag.text == "None":
                    subTag.text = ""

            # tag "decodifica" per campi associati a tabelle
            if campo.formula:
                try:
                    valore = getattr(objectRif, campo.campo_rif, "")
                    if not valore:
                        valore = ""
                    tipo = type(valore)
                    lquery = getattr(liste, campo.formula)()
                    if ((campo.campo_rif.endswith("_id")) or (tipo == int)):
                        lquery = lquery.filter("id = '%s'" % valore )
                    else:
                        lquery = lquery.filter("codice = '%s'" % valore )
                    lquery = lquery.first()
                    ET.SubElement(tag, "decodifica").text = unicode(lquery.descrizione)
                except Exception, errore:
                    ET.SubElement(tag, "decodifica").text = ""

            # abilitazione campi
            if campo.codice == "01230":
                cod_qual = subTag.text
            ET.SubElement(tag, "enabled").text = "True" if self._campo_abilitato(cod_qual, campo.codice) else "False"

        self._addHead("Content-Type", "text/xml")
        self.risposta.contenuto = ET.tostring(root)
        self._setDocType()


    def _campo_abilitato(self, cod_qual, cod_campo):
        abilita = True
        if cod_qual:
            if cod_campo == "01235" or cod_campo == "01236":
                if cod_qual[1:2] not in ["K", "P"]:
                    abilita = False
            if cod_campo == "01245":
                if cod_qual[0:1] in ["2", "4"]:
                    abilita = False
            if cod_campo == "01260":
                if cod_qual[0:1] not in ["2", "4"]:
                    abilita = False
        return abilita


    def _modifica_somm_dist(self):
        dipte = None
        root = ET.Element("campi")
        if self.stato != "inserisci":
            dipte = self._q(DittaSommDist).filter(DittaSommDist.id == self._dipte_id).first()

        if self.stato == "visualizza":
            # Progressivo
            tag = ET.SubElement(root, "campo")
            ET.SubElement(tag, "codice_campo").text = "00001"
            ET.SubElement(tag, "descrizione").text = "Progressivo"
            ET.SubElement(tag, "annotazione")
            ET.SubElement(tag, "formato").text = "03"
            ET.SubElement(tag, "caratteri").text = "5"
            ET.SubElement(tag, "required").text = "True"
            ET.SubElement(tag, "valore").text = str(dipte.id)
            ET.SubElement(tag, "enabled").text = "False"
            ET.SubElement(tag, "valida").text = "False"
        # Tipo
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00002"
        ET.SubElement(tag, "descrizione").text = "Tipo"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TU"
        ET.SubElement(tag, "caratteri").text = "1"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "listafrom").text = "getSommDist"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.tipo
        ET.SubElement(tag, "decodifica").text = "" if not dipte else self._decodifica_codice("getSommDist", dipte.tipo)
        # Cognome
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00003"
        ET.SubElement(tag, "descrizione").text = "Cognome"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TM"
        ET.SubElement(tag, "caratteri").text = "50"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.cognome
        # Nome
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00004"
        ET.SubElement(tag, "descrizione").text = "Nome"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TM"
        ET.SubElement(tag, "caratteri").text = "50"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.nome
        # Sesso
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00005"
        ET.SubElement(tag, "descrizione").text = "Sesso"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TU"
        ET.SubElement(tag, "caratteri").text = "1"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "listafrom").text = "getGeneri"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.sesso
        ET.SubElement(tag, "decodifica").text = "" if not dipte else self._decodifica_codice("getGeneri", dipte.sesso)
        # Data di nascita
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00006"
        ET.SubElement(tag, "descrizione").text = "Data di nascita"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "D8"
        ET.SubElement(tag, "caratteri").text = "8"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "valore").text = "" if not dipte else formatData(dipte.data_nascita)
        # Comune di nascita
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00007"
        ET.SubElement(tag, "descrizione").text = "Comune di nascita"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "03"
        ET.SubElement(tag, "caratteri").text = "5"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "listafrom").text = "getComuniAll"
        if dipte:
            ET.SubElement(tag, "valore").text = self._cf_comune(dipte.comune_nascita) or ""
        else:
            ET.SubElement(tag, "valore").text = ""
        ET.SubElement(tag, "decodifica").text = "" if not dipte else self._decodifica_codice("getComuniAll", dipte.comune_nascita)
        # Codice fiscale
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00008"
        ET.SubElement(tag, "descrizione").text = "Codice fiscale"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TU"
        ET.SubElement(tag, "caratteri").text = "16"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.cod_fiscale
        # Indirizzo residenza
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00009"
        ET.SubElement(tag, "descrizione").text = "Indirizzo residenza"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TM"
        ET.SubElement(tag, "caratteri").text = "100"
        ET.SubElement(tag, "required").text = "False"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.indirizzo_resid
        # Comune residenza
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00010"
        ET.SubElement(tag, "descrizione").text = "Comune residenza"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "03"
        ET.SubElement(tag, "caratteri").text = "5"
        ET.SubElement(tag, "required").text = "False"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "listafrom").text = "getComuni"
        if dipte:
            ET.SubElement(tag, "valore").text = self._cf_comune(dipte.comune_resid) or ""
        else:
            ET.SubElement(tag, "valore").text = ""
        ET.SubElement(tag, "decodifica").text = "" if not dipte else self._decodifica_codice("getComuni", dipte.comune_resid)
        # Qualifica
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00011"
        ET.SubElement(tag, "descrizione").text = "Qualifica"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TU"
        ET.SubElement(tag, "caratteri").text = "4"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "listafrom").text = "getQualifiche"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.qualifica
        ET.SubElement(tag, "decodifica").text = "" if not dipte else self._decodifica_codice("getQualifiche", dipte.qualifica)
        # Livello
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00012"
        ET.SubElement(tag, "descrizione").text = "Livello"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TU"
        ET.SubElement(tag, "caratteri").text = "2"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "listafrom").text = "getLivelli"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.livello
        ET.SubElement(tag, "decodifica").text = "" if not dipte else self._decodifica_codice("getLivelli", dipte.livello)
        # Data ingresso
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00013"
        ET.SubElement(tag, "descrizione").text = "Data ingresso"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "D8"
        ET.SubElement(tag, "caratteri").text = "8"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "valore").text = "" if not dipte else formatData(dipte.data_ingresso)
        # Data uscita
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00014"
        ET.SubElement(tag, "descrizione").text = "Data uscita"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "D8"
        ET.SubElement(tag, "caratteri").text = "8"
        ET.SubElement(tag, "required").text = "False"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "valore").text = "" if (not dipte or not dipte.data_uscita) else formatData(dipte.data_uscita)
        # Agenzia
        tag = ET.SubElement(root, "campo")
        ET.SubElement(tag, "codice_campo").text = "00015"
        ET.SubElement(tag, "descrizione").text = "Agenzia"
        ET.SubElement(tag, "annotazione")
        ET.SubElement(tag, "formato").text = "TM"
        ET.SubElement(tag, "caratteri").text = "50"
        ET.SubElement(tag, "required").text = "True"
        ET.SubElement(tag, "enabled").text = "True"
        ET.SubElement(tag, "valida").text = "False"
        ET.SubElement(tag, "valore").text = "" if not dipte else dipte.agenzia

        self._addHead("Content-Type", "text/xml")
        self.risposta.contenuto = ET.tostring(root)
        self._setDocType()


    def _salva_dipendente(self):
        lista = self._carica_lista_da_args(False)
        if not self._controlla_anagrafiche(lista):
            return
        if self._dipte_id:
            if self.nuovo_codice:
                # copia
                dipte = self._duplica_anagrafica()
            else:
                # modifica
                dipte = self._dipti[0]
        else:
            #inserimento
            # crea il record dipendente
            dipte = Dipendente()
            dipte.ditta_id = self._ditta.id
            dipte.codice =  self.nuovo_codice
            dipte.is_collab = True if self.archivio == "c" else False
            dipte.matricola = ""
            dipte.cognome = ""
            dipte.nome = ""
            self.sessiondb.add(dipte)
            self.sessiondb.commit()
            # crea il record qualifica del dipendente
            dipteQual = DipendenteQual()
            dipteQual.dipendente_id = dipte.id
            dipteQual.is_qual_att = True
            self.sessiondb.add(dipteQual)
            self.sessiondb.commit()

        cod_qual = None
        for campo in lista:
            # aggiorna il DB con i dati video
            campoDB = self._q(CampoWA).filter(CampoWA.codice == campo['codice']).first()
            if campoDB.classe_rif:
                cls = eval(campoDB.classe_rif)
                if campoDB.classe_rif == "Dipendente":
                    obj = dipte
                else:
                    obj = dipte.qualifica
                if obj:
                    if campo['valore']:
                        if campoDB.formato[0:1] == "0" or campoDB.formato[0:1] == "D":
                            if campoDB.formato[0:1] == "D":
                                valore = formatData(campo['valore'], "int", "%Y%m%d")
                            elif campoDB.formato == "03":
                                valore = campo['valore'].rjust(int(campoDB.num_caratteri), "0")
                            else:
                                valore = int(campo['valore'])
                        elif campoDB.formato[0:1].isdigit():
                            valore = float(campo['valore'])
                        elif campoDB.formato == "CH":
                            valore = bool(campo['valore'])
                        elif campoDB.formato == "TU":
                            valore = campo['valore']
                        else:
                            valore = campo['valore']
                    else:
                        valore = None
                    argomenti = campoDB.campo_rif.split(":")
                    if len(argomenti) > 1:
                        setattr(obj, argomenti[1], valore)
                    else:
                        setattr(obj, argomenti[0], valore)

            # forzatura valori
            if campo['codice'] == "01230" and not dipte.qualifica.data_inizio:
                dipte.qualifica.data_inizio = dipte.data_ass
        self.sessiondb.commit()
        if not dipte.is_collab:
            self._addStato("Dipendente aggiornato")
        else:
            self._addStato("Collaboratore aggiornato")

        #self._modifica_dipendente()


    def _duplica_anagrafica(self):
        dipte_cp = self._q(Dipendente).filter(and_(Dipendente.ditta_id == self._ditta.id,
                                                   Dipendente.id == self._dipte_id,
                                                   Dipendente.is_collab == (True if self.archivio == "c" else False),
                                                   )).first()
        # record Dipendente
        dipte = self._duplica_campi(dipte_cp, Dipendente())
        dipte.ditta_id = self._ditta.id
        dipte.codice = self.nuovo_codice
        # >>> necessario preimpostare il campo per la commit. Dopo gli verra' messo il dato proveniente dal video
        dipte.matricola = self.nuovo_codice
        # <<<
        dipte.is_collab = True if self.archivio == "c" else False
        self.sessiondb.add(dipte)
        self.sessiondb.commit()
        # record DipendenteCarFam
        for rec_cp in dipte_cp.detraz_fiscali_rel:
            rec = self._duplica_campi(rec_cp, DipendenteCarFam())
            dipte.detraz_fiscali_rel.append(rec)
            self.sessiondb.commit()
        # record DipendenteFest
        for rec_cp in dipte_cp.festivita_rel:
            rec = self._duplica_campi(rec_cp, DipendenteFest())
            dipte.festivita_rel.append(rec)
            self.sessiondb.commit()
            # record DipendenteFestTab
            for rec_cp_1 in rec_cp.festivita_rel:
                rec_1 = self._duplica_campi(rec_cp_1, DipendenteFestTab())
                rec.festivita_rel.append(rec_1)
                self.sessiondb.commit()
        # record DipendenteOrario
        for rec_cp in dipte_cp.div_orari_rel:
            rec = self._duplica_campi(rec_cp, DipendenteOrario())
            dipte.div_orari_rel.append(rec)
            self.sessiondb.commit()
            # record DipendenteOrarioSett
            for rec_cp_1 in rec_cp.settimane_rel:
                rec_1 = self._duplica_campi(rec_cp_1, DipendenteOrarioSett())
                rec.settimane_rel.append(rec_1)
                self.sessiondb.commit()
                # record DipendenteOrarioSettGG
                for rec_cp_2 in rec_cp_1.giorni_rel:
                    rec_2 = self._duplica_campi(rec_cp_2, DipendenteOrarioSettGG())
                    rec_1.giorni_rel.append(rec_2)
                    self.sessiondb.commit()
        # record DipendenteQual
        for rec_cp in dipte_cp.qualifiche_rel:
            rec = self._duplica_campi(rec_cp, DipendenteQual())
            dipte.qualifiche_rel.append(rec)
            self.sessiondb.commit()
        # record DipendenteRappPrec
        for rec_cp in dipte_cp.rapp_precedenti_rel:
            rec = self._duplica_campi(rec_cp, DipendenteRappPrec())
            dipte.rapp_precedenti_rel.append(rec)
            self.sessiondb.commit()
        # record DipendenteStraMagg
        for rec_cp in dipte_cp.stra_magg_rel:
            rec = self._duplica_campi(rec_cp, DipendenteStraMagg())
            dipte.stra_magg_rel.append(rec)
            self.sessiondb.commit()

        return dipte


    def _salva_somm_dist(self):
        lista = self._carica_lista_da_args(True)
        if not self._controlla_somm_dist(lista):
            return
        if self._dipte_id:
            if self.nuovo_codice:
                # copia
                dipte = self._duplica_somm_dist()
            else:
                # modifica
                dipte = [sd for sd in self._ditta.somm_dist_rel if sd.id == self._dipte_id][0]
        else:
            #inserimento
            # crea il record somm/dist
            dipte = DittaSommDist()
            dipte.ditta_id = self._ditta.id
            dipte.tipo = ""
            dipte.cognome = ""
            dipte.nome = ""
            dipte.sesso = ""
            dipte.data_nascita = 0
            dipte.comune_nascita = ""
            dipte.cod_fiscale = ""
            dipte.indirizzo_resid = ""
            dipte.comune_resid = ""
            dipte.qualifica = ""
            dipte.livello = ""
            dipte.data_ingresso = 0
            dipte.data_uscita = 0
            dipte.agenzia = ""
            self.sessiondb.add(dipte)
            self.sessiondb.commit()

        for campo in lista:
            # aggiorna il DB con i dati video
            if campo['codice'] == "00002":
                dipte.tipo = campo['valore']
            if campo['codice'] == "00003":
                dipte.cognome = campo['valore']
            if campo['codice'] == "00004":
                dipte.nome = campo['valore']
            if campo['codice'] == "00005":
                dipte.sesso = campo['valore']
            if campo['codice'] == "00006":
                dipte.data_nascita = formatData(campo['valore'], "int", "%Y%m%d")
            if campo['codice'] == "00007":
                dipte.comune_nascita = self._codice_comune(campo['valore'])
            if campo['codice'] == "00008":
                dipte.cod_fiscale = campo['valore']
                if not dipte.cod_fiscale:
                    cod_fisc = CodiceFiscale(db=self.sessiondb, nome=dipte.nome, cognome=dipte.cognome,
                                             sesso=dipte.sesso, data=dipte.data_nascita,
                                             comune=self._cf_comune(dipte.comune_nascita))
                    dipte.cod_fiscale = cod_fisc.calcola()
            if campo['codice'] == "00009":
                dipte.indirizzo_resid = campo['valore']
            if campo['codice'] == "00010":
                dipte.comune_resid = self._codice_comune(campo['valore'])
            if campo['codice'] == "00011":
                dipte.qualifica = campo['valore']
            if campo['codice'] == "00012":
                dipte.livello = campo['valore']
            if campo['codice'] == "00013":
                dipte.data_ingresso = formatData(campo['valore'], "int", "%Y%m%d")
            if campo['codice'] == "00014":
                dipte.data_uscita = formatData(campo['valore'], "int", "%Y%m%d")
            if campo['codice'] == "00015":
                dipte.agenzia = campo['valore']

        self.sessiondb.commit()
        self._addStato("Somministrato/distaccato aggiornato")


    def _duplica_somm_dist(self):
        somm_dist_cp = self._q(DittaSommDist).filter(DittaSommDist.id == self._dipte_id).first()
        somm_dist = self._duplica_campi(somm_dist_cp, DittaSommDist())
        self.sessiondb.add(somm_dist)
        self.sessiondb.commit()
        return somm_dist


    def _elimina_dipendente(self):
        """ Elinmina un dipendente dal database """
        try:
            dipte = self._q(Dipendente.codice, Dipendente.is_collab).filter(Dipendente.id == self._dipte_id).first()
            presenze = self._q(Presenze).filter(and_(Presenze.ditta_id == self._ditta.id,
                                                     Presenze.dipendente_id == self._dipte_id
                                                     )).all()
            for pres in presenze:
                pres.dipendente_cod = dipte.codice
                pres.dipendente_is_collab = dipte.is_collab
            self.sessiondb.commit()
        except Exception, exc: print exc

        self._q(Dipendente).filter(Dipendente.id == self._dipte_id).delete()
        self.sessiondb.commit()
        self.stato = "visualizza"
        self._elenca_dipendenti()


    def _elimina_somm_dist(self):
        self._q(DittaSommDist).filter(DittaSommDist.id == self._dipte_id).delete()
        self.sessiondb.commit()
        self.stato = "visualizza"
        self._elenca_somm_dist()


    def _verifica_dipendente(self):
        dipte = self._q(Dipendente).filter(and_(Dipendente.ditta_id == self._ditta.id,
                                                Dipendente.codice == self.nuovo_codice,
                                                Dipendente.is_collab == (True if self.archivio == "c" else False),
                                                )).first()
        if dipte:
            self._addErr("Codice %s gia' presente in archivio: %s" %("collaboratore" if self.archivio == "c" else "dipendente",
                                                                     self.nuovo_codice))
            return
        self._fresponse()


    def _controlla_anagrafiche(self, lista):
        dipte = None
        if self._dipte_id:
            dipte = self._dipti[0]
        for campo in lista:
            # codice ID
            if campo['codice'] == "00015":
                if not campo['valore']:
                    self._addErr("Campo %s: codice identificativo obbligatorio" %(campo['codice']))
                    continue
            # cognome
            if campo['codice'] == "00020":
                if not campo['valore']:
                    self._addErr("Campo %s: cognome dipendente obbligatorio" %(campo['codice']))
                    continue
            # nome
            if campo['codice'] == "00025":
                if not campo['valore']:
                    self._addErr("Campo %s: nome dipendente obbligatorio" %(campo['codice']))
                    continue
            # centro di costo
            if campo['codice'] == "00150":
                if not campo['valore']:
                    continue
                if not [c for c in self._ditta.centri_costo_rel if c.codice == campo['valore']]:
                    self._addErr("Campo %s:codice centro di costo errato" %(campo['codice']))
                    continue
            # data assunzione
            dataAss = 0
            if campo['codice'] == "01215":
                if campo['valore']:
                    dataAss = formatData(campo['valore'], "int", "%Y%m%d")
                    if isDataOK(dataAss):
                        if not self._impostazioni.is_segreteria:
                            for p in self._ditta.status_wa_rel[0].status_periodi_rel:
                                if p.periodo == formatData(dataAss, "int", "%Y%m") and p.stato == "C":
                                    self._addMess("Campo %s: la data assunzione interessa un periodo chiuso" %(campo['codice']))
                                    continue
                    else:
                        self._addErr("Campo %s: data assunzione errata" %(campo['codice']))
                        continue
                else:
                    self._addErr("Campo %s: data assunzione obbligatoria" %(campo['codice']))
                    continue
            # data cessazione
            if campo['codice'] == "01220":
                if not campo['valore']:
                    continue
                dataCess = formatData(campo['valore'], "int", "%Y%m%d")
                if isDataOK(dataCess):
                    if dataCess < dataAss:
                        self._addErr("Campo %s: data cessazione minore di data assunzione" %(campo['codice']))
                        continue
                    else:
                        if not self._impostazioni.is_segreteria:
                            for p in self._ditta.status_wa_rel[0].status_periodi_rel:
                                if p.periodo == formatData(dataCess, "int", "%Y%m") and p.stato == "C":
                                    self._addMess("Campo %s: la data cessazione interessa un periodo chiuso" %(campo['codice']))
                                    continue
                else:
                    self._addErr("Campo %s: data cessazione errata %s:%s" %(campo['codice'], campo['valore'], dataCess))
                    continue
            # qualifica
            if campo['codice'] == "01230":
                if campo['valore']:
                    # verifica se il codice è in tabella
                    query = self._q(Qualifica).filter(and_(Qualifica.codice == campo['valore'],
                                                           Qualifica.periodo_inizio <= self.periodo,
                                                           Qualifica.periodo_fine >= self.periodo,
                                                           Qualifica.contr_estensione == None,
                                                           )).first()
                    if not query:
                        self._addErr("Campo %s: codice qualifica errato" %(campo['codice']))
                        continue
                    # verifica se non previsto nella qualifica
                    cod_aliquota = None
                    if dipte:
                        cod_aliquota = dipte.qualifica.aliquota
                    if not cod_aliquota:
                        cod_aliquota = self._ditta.aliquota
                    aliquota = self._q(Aliquota).filter(and_(Aliquota.codice == cod_aliquota,
                                                             Aliquota.periodo_inizio <= self.periodo,
                                                             Aliquota.periodo_fine >= self.periodo,
                                                             )).first()
                    is_err = False
                    for qual in aliquota.qualifiche_rel:
                        if qual.sub_rif_cod_aliq == "NPR"                    and \
                           qual.sub_qual[0:1] in ['*', campo['valore'][0:1]] and \
                           qual.sub_qual[1:2] in ['*', campo['valore'][1:2]] and \
                           qual.sub_qual[2:3] in ['*', campo['valore'][2:3]] and \
                           qual.sub_qual[3:4] in ['*', campo['valore'][3:4]]:
                            self._addErr("Campo %s: codice qualifica non previsto per l'aliquota %s" %(campo['codice'], aliquota.codice))
                            continue
                    # verifica se previsto nel contratto del dipendente/ditta
                    cod_contratto = None
                    if dipte:
                        cod_contratto = dipte.qualifica.contratto
                    if not cod_contratto:
                        cod_contratto = self._ditta.contratto
                    contr_id = self._q(Contratto.id).filter(and_(Contratto.codice == cod_contratto,
                                                                 Contratto.periodo_inizio <= self.periodo,
                                                                 Contratto.periodo_fine >= self.periodo
                                                                 )).first()[0]
                    is_prev = False
                    for qual in self._q(ContrattoQual).filter(ContrattoQual.contratto_id == contr_id):
                        j_pos = qual.codice.find("*")
                        if j_pos == 0:
                            is_prev = True
                            break
                        else:
                          if campo['valore'][0:j_pos] == qual.codice[0:j_pos]:
                             is_prev = True
                             break
                    if not is_prev:
                        self._addErr("Campo %s: codice qualifica non previsto dal contratto" %(campo['codice']))
                        continue
                else:
                    self._addErr("Campo %s: codice qualifica obbligatorio" %(campo['codice']))
                    continue
            # tipo part-time
            if campo['codice'] == "01235":
                if campo['valore']:
                    query = self._q(TabellaP).filter(and_(TabellaP.codice == campo['valore'],
                                                          TabellaP.formula == "PART_TIME"
                                                          )).first()
                    if not query:
                        self._addErr("Campo %s: codice tipo part-time errato" %(campo['codice']))
                        continue
            # livello
            if campo['codice'] == "01245":
                if campo['valore']:
                    query = self._q(ContrattoLivello).filter(and_(ContrattoLivello.contratto == self._ditta.contratto,
                                                                  ContrattoLivello.codice == campo['valore'],
                                                                  ContrattoLivello.periodo_inizio <= self.periodo,
                                                                  ContrattoLivello.periodo_fine >= self.periodo
                                                                  )).first()
                    if not query:
                        self._addErr("Campo %s: codice livello errato" %(campo['codice']))
                        continue
            # apprendistato
            if campo['codice'] == "01260":
                if campo['valore']:
                    query = self._q(ContrattoApprend).filter(and_(ContrattoApprend.contratto == self._ditta.contratto,
                                                                  ContrattoApprend.codice == campo['valore'],
                                                                  ContrattoApprend.periodo_inizio <= self.periodo,
                                                                  ContrattoApprend.periodo_fine >= self.periodo
                                                                  )).first()
                    if not query:
                        self._addErr("Campo %s: codice apprendistato errato" %(campo['codice']))
                        continue

        if [m for m in self.risposta.messaggi if m['tipo'] == 'errore']:
            return False
        else:
            return True


    def _controlla_somm_dist(self, lista):
        for campo in lista:
            if campo['codice'] == "00002":
                if campo['valore']:
                    if not self._decodifica_codice("getSommDist", campo['valore']):
                        self._addErr("Campo %s: tipo dipendente errato" %(campo['codice']))
                else:
                    self._addErr("Campo %s: tipo dipendente obbligatorio" %(campo['codice']))
            if campo['codice'] == "00003":
                if campo['valore']:
                    cognome = campo['valore']
                else:
                    cognome = ''
                    self._addErr("Campo %s: cognome obbligatorio" %(campo['codice']))
            if campo['codice'] == "00004":
                if campo['valore']:
                    nome = campo['valore']
                else:
                    nome = ''
                    self._addErr("Campo %s: nome obbligatorio" %(campo['codice']))
            if campo['codice'] == "00005":
                if campo['valore']:
                    sesso = campo['valore']
                    if not self._decodifica_codice("getGeneri", sesso):
                        self._addErr("Campo %s: sesso errato" %(sesso))
                else:
                    sesso = ''
                    self._addErr("Campo %s: sesso obbligatorio" %(campo['codice']))
            if campo['codice'] == "00006":
                if campo['valore']:
                    data = campo['valore']
                    if not isDataOK(formatData(data, "int", "%Y%m%d")):
                        self._addErr("Campo %s: data di nascita errata" %(data))
                else:
                    data = ''
                    self._addErr("Campo %s: data di nascita obbligatoria" %(campo['codice']))
            if campo['codice'] == "00007":
                if campo['valore']:
                    comune = campo['valore']
                    if not self._decodifica_codice("getComuniAll", self._codice_comune(comune)):
                        self._addErr("Campo %s: codice comune di nascita errato" %(comune))
                else:
                    comune = ''
                    self._addErr("Campo %s: codice comune di nascita obbligatorio" %(campo['codice']))
            if campo['codice'] == "00008":
                cf = CodiceFiscale(db=self.sessiondb, nome=nome, cognome=cognome, sesso=sesso,
                                   data=formatData(data, "int", "%Y%m%d"), comune=comune)
                if campo['valore']:
                    if not cf.codice_ok(campo['valore']):
                        self._addErr("Campo %s: codice fiscale errato" %(campo['codice']))
                else:
                    campo['valore'] = cf.calcola()
            if campo['codice'] == "00010":
                if campo['valore']:
                    if not self._decodifica_codice("getComuni", self._codice_comune(campo['valore'])):
                        self._addErr("Campo %s: codice comune di residenza errato" %(campo['codice']))
            if campo['codice'] == "00011":
                if campo['valore']:
                    if not self._decodifica_codice("getQualifiche", campo['valore']):
                        self._addErr("Campo %s: codice qualifica errato" %(campo['codice']))
                else:
                    self._addErr("Campo %s: codice qualifica obbligatorio" %(campo['codice']))
            if campo['codice'] == "00012":
                if campo['valore']:
                    if not self._decodifica_codice("getLivelli", campo['valore']):
                        self._addErr("Campo %s: codice livello errato" %(campo['codice']))
                else:
                    self._addErr("Campo %s: codice livello obbligatorio" %(campo['codice']))
            if campo['codice'] == "00013":
                if campo['valore']:
                    data_in = formatData(campo['valore'], "int", "%Y%m%d")
                    if not isDataOK(data_in):
                        self._addErr("Campo %s: data ingresso errata" %(campo['codice']))
                else:
                    self._addErr("Campo %s: data ingresso obbligatoria" %(campo['codice']))
            if campo['codice'] == "00014":
                if campo['valore']:
                    data_out = formatData(campo['valore'], "int", "%Y%m%d")
                    if not isDataOK(data_out):
                        self._addErr("Campo %s: data uscita errata" %(campo['codice']))
                    if data_in > data_out:
                        self._addErr("Campo %s: data uscita > data ingresso" %(campo['codice']))
            if campo['codice'] == "00015":
                if not campo['valore']:
                    self._addErr("Campo %s: agenzia obbligatoria" %(campo['codice']))

        if [m for m in self.risposta.messaggi if m['tipo'] == 'errore']:
            return False
        else:
            return True


    def _carica_lista_da_args(self, somm_dist=False):
        dati = self.ga("dati")
        if not dati:
            return []
        r = ET.fromstring(dati)
        lista = []
        if not r.findall("elem"):
            return lista
        for c in r.findall("elem"):
            codice = c.find("codice_campo")
            valore = c.find("valore")
            if codice == None or valore == None:
                continue
            codice = codice.text
            formato = ""
            if somm_dist:
                if codice in ['00002', '00005', '00007', '00008', '00010', '00011', '00012']:
                    formato = "TU"
            else:
                formato = self._q(CampoWA.formato).filter(CampoWA.codice == codice).first()[0]
            if valore.text:
                valore = valore.text
                if formato == "TU":
                    valore = valore.upper()
            else:
                valore = ""
            lista.append({"codice":codice, "valore":valore})
        return lista


    def _decodifica_codice(self, formula, codice):
        if not codice:
            return ""
        # Per i comuni fa la query diretta per velocizzare
        if formula.startswith("getComuni"):
            return self._q(Comune.descrizione).filter(Comune.codice == codice).first()[0]

        liste = GestioneListe(self)
        if formula in ["getLivelli", "getQualifiche"]:
            dipte = None
            if self._dipte_id and self.archivio != "s":
                dipte = self._dipti[0]
            lquery = getattr(liste, formula)(ditta=self._ditta, dipte=dipte, periodo=self.periodo)\
                    .filter("codice = '%s'" %(codice)).first()
        else:
            lquery = getattr(liste, formula)().filter("codice = '%s'" %(codice)).first()
        if lquery:
            return unicode(lquery.descrizione)
        else:
            return ""


    def _cf_comune(self, codice):
        try:
            return self._q(Comune.cod_fiscale).filter(Comune.codice == codice).first()[0]
        except:
            return None


    def _codice_comune(self, cf):
        comune = self._q(Comune).filter(Comune.cod_fiscale == cf).first()
        try:
            return self._q(Comune.codice).filter(Comune.cod_fiscale == cf).first()[0]
        except:
            return None


    def _duplica_campi(self, da, a):
        for campo in da.__table__.columns:
            if campo.name == "id" or campo.name == "data_modifica" or campo.name == "utente_modifica":
                continue
            setattr(a, campo.name, getattr(da, campo.name))
        return a


    def _campi_abilitati(self):
        if self.archivio != "a" or self.ga("codice_campo") != "01230":
            return
        campi = {}
        for campo in ['01235', '01236', '01245', '01260']:
            campi[campo] = str(self._campo_abilitato(self.ga("valore").upper(), campo)).lower()
        self.risposta.contenuto = json.dumps(campi)



    """
        G E S T I O N E  -  O R A R I O
    """
    def _modifica_orario(self):
        try:
            orario = self._dipti[0].div_orari_rel[0]
        except:
            orario = None
        root = ET.Element("orari")
        gsett = ['Lunedi', 'Martedi', 'Mercoledi', 'Giovedi', 'Venerdi', 'Sabato', 'Domenica']
        csett = 1
        if orario:
            nsett = len(orario.settimane_rel)
            for sett in orario.settimane_rel:
                tag = ET.SubElement(root, "orario")
                ET.SubElement(tag, "giorni").text = "%da Settimana" %(csett)
                ET.SubElement(tag, "customStyle").text = "listOrarioSettimana"
                ET.SubElement(tag,"enabled").text = "false"
                for giorno in sett.giorni_rel:
                    tag = ET.SubElement(root, "orario")
                    ET.SubElement(tag, "giorni").text = "%s %s" % (giorno.giorno, gsett[giorno.giorno-1])
                    if giorno.ore_teoriche == 50:
                        ET.SubElement(tag, "ore_lav").text = "RS"
                    elif giorno.ore_teoriche == 60:
                        ET.SubElement(tag, "ore_lav").text = "--"
                    else:
                        ET.SubElement(tag, "ore_lav").text = self._nn(giorno.ore_teoriche)
                    ET.SubElement(tag, "ore_retr").text = self._nn(giorno.ore_retribuite)
                    if giorno.is_fest_ng:
                        ET.SubElement(tag, "fng").text = str(self._nn(giorno.is_fest_ng)).lower()
                    ET.SubElement(tag, "cod_cau_1").text = self._nn(giorno.cod_causale_1)
                    ET.SubElement(tag, "ore_cau_1").text = self._nn(giorno.ore_causale_1)
                    ET.SubElement(tag, "cod_cau_2").text = self._nn(giorno.cod_causale_2)
                    ET.SubElement(tag, "ore_cau_2").text = self._nn(giorno.ore_causale_2)
                    ET.SubElement(tag, "enabled").text = "true"
                    ET.SubElement(tag, "settimana").text = str(csett)
                csett += 1
        else:
            nsett = 0
        for sett in xrange(nsett, 4):
            tag = ET.SubElement(root, "orario")
            ET.SubElement(tag, "giorni").text = "%da Settimana" %(csett)
            ET.SubElement(tag, "customStyle").text = "listOrarioSettimana"
            ET.SubElement(tag, "enabled").text = "false"
            for giorno in xrange(1, 8):
                tag = ET.SubElement(root, "orario")
                ET.SubElement(tag, "giorni").text = "%s %s" % (giorno, gsett[giorno-1])
                ET.SubElement(tag, "ore_lav").text = ""
                ET.SubElement(tag, "ore_retr").text = ""
                ET.SubElement(tag, "cod_cau_1").text = ""
                ET.SubElement(tag, "ore_cau_1").text = ""
                ET.SubElement(tag, "cod_cau_2").text = ""
                ET.SubElement(tag, "ore_cau_2").text = ""
                ET.SubElement(tag, "enabled").text = "false"
                ET.SubElement(tag, "settimana").text = str(csett)
            csett += 1

        tag = ET.SubElement(root, "totali")
        if orario:
            ET.SubElement(tag, "ore_sett_ft").text = self._nn(orario.ore_sett_ft)
            ET.SubElement(tag, "tot_ore_lav").text = self._nn(orario.tot_ore_teo)
            ET.SubElement(tag, "tot_ore_retr").text = self._nn(orario.tot_ore_retr)
            ET.SubElement(tag, "tot_gg_lav").text = self._nn(orario.tot_giorni_teo)
            ET.SubElement(tag, "tot_gg_retr").text = self._nn(orario.tot_giorni_retr)
        else:
            ET.SubElement(tag, "ore_sett_ft").text = ""
            ET.SubElement(tag, "tot_ore_lav").text = ""
            ET.SubElement(tag, "tot_ore_retr").text = ""
            ET.SubElement(tag, "tot_gg_lav").text = ""
            ET.SubElement(tag, "tot_gg_retr").text = ""

        self.risposta.contenuto = ET.tostring(root)
        self._setDocType()
        self._repHead ("Content-Type","text/xml")


    def _salva_orario(self):
        lista = self._carica_orario_da_args()
        if not self._controlla_orario(lista):
            return
        orario = None
        if self._dipte_id and not self.nuovo_codice:
            # modifica
            dipte = self._dipti[0]
        else:
            # copia/inserimento: interroga con il codice, perche' il nuovo "id" dipendente (salvato nella chiamata
            # precedente tramite "_salva_dipendente") non e' ancora disponibile nel Post
            dipte = self._q(Dipendente).filter(and_(Dipendente.ditta_id == self._ditta.id,
                                                    Dipendente.codice == self.nuovo_codice,
                                                    Dipendente.is_collab == (True if self.archivio == "c" else False),
                                                    )).first()

        if dipte.div_orari_rel:
            orario = dipte.div_orari_rel[0]
            for s in orario.settimane_rel:
                self.sessiondb.delete(s)
        else:
            orario = DipendenteOrario()
            dipte.div_orari_rel.append(orario)
        self.sessiondb.commit()

        if lista.has_key("ore_sett_ft"):
            orario.ore_sett_ft = lista['ore_sett_ft']
        else:
            orario.ore_sett_ft = None
        if lista.has_key("tot_ore_lav"):
            orario.tot_ore_teo = lista['tot_ore_lav']
        else:
            orario.tot_ore_teo = None
        if lista.has_key("tot_ore_retr"):
            orario.tot_ore_retr = lista['tot_ore_retr']
        else:
            orario.tot_ore_retr = None
        if lista.has_key("tot_gg_lav"):
            orario.tot_giorni_teo = lista['tot_gg_lav']
        else:
            orario.tot_giorni_teo = None
        if lista.has_key("tot_gg_retr"):
            orario.tot_giorni_retr = lista['tot_gg_retr']
        else:
            orario.tot_giorni_retr = None

        for sett_att in range(1, lista['num_sett'] + 1):
            sett = DipendenteOrarioSett()
            sett.orario_id = orario.id
            sett.num_sett = sett_att
            orario.settimane_rel.append(sett)
            self.sessiondb.commit()
            for gg_att in range(1, 8):
                gg = DipendenteOrarioSettGG()
                gg.orario_sett_id = sett.id
                gg.giorno = gg_att
                if lista['orario'][sett_att][gg_att]['ore_lav'] == "RS":
                    gg.ore_teoriche = float(50)
                elif lista['orario'][sett_att][gg_att]['ore_lav'] == "--":
                    gg.ore_teoriche = float(60)
                else:
                    gg.ore_teoriche = float(lista['orario'][sett_att][gg_att]['ore_lav'])
                gg.ore_retribuite = lista['orario'][sett_att][gg_att]['ore_retr']
                gg.is_fest_ng = lista['orario'][sett_att][gg_att]['fng']
                gg.cod_causale_1 = lista['orario'][sett_att][gg_att]['cod_cau_1']
                gg.ore_causale_1 = lista['orario'][sett_att][gg_att]['ore_cau_1']
                gg.cod_causale_2 = lista['orario'][sett_att][gg_att]['cod_cau_2']
                gg.ore_causale_2 = lista['orario'][sett_att][gg_att]['ore_cau_2']
                sett.giorni_rel.append(gg)
                self.sessiondb.commit()

        self.sessiondb.commit()
        #self._modifica_orario()


    def _carica_orario_da_args(self):
        dati = self.ga("dati")
        if not dati:
            return []
        r = ET.fromstring(dati)
        lista = {}
        if not r.findall("elem"):
            return lista
        lista['num_sett'] = int(self.ga("numero_settimane"))
        lista['sett_att'] = int(self.ga("settiama_attuale"))
        lista['ore_sett_ft'] = self.ga("ore_sett_ft")
        lista['tot_ore_lav'] = self.ga("tot_ore_lav")
        lista['tot_ore_retr'] = self.ga("tot_ore_retr")
        lista['tot_gg_lav'] = self.ga("tot_gg_lav")
        lista['tot_gg_retr'] = self.ga("tot_gg_retr")
        lista['orario'] = {}
        for t1 in r.findall("elem"):
            try:
                if t1.find("giorni").text[1:] == "a Settimana":
                    sett_att = int(t1.find("giorni").text[0:1])
                    lista['orario'][sett_att] = {}
                    continue
                else:
                    for t2 in t1.getchildren():
                        if t2.tag == "giorni":
                            gg_att = int(t2.text[0:1])
                            lista['orario'][sett_att][gg_att] = {}
                            continue
                        else:
                            if t2.tag == "ore_lav":
                                lista['orario'][sett_att][gg_att]['ore_lav'] = t2.text.upper() if t2.text else None
                            elif t2.tag == "ore_retr":
                                lista['orario'][sett_att][gg_att]['ore_retr'] = float(t2.text) if t2.text else None
                            elif t2.tag == "fng":
                                lista['orario'][sett_att][gg_att]['fng'] = t2.text.capitalize() if t2.text else None
                            elif t2.tag == "cod_cau_1":
                                lista['orario'][sett_att][gg_att]['cod_cau_1'] = t2.text.upper() if t2.text else None
                            elif t2.tag == "ore_cau_1":
                                lista['orario'][sett_att][gg_att]['ore_cau_1'] = float(t2.text) if t2.text else None
                            elif t2.tag == "cod_cau_2":
                                lista['orario'][sett_att][gg_att]['cod_cau_2'] = t2.text.upper() if t2.text else None
                            elif t2.tag == "ore_cau_2":
                                lista['orario'][sett_att][gg_att]['ore_cau_2'] = float(t2.text) if t2.text else None
            except:
                break
        return lista


    def _controlla_orario(self, lista):
        if lista['num_sett'] == 0:
            return True
        ore_sett_ft = lista['ore_sett_ft']
        tot_ore_lav = lista['tot_ore_lav']
        tot_ore_retr =lista['tot_ore_retr']
        tot_gg_lav =  lista['tot_gg_lav']
        tot_gg_retr = lista['tot_gg_retr']
        for sett_att in range(1, lista['num_sett'] + 1):
            if not lista['orario'].has_key(sett_att):
                continue
            for gg_att in range(1, 8):
                if not lista['orario'][sett_att].has_key(gg_att):
                    continue
                ore_lav = lista['orario'][sett_att][gg_att]['ore_lav']
                ore_retr = lista['orario'][sett_att][gg_att]['ore_retr']
                cod_cau_1 = lista['orario'][sett_att][gg_att]['cod_cau_1']
                ore_cau_1 = lista['orario'][sett_att][gg_att]['ore_cau_1']
                cod_cau_2 = lista['orario'][sett_att][gg_att]['cod_cau_2']
                ore_cau_2 = lista['orario'][sett_att][gg_att]['ore_cau_2']
                # ore lavorate
                if ore_lav != "--" and ore_lav != "RS":
                    try:
                        ore_lav = float(ore_lav)
                        if not ore_lav or ore_lav > 12:
                            ore_lav = None
                    except:
                        ore_lav = None
                if not ore_lav:
                    self._addErr("Settimana '%s', giorno '%s': valore ore lavorate errato. Ammessi solo valori compresi tra >0 e 12, 'RS' o '--'"
                                 %(sett_att, gg_att))
                    break
                # ore retribuite
                if ore_lav != "--" and ore_lav != "RS" and (not ore_retr or ore_retr > 12):
                    self._addErr("Settimana '%s', giorno '%s': valore ore retribuite errato. Ammessi solo valori compresi tra >0 e 12"
                                 %(sett_att, gg_att))
                    break
                # codice causale 1
                if cod_cau_1:
                    if ore_lav == "--" or ore_lav == "RS":
                        self._addErr("Settimana '%s', giorno '%s': causale(1) non prevista in assenza di ore lavorate"
                                     %(sett_att, gg_att))
                        break
                    else:
                        if cod_cau_1 == cod_cau_2:
                            self._addErr("Settimana '%s', giorno '%s': causale(1) doppia nel giorno"
                                         %(sett_att, gg_att))
                            break
                        else:
                            causale = self._q(Causale).filter(and_(Causale.codice ==cod_cau_1,
                                                                   Causale.periodo_inizio <= self.periodo,
                                                                   Causale.periodo_fine >= self.periodo,
                                                                   )).first()
                            if not causale:
                                self._addErr("Settimana '%s', giorno '%s': codice causale(1) errato"
                                             %(sett_att, gg_att))
                                break
                else:
                    if ore_cau_1:
                        self._addErr("Settimana '%s', giorno '%s': inserire un codice causale(1)"
                                     %(sett_att, gg_att))
                        break
                # codice causale 2
                if cod_cau_2:
                    if ore_lav == "--" or ore_lav == "RS":
                        self._addErr("Settimana '%s', giorno '%s': causale(2) non prevista in assenza di ore lavorate"
                                     %(sett_att, gg_att))
                        break
                    else:
                        if cod_cau_2 == cod_cau_1:
                            self._addErr("Settimana '%s', giorno '%s': causale(2) doppia nel giorno"
                                         %(sett_att, gg_att))
                            break
                        else:
                            causale = self._q(Causale).filter(and_(Causale.codice ==cod_cau_2,
                                                                   Causale.periodo_inizio <= self.periodo,
                                                                   Causale.periodo_fine >= self.periodo,
                                                                   )).first()
                            if not causale:
                                self._addErr("Settimana '%s', giorno '%s': codice causale(2) errato"
                                             %(sett_att, gg_att))
                                break
                else:
                    if ore_cau_2:
                        self._addErr("Settimana '%s', giorno '%s': inserire un codice causale(2)"
                                     %(sett_att, gg_att))
                        break

        if [m for m in self.risposta.messaggi if m['tipo'] == 'errore']:
            return False

        # ore settimana FT
        if ore_sett_ft and ore_sett_ft <= tot_ore_lav:
            self._addErr("Ore settimana tempo pieno <= totale ore lavorate")
            return False

        # totale ore lavorate
        if not tot_ore_lav:
            self._addErr("Totale ore lavorate: dato mancante")
            return False

        # totale ore retribuite
        if not tot_ore_retr:
            self._addErr("Totale ore retribuite: dato mancante")
            return False

        # totale giorni lavorati
        if not tot_gg_lav:
            self._addErr("Totale giorni lavorati: dato mancante")
            return False

        # totale giorni retribuiti
        if not tot_gg_retr:
            self._addErr("Totale giorni retribuiti: dato mancante")
            return False

        return True


    """
        F U N Z I O N I  -  C A M P I (campo_rif)
    """
    def formatData(self, data):
        return formatData(data)

def dogentask(sessiondb, session, request):
    return GestDipti(sessiondb,session,request).doTask()

Main = GestDipti

