# -*- coding: utf-8 -*-
from sqlalchemy.sql                     import and_

from waonline.core.rbase                import RBase
from .tabelle.ditte                     import DittaWAStatusPeriodo, Ditta
from waonline.apps.presenze.tabelle.presenze          import Presenze
from waonline.core.wadecorators     import WaAuth
from waonline.utils.objutils            import toJSON
from waonline.utils.objutils            import getObjInfo
from waonline.core.ete_dumbobj          import EteDumbObj

class GestDitte(RBase):
    """
        Classe per la gestione delle ditte
    """
    def lista_ditte(self):
        info = []
        if self.user.livello in [self.user.UAZIENDA, self.user.USTUDIO]:
            ditte = self._q(Ditta, Ditta.codice.in_(
                self.user.codici_ditte)).order_by("codice").all()
            try:
                info = getObjInfo(ditte, includi=['id', 'codice:cod_ditta'],
                        custom={'uid': lambda x: \
                                self.user.ditta_by_codice(x.codice).id,
                            'descrizione': lambda x:\
                                    x.codice + "-" + x.descrizione})
            except Exception, err:
                self.errore(err)
                info = []
        self.risposta.contenuto = toJSON(info)

    @WaAuth()
    def periodi_ditta(self, stato=None, inviato=None):
        """
            Lista i periodi per la ditta speficicata anche in base allo stato
            o se sono stati inviati o meno.
        """
        if not self._cod_ditta:
            self._addErr("Non ho il codice della ditta!!!")
            self.risposta.contenuto = toJSON([])
            return
        descrizione_stati = {"A": "Aperto", "C": "Chiuso"}
        lista_periodi = []
        if not self._ditta.status_wa_rel:
            display = '<span class="listPeriodi"><span class="testo">'\
                'Nessun periodo aperto</span></span>'
            lista_periodi.append({'display': display})
        else:
            periodi = self._ditta.status_wa_rel[0].status_periodi_rel[::-1]
            for periodo_rel in periodi:
                if stato != None and periodo_rel.stato != stato:
                        continue
                if inviato != None and periodo_rel.is_inviato != inviato:
                        continue
                periodo = EteDumbObj()
                periodo.periodo = str(periodo_rel.periodo)[4:6] + '/' + \
                        str(periodo_rel.periodo)[0:4]
                periodo.stato = descrizione_stati[periodo_rel.stato]
                display = '<span class="listPeriodi"><span class="periodo">'\
                        '%s</span> <span class="testo">' % periodo.periodo
                if periodo_rel.stato == "A":
                    display += '<span class="aperto">aperto</span> '
                else:
                    display += '<span class="chiuso">chiuso</span> '
                if periodo_rel.is_inviato:
                    periodo.inviato = 'S'
                    display += '<span class="inviato">inviato</span>'
                else:
                    periodo.inviato = 'N'
                    display += '<span class="non_inviato">non inviato</span>'
                display += "</span></span>"
                periodo.display = display
                lista_periodi.append(periodo)
        self.risposta.contenuto = toJSON(lista_periodi)

    @WaAuth(['cod_ditta'])
    def elimina_periodo(self, periodo_id):
        """
            Elimina il periodo per la ditta specificata

            :param period_id: id del periodo da eliminare.
        """
        self._add_to_log()
        periodo = self._q(DittaWAStatusPeriodo).filter(DittaWAStatusPeriodo.id == periodo_id).first()
        ditta = periodo.status_rel.ditta_rel
        self._q(Presenze).filter(and_(Presenze.periodo == periodo.periodo, Presenze.ditta_id == ditta.id)).delete("evaluate")
        self.sessiondb.delete(periodo)
        self.sessiondb.commit()

    @WaAuth(['cod_ditta'])
    def status_ditta(self):
        """
            Recupera lo status per la ditta
        """
        descrizione_stati = {"A": "Aperto", "C": "Chiuso"}
        lista_periodi = []
        if self._ditta:
            self._ditta.status_wa_rel[0].status_periodi_rel.reverse()
            for periodo_rel in self._ditta.status_wa_rel[0].status_periodi_rel:
                periodo_status = str(periodo_rel.periodo)
                lista_periodi.append({'id_periodo': str(periodo_rel.id),
                        'periodoStatus': periodo_status,
                        'statoStatus': descrizione_stati[periodo_rel.stato],
                        'inviatoStatus': 'S' if periodo_rel.is_inviato else 'N'})
        self.risposta.contenuto = toJSON(lista_periodi)
