# -*- coding: utf-8 -*-
from waonline.core.tabelle import *

class Dipendente(Base):
    """
        Classe per la tabella Dipendenti
    """
    __tablename__ = "dipendenti"
    __table_args__ = (UniqueConstraint('ditta_id', 'codice', 'is_collab'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='SET NULL'), nullable=False)
    codice = Column(String(5), nullable=False)
    is_collab = Column(Boolean, nullable=False)
    matricola = Column(String(5), nullable=False)
    cognome = Column(String(50), nullable=False)
    nome = Column(String(50), nullable=False)
    sesso = Column(String(1))
    data_nascita = Column(Integer)
    comune_nascita = Column(String(5))
    cod_fiscale = Column(String(16))
    is_resid_estero = Column(Boolean)
    indirizzo_resid = Column(String(100))
    comune_resid = Column(String(5))
    periodo_cambio_resid = Column(Integer)
    telefono = Column(String(50))
    fax = Column(String(50))
    mail = Column(String(50))
    cittadinanza = Column(String(5))
    is_extra_comunit = Column(Boolean)
    stato_civile = Column(String(1))
    titolo_studio = Column(String(1))
    rif_utente = Column(String(100))
    is_socio_coop = Column(Boolean)
    is_regol_ex_dl195 = Column(Boolean)
    is_primo_rapp_italia = Column(Boolean)
    invalidita = Column(String(2))
    posizione = Column(String(1))
    data_ass = Column(Integer)
    data_cess = Column(Integer)
    data_cess_fut = Column(Integer)
    causa_cess = Column(String(2))
    mesi_collabora = Column(Integer)
    compenso_annuo = Column(Numeric(precision=12, scale=2))
    attivita = Column(String(2))
    forme_assicura = Column(String(3))
    rapporto_coll = Column(String(2))
    certif_emens = Column(String(3))
    deduzione = Column(String(1))
    detrazione = Column(String(1))
    is_conguaglio = Column(Boolean)
    dichiarazione = Column(String(1))
    imp_redd_diversi = Column(Numeric(precision=12, scale=2))
    reddito_anf = Column(Numeric(precision=12, scale=2))
    num_componenti = Column(Integer)
    anf = Column(String(3))
    num_figli_minori = Column(Integer)
    num_figli_inabili = Column(Integer)
    imp_forzatura = Column(Numeric(precision=12, scale=2))
    is_carico_ditta = Column(Boolean)
    is_ricov_osped = Column(Boolean)
    banca = Column(Integer)
    cod_paese = Column(String(2))
    cin_eu = Column(String(2))
    cin = Column(String(1))
    cod_abi = Column(String(5))
    cod_cab = Column(String(5))
    des_banca = Column(String(100))
    cod_cc = Column(String(15))
    banca_ditta = Column(String(3))
    alt_divisori = Column(String(4))
    alt_ferie = Column(String(4))
    alt_infortunio = Column(String(4))
    alt_istituti = Column(String(4))
    alt_malattia = Column(String(4))
    alt_maternita = Column(String(4))
    alt_mensilita = Column(String(4))
    alt_straord = Column(String(4))
    alt_valorizza = Column(String(4))
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')
    # Relazioni one-to-many
    detraz_fiscali_rel = relationship('DipendenteCarFam', order_by='DipendenteCarFam.id')
    festivita_rel = relationship('DipendenteFest', order_by='DipendenteFest.id')
    div_orari_rel = relationship('DipendenteOrario', order_by='DipendenteOrario.id')
    qualifiche_rel = relationship('DipendenteQual', order_by='DipendenteQual.id')
    rapp_precedenti_rel = relationship('DipendenteRappPrec', order_by='DipendenteRappPrec.id')
    stra_magg_rel = relationship('DipendenteStraMagg', order_by='DipendenteStraMagg.id')

    __qualifica__ = None

    @property
    def nominativo(self):
        return " ".join([self.cognome, self.nome])

    @property
    def qualifica(self):
        if self.__qualifica__ == None and self.qualifiche_rel:
            self.__qualifica__ = [qualifica for qualifica in self.qualifiche_rel if qualifica.is_qual_att][0]
        return self.__qualifica__


class DipendenteCarFam(Base):
    __tablename__ = "dipendenti_car_fam"
    id = Column(Integer, primary_key=True, autoincrement=True)
    dipendente_id = Column(Integer, ForeignKey('dipendenti.id', onupdate='CASCADE', ondelete='CASCADE'))
    parentela = Column(String(1))
    is_anf = Column(Boolean)
    tipo_detraz = Column(String(1))
    perc_detraz = Column(Numeric(precision=5, scale=2))
    is_inabile = Column(Boolean)
    cod_fiscale = Column(String(16))
    cognome = Column(String(50))
    nome = Column(String(50))
    sesso = Column(String(1))
    data_nascita = Column(Integer)
    comune_nascita = Column(String(5))
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    dipendente_rel = relationship('Dipendente', order_by='Dipendente.id')


class DipendenteFest(Base):
    __tablename__ = "dipendenti_fest"
    __table_args__ = (UniqueConstraint('dipendente_id'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    dipendente_id = Column(Integer, ForeignKey('dipendenti.id', onupdate='CASCADE', ondelete='CASCADE'))
    is_ignora = Column(Boolean)
    # Relazioni many-to-one
    dipendente_rel = relationship('Dipendente', order_by='Dipendente.id')
    # Relazioni one-to-many
    festivita_rel = relationship('DipendenteFestTab', order_by='DipendenteFestTab.id')


class DipendenteFestTab(Base):
    __tablename__ = "dipendenti_fest_tab"
    __table_args__ = (UniqueConstraint('festivita_id', 'aaaa', 'mmgg'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    festivita_id = Column(Integer, ForeignKey('dipendenti_fest.id', onupdate='CASCADE', ondelete='CASCADE'))
    aaaa = Column(String(4))
    mmgg = Column(String(4))
    is_ignora = Column(Boolean)
    descrizione = Column(String(100))
    tipo = Column(String(1))
    # Relazioni many-to-one
    dipteFest_rel = relationship('DipendenteFest', order_by='DipendenteFest.id')


class DipendenteOrario(Base):
    __tablename__ = "dipendenti_orari"
    __table_args__ = (UniqueConstraint('dipendente_id'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    dipendente_id = Column(Integer, ForeignKey('dipendenti.id', onupdate='CASCADE', ondelete='CASCADE'))
    div_orario = Column(Numeric(precision=5, scale=2))
    div_giorno = Column(Numeric(precision=5, scale=2))
    div_straord = Column(Numeric(precision=5, scale=2))
    div_assenze = Column(Numeric(precision=5, scale=2))
    cod_assenze = Column(String(3))
    rif_div_var = Column(String(5))
    tot_ore_teo = Column(Numeric(precision=4, scale=2))
    tot_ore_retr = Column(Numeric(precision=4, scale=2))
    tot_giorni_teo = Column(Numeric(precision=4, scale=2))
    tot_giorni_retr = Column(Numeric(precision=4, scale=2))
    ore_sett_ft = Column(Numeric(precision=4, scale=2))
    orario_part = Column(String(1))
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    dipendente_rel = relationship('Dipendente', order_by='Dipendente.id')
    # Relazioni one-to-many
    settimane_rel = relationship('DipendenteOrarioSett', order_by='DipendenteOrarioSett.num_sett')


class DipendenteOrarioSett(Base):
    __tablename__ = "dipendenti_orari_sett"
    __table_args__ = (UniqueConstraint('orario_id', 'num_sett'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    orario_id = Column(Integer, ForeignKey('dipendenti_orari.id', onupdate='CASCADE', ondelete='CASCADE'))
    num_sett = Column(Integer)
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    div_orari_rel = relationship('DipendenteOrario', order_by='DipendenteOrario.id')
    # Relazioni one-to-many
    giorni_rel = relationship('DipendenteOrarioSettGG', order_by='DipendenteOrarioSettGG.giorno')


class DipendenteOrarioSettGG(Base):
    __tablename__ = "dipendenti_orari_sett_gg"
    id = Column(Integer, primary_key=True, autoincrement=True)
    orario_sett_id = Column(Integer, ForeignKey('dipendenti_orari_sett.id', onupdate='CASCADE', ondelete='CASCADE'))
    giorno = Column(Integer)
    ore_teoriche = Column(Numeric(precision=4, scale=2))
    ore_retribuite = Column(Numeric(precision=4, scale=2))
    is_fest_ng = Column(Boolean)
    cod_causale_1 = Column(String(4))
    ore_causale_1 = Column(Numeric(precision=4, scale=2))
    cod_causale_2 = Column(String(4))
    ore_causale_2 = Column(Numeric(precision=4, scale=2))
    cod_causale_3 = Column(String(4))
    ore_causale_3 = Column(Numeric(precision=4, scale=2))
    cod_causale_4 = Column(String(4))
    ore_causale_4 = Column(Numeric(precision=4, scale=2))
    cod_causale_5 = Column(String(4))
    ore_causale_5 = Column(Numeric(precision=4, scale=2))
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    div_orari_sett_rel = relationship('DipendenteOrarioSett', order_by='DipendenteOrarioSett.num_sett')


class DipendenteQual(Base):
    __tablename__ = "dipendenti_qual"
    id = Column(Integer, primary_key=True, autoincrement=True)
    dipendente_id = Column(Integer, ForeignKey('dipendenti.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(4))
    is_qual_att = Column(Boolean)
    data_inizio = Column(Integer)
    part_time = Column(String(1))
    perc_part_time = Column(Numeric(precision=5, scale=2))
    livello = Column(String(4))
    mansione = Column(String(100))
    rapporto = Column(String(2))
    apprendistato = Column(String(4))
    tipo_contr_deter = Column(String(1))
    mesi_contr_deter = Column(Integer)
    cantiere = Column(String(3))
    prec_periodo = Column(Integer)
    rif_inail = Column(String(4))
    num_pos_inail = Column(String(8))
    cc_pos_inail = Column(String(2))
    contratto_inps = Column(String(4))
    contratto = Column(String(3))
    sudd_contr = Column(String(2))
    sudd_terr = Column(String(2))
    aliquota = Column(String(3))
    espo_paga_cedo = Column(String(1))
    gest_retrib_mov = Column(String(1))
    centro_costo = Column(String(12))
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    dipendente_rel = relationship('Dipendente', order_by='Dipendente.id')


class DipendenteRappPrec(Base):
    __tablename__ = "dipendenti_rapp_prec"
    __table_args__ = (UniqueConstraint('dipendente_id', 'tipo', 'periodo'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    dipendente_id = Column(Integer, ForeignKey('dipendenti.id', onupdate='CASCADE', ondelete='CASCADE'))
    tipo = Column(Integer)
    periodo = Column(Integer)
    ditta = Column(String(7))
    codice = Column(String(5))
    causa_conguaglio = Column(String(1))
    tipo_conguaglio = Column(String(1))
    data_inizio = Column(Integer)
    data_fine = Column(Integer)
    # Relazioni many-to-one
    dipendente_rel = relationship('Dipendente', order_by='Dipendente.id')


class DipendenteStraMagg(Base):
    __tablename__ = "dipendenti_stra_magg"
    __table_args__ = (UniqueConstraint('dipendente_id', 'cod_straord'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    dipendente_id = Column(Integer, ForeignKey('dipendenti.id', onupdate='CASCADE', ondelete='CASCADE'))
    cod_straord = Column(String(2))
    # Relazioni many-to-one
    dipendente_rel = relationship('Dipendente', order_by='Dipendente.id')

