from waonline.core.tabelle import *

class Ditta(Base):
    __tablename__ = "ditte"
    __table_args__ = (UniqueConstraint('codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(7))
    tipo_ditta = Column(String(1))
    descrizione = Column(String(100))
    cognome = Column(String(50))
    nome = Column(String(50))
    sesso = Column(String(1))
    data_nascita = Column(Integer)
    comune_nascita = Column(String(5))
    cod_fiscale = Column(String(16))
    partita_iva = Column(String(11))
    is_sede = Column(Boolean)
    indirizzo_att = Column(String(100))
    comune_att = Column(String(5))
    telefono_att = Column(String(50))
    descriz_att = Column(String(100))
    indirizzo_legale = Column(String(100))
    comune_legale = Column(String(5))
    accentr_1 = Column(String(7))
    tipo_accentr_1 = Column(String(1))
    accentr_2 = Column(String(7))
    tipo_accentr_2 = Column(String(1))
    fax = Column(String(50))
    mail = Column(String(50))
    contratto = Column(String(3))
    sudd_contr = Column(String(2))
    sudd_terr = Column(String(2))
    aliquota = Column(String(3))
    pos_inps = Column(String(10))
    zona_inps = Column(String(2))
    filtro_caus_wa = Column(String(1))
    filtro_voci_wa = Column(String(1))
    filtro_campi_wa = Column(String(1))
    # Relazioni one-to-many
    abilit_campi_wa_rel = relationship('DittaWACampi', order_by='DittaWACampi.id')
    abilit_caus_rel = relationship('DittaWACausali', order_by='DittaWACausali.id')
    abilit_voci_rel = relationship('DittaWAVoci', order_by='DittaWAVoci.id')
    bache_rel = relationship('DittaBanca', order_by='DittaBanca.id')
    cantieri_rel = relationship('DittaCantiere', order_by='DittaCantiere.id')
    centri_costo_rel = relationship('DittaCentroCosto', order_by='DittaCentroCosto.id')
    div_orari_rel = relationship('DittaOrario', order_by='DittaOrario.id')
    festivita_rel = relationship('DittaFest', order_by='DittaFest.id')
    pos_inail_rel = relationship('DittaPosInail', order_by='DittaPosInail.id')
    somm_dist_rel = relationship('DittaSommDist', order_by='DittaSommDist.id')
    status_wa_rel = relationship('DittaWAStatus', order_by='DittaWAStatus.id')
    stra_magg_rel = relationship('DittaStraMagg', order_by='DittaStraMagg.id')


class DittaBanca(Base):
    __tablename__ = "ditte_banche"
    __table_args__ = (UniqueConstraint('ditta_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(3))
    descrizione = Column(String(100))
    agenzia = Column(String(100))
    cod_paese = Column(String(2))
    cin_eu = Column(String(2))
    cin = Column(String(1))
    cod_abi = Column(String(5))
    cod_cab = Column(String(5))
    cod_cc = Column(String(15))
    tipo_ope = Column(String(1))
    cod_sia = Column(String(5))
    rif_banca = Column(Integer)
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')


class DittaCantiere(Base):
    __tablename__ = "ditte_cantieri"
    __table_args__ = (UniqueConstraint('ditta_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(3))
    codice_ce = Column(String(10))
    cassa_edile = Column(String(4))
    descrizione = Column(String(100))
    cap = Column(String(5))
    cod_consul = Column(String(5))
    cod_impresa = Column(String(8))
    situazione = Column(String(1))
    gg_situazione = Column(Integer)
    periodo_inizio = Column(Integer)
    committente = Column(String(16))
    tipo_comm = Column(Integer)
    att_produttiva = Column(Integer)
    is_Trasfertisti = Column(Boolean)
    is_Subappalto = Column(Boolean)
    is_LavInterinale = Column(Boolean)
    indirizzo = Column(String(100))
    comune = Column(String(5))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')


class DittaCentroCosto(Base):
    __tablename__ = "ditte_centri_costo"
    __table_args__ = (UniqueConstraint('ditta_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(12))
    descrizione = Column(String(100))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')


class DittaFest(Base):
    __tablename__ = "ditte_fest"
    __table_args__ = (UniqueConstraint('ditta_id'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    is_ignora = Column(Boolean)
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')
    # Relazioni one-to-many
    festivita_rel = relationship('DittaFestTab', order_by='DittaFestTab.id')


class DittaFestTab(Base):
    __tablename__ = "ditte_fest_tab"
    __table_args__ = (UniqueConstraint('festivita_id', 'aaaa', 'mmgg'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    festivita_id = Column(Integer, ForeignKey('ditte_fest.id', onupdate='CASCADE', ondelete='CASCADE'))
    aaaa = Column(String(4))
    mmgg = Column(String(4))
    is_ignora = Column(Boolean)
    descrizione = Column(String(100))
    tipo = Column(String(1))
    # Relazioni many-to-one
    dittaFest_rel = relationship('DittaFest', order_by='DittaFest.id')


class DittaOrario(Base):
    __tablename__ = "ditte_orari"
    __table_args__ = (UniqueConstraint('ditta_id', 'qualifica'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    qualifica = Column(String(4))
    div_orario = Column(Numeric(precision=5, scale=2))
    div_giorno = Column(Numeric(precision=5, scale=2))
    div_straord = Column(Numeric(precision=5, scale=2))
    div_assenze = Column(Numeric(precision=5, scale=2))
    cod_assenze = Column(String(3))
    rif_div_var = Column(String(5))
    tot_ore_teo = Column(Numeric(precision=4, scale=2))
    tot_ore_retr = Column(Numeric(precision=4, scale=2))
    tot_giorni_teo = Column(Numeric(precision=4, scale=2))
    tot_giorni_retr = Column(Numeric(precision=4, scale=2))
    ore_sett_ft = Column(Numeric(precision=4, scale=2))
    orario_part = Column(String(1))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')
    # Relazioni one-to-many
    settimane_rel = relationship('DittaOrarioSett', order_by='DittaOrarioSett.num_sett')


class DittaOrarioSett(Base):
    __tablename__ = "ditte_orari_sett"
    __table_args__ = (UniqueConstraint('orario_id', 'num_sett'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    orario_id = Column(Integer, ForeignKey('ditte_orari.id', onupdate='CASCADE', ondelete='CASCADE'))
    num_sett = Column(Integer)
    # Relazioni many-to-one
    div_orari_rel = relationship('DittaOrario', order_by='DittaOrario.id')
    # Relazioni one-to-many
    giorni_rel = relationship('DittaOrarioSettGG', order_by='DittaOrarioSettGG.giorno')


class DittaOrarioSettGG(Base):
    __tablename__ = "ditte_orari_sett_gg"
    id = Column(Integer, primary_key=True, autoincrement=True)
    orario_sett_id = Column(Integer, ForeignKey('ditte_orari_sett.id', onupdate='CASCADE', ondelete='CASCADE'))
    giorno = Column(Integer)
    ore_teoriche = Column(Numeric(precision=4, scale=2))
    ore_retribuite = Column(Numeric(precision=4, scale=2))
    is_fest_ng = Column(Boolean)
    cod_causale_1 = Column(String(4))
    ore_causale_1 = Column(Numeric(precision=4, scale=2))
    cod_causale_2 = Column(String(4))
    ore_causale_2 = Column(Numeric(precision=4, scale=2))
    cod_causale_3 = Column(String(4))
    ore_causale_3 = Column(Numeric(precision=4, scale=2))
    cod_causale_4 = Column(String(4))
    ore_causale_4 = Column(Numeric(precision=4, scale=2))
    cod_causale_5 = Column(String(4))
    ore_causale_5 = Column(Numeric(precision=4, scale=2))
    # Relazioni many-to-one
    div_orari_sett_rel = relationship('DittaOrarioSett', order_by='DittaOrarioSett.num_sett')


class DittaPosInail(Base):
    __tablename__ = "ditte_pos_inail"
    __table_args__ = (UniqueConstraint('ditta_id', 'cod_pos', 'sub_pos'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    cod_pos = Column(Integer)
    sub_pos = Column(Integer)
    num_pos = Column(String(8))
    cc_pos = Column(String(2))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')


class DittaSommDist(Base):
    __tablename__ = "ditte_somm_dist"
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    tipo = Column(String(1), nullable=False)
    cognome = Column(String(50), nullable=False)
    nome = Column(String(50), nullable=False)
    sesso = Column(String(1), nullable=False)
    data_nascita = Column(Integer, nullable=False)
    comune_nascita = Column(String(5), nullable=False)
    cod_fiscale = Column(String(16), nullable=False)
    indirizzo_resid = Column(String(100))
    comune_resid = Column(String(5))
    qualifica = Column(String(4), nullable=False)
    livello = Column(String(4), nullable=False)
    data_ingresso = Column(Integer, nullable=False)
    data_uscita = Column(Integer)
    agenzia = Column(String(50), nullable=False)
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')
    @property
    def nominativo(self):
        return " ".join([self.cognome, self.nome])


class DittaStraMagg(Base):
    __tablename__ = "ditte_stra_magg"
    __table_args__ = (UniqueConstraint('ditta_id', 'qualifica'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    qualifica = Column(String(4))
    cod_straord = Column(String(2))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')


class DittaWACampi(Base):
    __tablename__ = "ditte_wa_campi"
    __table_args__ = (UniqueConstraint('ditta_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(5))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')


class DittaWACausali(Base):
    __tablename__ = "ditte_wa_causali"
    __table_args__ = (UniqueConstraint('ditta_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(4))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')


class DittaWAStatus(Base):
    __tablename__ = "ditte_wa_status"
    __table_args__ = (UniqueConstraint('ditta_id', 'sede_inail'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    sede_inail = Column(String(5))
    numero_inail = Column(String(8))
    data_inail = Column(Integer)
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')
    # Relazioni one-to-many
    status_periodi_rel = relationship('DittaWAStatusPeriodo', order_by='DittaWAStatusPeriodo.periodo')


class DittaWAStatusPeriodo(Base):
    __tablename__ = "ditte_wa_status_periodi"
    __table_args__ = (UniqueConstraint('status_id', 'periodo'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    status_id = Column(Integer, ForeignKey('ditte_wa_status.id', onupdate='CASCADE', ondelete='CASCADE'))
    periodo = Column(Integer)
    stato = Column(String(1))
    is_inviato = Column(Boolean)
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    status_rel = relationship('DittaWAStatus', order_by='DittaWAStatus.id')


class DittaWAVoci(Base):
    __tablename__ = "ditte_wa_voci"
    __table_args__ = (UniqueConstraint('ditta_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(4))
    # Relazioni many-to-one
    ditta_rel = relationship('Ditta', order_by='Ditta.id')
