from waonline.core.tabelle import *


class Studio(Base):
    __tablename__ = "studi"
    __table_args__ = (UniqueConstraint('codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(3))
    denominazione = Column(String(100))
    indirizzo = Column(String(100))
    comune = Column(String(5))
    partita_iva = Column(String(11))
    cod_fiscale_st = Column(String(16))
    telefono = Column(String(50))
    fax = Column(String(50))
    mail = Column(String(50))
    modem = Column(String(50))
    cognome = Column(String(50))
    nome = Column(String(50))
    sesso = Column(String(1))
    data_nascita = Column(Integer)
    comune_nascita = Column(String(5))
    cod_fiscale_cn = Column(String(16))
    albo_tipo = Column(String(1))
    albo_numero = Column(String(20))
    albo_data = Column(Integer)
    albo_inail = Column(String(10))
    # Relazioni one-to-many
    campi_pers_rel = relationship('StudioCampoPers', order_by='StudioCampoPers.id')
    campi_wa_rel = relationship('StudioCampoWA', order_by='StudioCampoWA.id')
    voce_wa_rel = relationship('StudioVoce', order_by='StudioVoce.id')

    @property
    def nominativo(self):
        return " ".join([self.cognome, self.nome])


class StudioCampoPers(Base):
    __tablename__ = "studi_campi_pers"
    __table_args__ = (UniqueConstraint('studio_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    studio_id = Column(Integer, ForeignKey('studi.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(10))
    descrizione = Column(String(100))
    archivio = Column(String(1))
    formato = Column(String(2))
    lunghezza = Column(Integer)
    # Relazioni many-to-one
    studio_rel = relationship('Studio', order_by='Studio.id')


class StudioCampoWA(Base):
    __tablename__ = "studi_campi_wa"
    __table_args__ = (UniqueConstraint('studio_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    studio_id = Column(Integer, ForeignKey('studi.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(5))
    is_obbligatorio = Column(Boolean)
    funzione = Column(String(2))
    # Relazioni many-to-one
    studio_rel = relationship('Studio', order_by='Studio.id')


class StudioVoce(Base):
    __tablename__ = "studi_voci"
    __table_args__ = (UniqueConstraint('studio_id', 'codice', 'cod_ditta'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    studio_id = Column(Integer, ForeignKey('studi.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(4))
    descrizione = Column(String(100))
    cod_ditta = Column(String(7))
    # Relazioni many-to-one
    studio_rel = relationship('Studio', order_by='Studio.id')

