# -*- coding: utf-8 -*-
""" Modulo per alcune operazioni con i files """
import os
import re
import time
import zipfile
from glob import glob

from xml.etree.cElementTree import tostring, Element, SubElement
from xml.etree.cElementTree import parse as xml_parse

from waonline.core.rbase    import RBase
from waonline.core.wadecorators       import WaAuth

from waonline import conf


PATH_INVIA = conf.DEFAULT_DATA_PATH + "/%s/invia"
PATH_RICEVI = conf.DEFAULT_DATA_PATH + "/%s/ricevi"

def estrai(zipfilepath, extractiondir):
    fzip = zipfile.ZipFile(zipfilepath)
    fzip.extractall(path=extractiondir)

def unzippatutto(cartella):
    lista = glob(cartella+"/*.zip")
    for f in lista:
        bname = os.path.basename(f)
        if not bname.startswith("_") and bname != "TabelleUtenti.zip":
            estrai(f, cartella)
    for f in lista:
        bname = os.path.basename(f)
        if not bname.startswith("_") and bname != "TabelleUtenti.zip":
            os.unlink(f)


class _MiniFileInfo(object):
    """ Classe per le info del file """

    def __init__(self, percorsoFile):
        self.percorsoFile = percorsoFile
        self.nome = os.path.basename(percorsoFile)
        self.percorso = os.path.dirname(percorsoFile)
        self._lista_dipe = None
        self._lista_ditte = None
        m = re.match(r'(\w{7})-(\d{4})(\d{2})(\d{2})-(\d{2})(\d{2})(\d{2})',
                self.nome)
        self._root = None
        if m:
            self.codice = m.group(1)
            self.anno = int(m.group(2))
            self.mese = int(m.group(3))
            self.giorno = int(m.group(4))
            self.ore = int(m.group(5))
            self.minuti = int(m.group(6))
            self.secondi = int(m.group(7))
        else:
            info = os.stat(percorsoFile)
            data = time.gmtime(info.st_ctime)
            self.codice = ''
            self.studio = ''
            self.giorno, self.mese, self.anno = (data.tm_mday, data.tm_mon,
                    data.tm_year)
            self.ore, self.minuti, self.secondi = (data.tm_hour, data.tm_min,
                    data.tm_sec)

    @property
    def root(self):
        """ ritorna il root xml del file """
        if self._root == None:
            try:
                self._root = xml_parse(self.percorsoFile).getroot()
            except:
                self._root = False
        return self._root

    @property
    def sdata(self):
        """ Ritorna la stringa formattata per la data del file """
        return "%02d/%02d/%04d" % (self.giorno, self.mese, self.anno)

    @property
    def sora(self):
        """ Ritorna la stringa formattata per l'ora del file """
        return "%02d.%02d.%02d" % (self.ore, self.minuti, self.secondi)

    def getDitte(self, lista=False):
        """ Ritorna le ditte presenti nel file """
        if self.root:
            if lista:
                if self._lista_ditte == None:
                    self._lista_ditte = [l for l in \
                            self.root.getiterator("Ditta")]
                return self._lista_ditte
            else:
                return self.root.getiterator("Ditta")
        else:
            return []

    def getDipendenti(self, lista=False):
        """ Ritorna i dipendenti presenti nel file """
        if self.root:
            if lista:
                if self._lista_dipe == None:
                    self._lista_dipe =  [l for l in \
                            self.root.getiterator("Dipendente")]
                return self._lista_dipe
            else:
                return self.root.getiterator("Dipendente")
        else:
            return []

    def getInfo(self):
        """ Ritorna le info del file """
        if self.root:
            tipi = {'S': '', 'N': '', 'T': '', 'P': ': '}
            html = '<div class="fInfo">'
            ditte = self.getDitte(True)
            if ditte:
                codici = ', '.join([l.find("codice").text for l in ditte])
                html += "<p>Ditta: %s</p>" % codici
            del ditte
            if self.root.find("info").attrib.get("somm_dist", "N") == "S":
                if self.root.findall("DittaSommDist"):
                    html += "<p>Somministrati/Distaccati</p>"
            if self.root.find("info").attrib.get("dipendenti", "N") != "N":
                dipe = [d for d in self.getDipendenti(True) \
                        if d.find("is_collab").text == "N"]
                if dipe:
                    if self.root.find("info").attrib.get("dipendenti", "N") == "P":
                        html += "<p>Dipendenti (%s)%s" % (len(dipe),
                                tipi[self.root.find("info").attrib.get("dipendenti", "N")])
                        html += "<ul>"
                        for d in dipe:
                            html += "<li>%s - %s %s</li>" % (
                                    d.find("codice").text,
                                    d.find("cognome").text,
                                    d.find("nome").text)
                        html += "</ul>"
                        html += "</p>"
                    else:
                        html += "<p>Dipendenti</p>"
                    del dipe
            if self.root.find("info").attrib.get("collaboratori", "N") != "N":
                coll = [d for d in self.getDipendenti(True)\
                        if d.find("is_collab").text == "S"]
                if coll:
                    if self.root.find("info").attrib.get("dipendenti", "N") == "P":
                        html += "<p>Collaboratori (%s)%s" % (len(coll),
                            tipi[self.root.find("info").attrib.get("collaboratori", "N")])
                        html += "<ul>"
                        for d in coll:
                            html += "<li>%s - %s %s</li>" % (
                                    d.find("codice").text,
                                    d.find("cognome").text,
                                    d.find("nome").text)
                        html += "</ul>"
                        html += "</p>"
                    else:
                        html += "<p>Collaboratori</p>"
                    del coll
            html += "</div>"
            return html
        else:
            return ''


    def getInfoWData(self):
        """ Ritorna lef del file """
        html = '<div class="dInfo">'
        html += '<p class="t2">%s - %s - %s</p>' % (self.codice, self.sdata,
                self.sora)
        html += self.getInfo()
        html += '</div>'
        return html


class GestioneInviaRicevi(RBase):
    """ Classi per la gestinoe dei file """
    listaFiles = []
    cartella = ''
    _cartella = ''

    def _post_init_(self):
        self.cartella = self.ga("cartella", "ricevi")
        if self.cartella == 'invia':
            self._cartella = self._ninvia
        else:
            self._cartella = self._nricevi

    @property
    def _ninvia(self):
        """ Ritorna la cartella "Invia" """
        cartella = ""
        if self.user.livello > self.user.USTUDIO:
            cartella = PATH_INVIA % 'admin'
        else:
            cartella = PATH_INVIA % self.user.studio_codice
        return cartella

    @property
    def _nricevi(self):
        """ Ritorna la cartella "Ricevi" """
        cartella = ""
        if self.user.livello > self.user.USTUDIO:
            cartella = PATH_RICEVI % 'admin'
        else:
            cartella = PATH_RICEVI % self.user.studio_codice
        return cartella

    def _ginvia(self, nome_file):
        """ Ritorna il percorso completo del file da Inviare """
        return self._ninvia + "/" + nome_file

    def _gricevi(self, nome_file):
        """ Ritorna il percorso completo del file da Ricevere """
        return self._nricevi + "/" + nome_file

    @WaAuth()
    def eliminaFile(self, nome_file):
        """ Elimina il file indicato """
        os.unlink("%s/%s" % (self._cartella, nome_file))

    def _getListaFiles(self, cartella):
        """ Recupera la lista dei file per la cartella """
        unzippatutto(cartella)
        self.listaFiles = []
        tlista = {}
        for nfile in glob(cartella+"/*.xml"):
            tlista[os.path.basename(nfile)] = os.path.getmtime(nfile)
        nfiles = tlista.keys()
        nfiles.sort(lambda x, y: cmp(tlista[x], tlista[y]))
        nfiles.reverse()
        for nfile in nfiles:
            ilFile = _MiniFileInfo(cartella+"/"+nfile)
            if self.user.livello < self.user.ADMIN:
                if ilFile.codice in self.user.codici_ditte or \
                        self.user.livello > self.user.UAZIENDA:
                    self.listaFiles.append(ilFile)
            else:
                self.listaFiles.append(ilFile)
        return self.listaFiles

    def leggiListaFile(self):
        """ Ritorna la lista dei files """
        self._getListaFiles(self._cartella)
        root = Element('files')
        for nfile in self.listaFiles:
            ffile = SubElement(root, 'file')
            SubElement(ffile, 'nome').text= nfile.nome
            SubElement(ffile, 'data').text= nfile.sdata
            SubElement(ffile, 'ora').text= nfile.sora
            SubElement(ffile, 'info').text= nfile.getInfo()
        self.risposta.contenuto = tostring(root)

    @WaAuth()
    def getFileInfo(self, nome_file):
        """ recupera le info di un file """
        il_file = _MiniFileInfo(self._cartella + "/" + nome_file)
        self.risposta.contenuto = il_file.getInfo()

    def getFilesInfo(self):
        """ Recupera le info per i file ricevuto (lato) """
        self._getListaFiles(self._nricevi)
        html = '<div class="t1"><p class="p1">Ricevuti i seguenti files: </p>'
        for f in self.listaFiles:
            html += f.getInfoWData()
        html += '</div>'
        self.risposta.contenuto = html

    def getRicInfo(self):
        """ Recupera le info per la ricezione """
        html =  '<div class="dInfo">'
        html += '<p class="t2">Ultima ricezione</p>'
        html += '<div class="fInfo"><p>%s</p></div></div>'
        html += '<div class="dInfo">'
        html += '<p class="t2">Inizio connessione</p>'
        html += '<div class="fInfo"><p>%s</p></div></div>'
        if self._getListaFiles(self._nricevi):
            ricezione = "%s %s" % (
                    self.listaFiles[0].sdata,
                    self.listaFiles[0].sora)
        else:
            ricezione = 'Nessuna'
        self.risposta.contenuto = html % (
                ricezione,
                self.session['login_datetime'].strftime("%d/%m%Y %H.%M.%S")
                )

    def upLoadFile(self):
        """ upload di un file """
        for nfile in self._files['file']:
            open(self._cartella + "/" + nfile['filename'],
                    "wb").write(nfile['body'])
        self._addHead("Location", "/")
        self._addHead("Refresh", "0; url=/")
        self.leggiListaFile()

    @WaAuth()
    def invioGenerale(self, presenze=None, somm_dist=None, note=None,
            invia_ana_dipe=None):
        """ Gestisce l'invio da parte dell'utente """
        if invia_ana_dipe:
            pass
        if presenze or somm_dist or note:
            from invia_ricevi_dati import Invia
            self.risposta = Invia(self).invia()


def dogentask(sessiondb, session, request):
    """ Esegue il task richiesto """
    return GestioneInviaRicevi(sessiondb, session, request).doTask()
