# -*- coding: utf-8 -*-

from xml.etree          import cElementTree as ET
from sqlalchemy         import and_

from waonline.utils.utility import toJSON, to_dict, accentate2entity
from waonline.apps.anagrafiche.tabelle.ditte import DittaCentroCosto,\
        DittaBanca

from waonline.core.rbase import RBase
from .tabelle.tabelle_c  import *
from .tabelle.tabelle_v  import *
from .tabelle.tabelle_p  import *
from .tabelle.tabelle_l  import *
from waonline.core import sessions

# commento
class GestioneListe(RBase):
    """ Ritorna la lista per la tabella specificata"""
    def _post_init_(self):
        self._set_contr_sudd_terr()


    def getANF(self):
        return self._getListaValori(classe=ANF, filtri=self._getPeriodoVal(ANF))


    def getApprendistati(self):
        filtri = [ContrattoApprend.contratto == self.cod_contr] + self._getPeriodoVal(ContrattoApprend)
        if self.sudd_contr:
            filtri.append(ContrattoApprend.sudd_contr == self.sudd_contr or ContrattoApprend.sudd_contr == None)
        if self.sudd_terr:
            filtri.append(ContrattoApprend.sudd_terr == self.sudd_terr or ContrattoApprend.sudd_terr == None)
        return self._getListaValori(classe=ContrattoApprend, filtri=filtri)


    def getBanche(self):
        return self._getListaValori(classe=Banca)


    def getCausali(self, tipo=None, campi=['codice', 'descrizione']):
        filtri = ["codice NOT IN ('I1', 'AA', 'ZZ', 'FG', 'Y0')"]
        if tipo != None:
            filtri.append("is_no_calendario = %s" % tipo)
        if not self._ditta.filtro_caus_wa:
            self._ditta.filtro_caus_wa = "T"
        if self._ditta.filtro_caus_wa != "N":
            if self._ditta.filtro_caus_wa != "T":
                filtri.append("codice in (SELECT codice FROM "\
                "ditte_wa_causali "\
                "WHERE ditta_id = %(ditta_id)s)")
            return self._getListaValoriJ('causali', filtri=filtri, campi=campi,
                    periodo=self._periodo)
        else:
            self._repHead("Content-Type", "application/javascript")
            self.risposta.contenuto = toJSON([])
            return []

    def getCausaliCalend(self):
        self.getCausali(False)


    def getCausaliEvento(self):
        self.getCausali(True)


    def getCauseCessazioni(self):
        return self._getListaValori(classe=CausaCessazione)


    def getCentriCosto(self):
        return self._getListaValoriJ(classe=DittaCentroCosto,
                filtri=['ditta_id = %s' % self._ditta.id])
        #return self._getListaValori(classe=DittaCentroCosto, filtri=[DittaCentroCosto.ditta_id == self._ditta.id])


    def getCollaborazioni(self):
        return self._getListaValori(classe=Collaborazione)


    def getComuni(self):
        self._carica_province()
        return self._getListaValori(classe=Comune, filtri=[Comune.is_nazione == False, Comune.is_ex_comune == False],
                                     campi=[("codice", "cod_fiscale"), ("descrizione", "descrizione"),
                                            ("tipo", "_tipo_comune"), ("prov.", "_prov_comune"), ("cap", "cap")],
                                     ordinamento=Comune.descrizione
                                     )


    def getComuniAll(self):
        self._carica_province()
        return self._getListaValori(classe=Comune, filtri=[],
                                     campi=[("codice", "cod_fiscale"), ("descrizione", "descrizione"),
                                            ("tipo", "_tipo_comune"), ("prov.", "_prov_comune"), ("cap", "cap")],
                                     ordinamento=Comune.descrizione
                                     )


    def getContratti(self):
        return self._getListaValori(classe=Contratto, filtri=self._getPeriodoVal(Contratto))


    def getContrattiSuddContr(self):
        return self._getListaValori(classe=ContrattoSuddContr, filtri=self._getPeriodoVal(ContrattoSuddContr) +
                                    ["contratto = '%s'" % self.cod_contr])


    def getContrattiSuddTerr(self):
        return self._getListaValori(classe=ContrattoSuddTerr, filtri=self._getPeriodoVal(ContrattoSuddTerr) +
                                    ["contratto = '%s'" % self.cod_contr])


    def getDetrazArt13(self):
        return self._getListaValori(classe=DetrazioniArt13)


    def getDetrazFiscParent(self):
        return self._getListaValori(classe=DetrazioniFiscParent)


    def getDichiarazSpettanza(self):
        return self._getListaValori(classe=DichiarazSpettanza)


    def getDittaBanche(self):
        return self._getListaValori(classe=DittaBanca, filtri=[DittaBanca.ditta_id == self._ditta.id])


    def getEspoCedolini(self):
        return self._getListaValori(classEspoCedolino)


    def getInpsContratti(self):
        return self._getListaValori(classe=InpsContratto)


    def getInvalidita(self):
        return self._getListaValori(classe=Invalidita)


    def getLivelli(self, **kwargs):
        if kwargs:
            self._set_contr_sudd_terr(kwargs.get('periodo'), kwargs.get('ditta'), kwargs.get('dipte'))
        filtri = ["contratto = '%s' " % self.cod_contr] + self._getPeriodoVal(ContrattoLivello)
        if self.sudd_contr:
            filtri.append(ContrattoLivello.sudd_contr == self.sudd_contr or ContrattoLivello.sudd_contr == None)
        if self.sudd_terr:
            filtri.append(ContrattoLivello.sudd_terr == self.sudd_terr or ContrattoLivello.sudd_terr == None)
        return self._getListaValori(classe=ContrattoLivello, filtri=filtri)


    def getNazioni(self):
        return self._getListaValori(classe=Comune, filtri=[Comune.cod_uniemens != None])


    def getParentele(self):
        return self._getListaValori(classe=Parentela)


    def getPartTime(self):
        return self._getListaValori(classe=PartTime)


    def getQualifiche(self, **kwargs):
        if kwargs:
            self._set_contr_sudd_terr(kwargs.get('periodo'), kwargs.get('ditta'), kwargs.get('dipte'))
        contr_id = self._q(Contratto.id).filter(and_(Contratto.codice == self.cod_contr,
                                                     Contratto.periodo_inizio <= self._periodo,
                                                     Contratto.periodo_fine >= self._periodo
                                                     )).first()[0]
        for qual in self._q(ContrattoQual).filter(ContrattoQual.contratto_id == contr_id):
            try:
                orf = or_(orf, Qualifica.codice.like(qual.codice.replace("*", "_")))
            except:
                orf = Qualifica.codice.like(qual.codice.replace("*", "_"))
        return self._getListaValori(classe=Qualifica, filtri=self._getPeriodoVal(Qualifica) +
                                    ["codice not like '%*%'", "contr_estensione is NULL", orf],
                                    )


    def getRapporti(self):
        return self._getListaValori(classe=Rapporto, filtri=self._getPeriodoVal(Rapporto))


    def getRetribMovti(self):
        return self._getListaValori(classe=RetribuzioneMov)


    def getGeneri(self):
        return self._getListaValori(classe=Sesso)


    def getSommDist(self):
        return self._getListaValori(classe=SommDist)


    def getStatiCivili(self):
        return self._getListaValori(classe=StatoCivile)


    def getTipiContrDeterm(self):
        return self._getListaValori(classe=TipoContrDeterm)


    def getTitoliStudio(self):
        return self._getListaValori(classe=TitoloStudio)


    def getVoci(self):
        filtri = []
        if not self._ditta.filtro_voci_wa:
            self._ditta.filtro_voci_wa = "T"
        if self._ditta.filtro_voci_wa != "N":
            if self._ditta.filtro_voci_wa != "T":
                filtri.append("codice in (SELECT codice FROM "\
                "ditte_wa_voci "\
                "WHERE ditta_id = %(ditta_id)s)")
            return self._getListaValoriJ('voci', filtri=filtri,
                    periodo=self._periodo)
        else:
            self._repHead("Content-Type", "application/javascript")
            self.risposta.contenuto = toJSON([])
            return []


    """
        P R I V A T E
    """
    def _set_contr_sudd_terr(self, periodo=None, ditta=None, dipte=None):
        self.cod_contr = None
        self.sudd_contr = None
        self.sudd_terr = None
        if not self._ditta and not ditta:
            return
        if periodo:
            self._periodo = periodo
        if not ditta:
            ditta = self._ditta
        if not dipte:
            if self._dipti:
                dipte = self._dipti[0]
        if not self._periodo:
            self._periodo = int(self.session['periodo_attuale'].strftime("%Y%m"))
        if dipte:
            if dipte.qualifica.contratto:
                self.cod_contr = dipte.qualifica.contratto
            if dipte.qualifica.sudd_contr:
                self.sudd_contr = dipte.qualifica.sudd_contr
            if dipte.qualifica.sudd_terr:
                self.sudd_terr = dipte.qualifica.sudd_terr
        if not self.cod_contr:
            self.cod_contr = ditta.contratto
        if not self.sudd_contr:
            self.sudd_contr = ditta.sudd_contr
        if not self.sudd_terr:
            self.sudd_terr = ditta.sudd_terr

    def _getListaValoriJ(self, classe, filtri=[],
            campi=['codice', 'descrizione'], ordinamento='codice',
            periodo=None, union=True):
        if self.ga("elemento", None):
            filtri.append("lower(descrizione) LIKE lower('%s%%')" %\
                    self.ga("elemento", None))
        if periodo:
            filtri.append("periodo_inizio <= %(periodo)s "\
                    "AND periodo_fine >= %(periodo)s")
        periodo = periodo or self._periodo
        if hasattr(classe, "__tablename__"):
            tname = classe.__tablename__
        else:
            tname = classe
        squery = "SELECT %(campi)s FROM %(tabella)s WHERE 1=1"+\
                ''.join(map(lambda x: " AND " + x, filtri))+\
                " GROUP BY %(campi)s ORDER BY %(ordinamento)s"
        cquery = {'tabella': tname, 'campi': ', '.join(campi),
                    'ordinamento': ordinamento, 'periodo': periodo,
                    'ditta_id': self._ditta.id}
        squery = squery % cquery
        try:
            info = to_dict(self._ex(squery).fetchall())
        except Exception, err:
            self.errore(err)
            self.warn(squery)
            self.sessiondb.rollback()
            info = []
        self._repHead("Content-Type", "application/javascript")
        self.risposta.contenuto = toJSON(info)
        return info


    def _getListaValori(self, classe, filtri=[], campi=[], ordinamento=None, ritorno="codice",):
        root = ET.Element("elementi")
        query = self._q(classe)
        # !!!!! il filtro viene gestito direttamente lato client: in teoria non serve
        if self.ga("elemento", None):
            filtri.append("lower(descrizione) LIKE lower('%s%%')" % self.ga("elemento", None))
        # !!!!!
        for filtro in filtri:
            query = query.filter(filtro)
        if not ordinamento:
            try:
                ordinamento = classe.codice
            except:
                ordinamento = classe.id
        query = query.order_by(ordinamento)
        for elemento in query.all():
            tag = ET.SubElement(root, "elemento")
            if campi:
                for campo in campi:
                    try:
                        # estrazione tramite funzione
                        ET.SubElement(tag, campo[0]).text = unicode(getattr(self, campo[1])(elemento))
                    except:
                        # estrazione diretta del campo
                        ET.SubElement(tag, campo[0]).text = unicode(getattr(elemento, campo[1]))
            else:
                ET.SubElement(tag, "codice").text = accentate2entity(unicode(elemento.codice))
                ET.SubElement(tag, "descrizione").text = accentate2entity(unicode(elemento.descrizione))

        self._repHead("Content-Type","text/xml")
        self.risposta.contenuto = ET.tostring(root)
        return query


    def _getPeriodoVal(self, classe):
        return [classe.periodo_inizio <= self._periodo , classe.periodo_fine >= self._periodo]


    def _carica_province(self):
        self.province = {}
        for id, cod in self._q(Provincia.id, Provincia.codice):
            self.province[id] = cod


    def _prov_comune(self, comune):
        if comune.provincia_id in self.province:
            return self.province[comune.provincia_id]
        else:
            return ""


    def _tipo_comune(self, comune):
        if comune.is_nazione:
            return "N"
        elif comune.comune_locFisc == "L":
            return "L"
        elif comune.is_ex_comune:
            return "*"
        else:
            return ""


@sessions.require_lock
def carica_causali_in_session(rbase):
    # carica le causali in session
    causali_db = GestioneListe(rbase).getCausali(campi=['codice', 'descrizione', 'tipo', 'is_no_calendario',
                                                        'valida_in_giorni', 'is_unica_giorno','is_giorno_completo',])
    causali = {}
    for causale in causali_db:
        causali[causale['codice']] = {'descrizione':causale['descrizione'],
                                      'tipo':causale['tipo'],
                                      'is_no_calendario':causale['is_no_calendario'],
                                      'valida_in_giorni':causale['valida_in_giorni'],
                                      'is_unica_giorno':causale['is_unica_giorno'],
                                      'is_giorno_completo':causale['is_giorno_completo'],
                                      }
    rbase.session['causali_db'] = causali
    rbase.session.to_save = True

def dogentask(sessiondb, session, request):
    return GestioneListe(sessiondb, session, request).doTask()
