# -*- coding: utf-8 -*-

import ftplib
import os
import sys
import socket
import subprocess
import traceback
import zipfile
from datetime import datetime
from xml.etree.cElementTree import parse as xml_parse

from sqlalchemy.sql import and_

from waonline import conf
from waonline.core.rbase import RBase
from waonline.core import tracciati
from waonline.apps.gestione.tabelle.banche import *
from waonline.apps.anagrafiche.tabelle.dipendenti import *
from waonline.apps.anagrafiche.tabelle.ditte import *
from waonline.apps.anagrafiche.tabelle.studio import *
from waonline.apps.presenze.tabelle.presenze import *
from waonline.apps.gestione.tabelle.tabelle_c import *
from waonline.apps.gestione.tabelle.tabelle_l import *
from waonline.apps.gestione.tabelle.tabelle_v import *
from waonline.apps.gestione.tabelle.tabelle_p import *
from waonline.core.users.tabelle.ftp_passwd import FtpPasswd
from waonline.utils.utility import *


from waonline.utils.xmldictconfig import XmlDictConfig as x2d
from waonline.core.ete_dumbobj import dict2edo



_esterna = False


"""
    G E S T I O N E  -  R I C E V I
"""
class Ricevi(RBase):
    """
        Classe per la ricezione dei file inviati dalle paghe
    """
    def _post_init_(self):
        self._i = 0
        self._tag = ""
        self.ditta_id = None
        self.dipte_id = None
        self.dipti_id_ricevuti = []
        self.nome_file = self.ga("nome_file")
        if self.nome_file:
            self.perc_file = "%s/%s/ricevi/%s" %(conf.DEFAULT_DATA_PATH, self.user.studio_codice, self.nome_file)
            self.root = xml_parse(self.perc_file).getroot()
        return
        try:
            # le tabelle "C" personalizzate inviate dallo studio vengono sempre ricaricate tutte da capo
            if self.root.find('info') and self.root.find('info').attrib.get('tab_personalizzate', 'N') == "S":
                for cls in [Aliquota, CassaEdile, Causale, Contratto, Voce]:
                    try:
                        self.sessiondb.query(cls).filter(cls.codice.like('9%')).delete('fetch')
                    except:pass
                for cls in [ContrattoApprend, ContrattoFest, ContrattoLivello, ContrattoOrario,
                            ContrattoStraMagg, ContrattoSuddContr, ContrattoSuddTerr]:
                    try:
                        self.sessiondb.query(cls).filter(cls.contratto.like('9%')).delete('fetch')
                    except:pass
                self.sessiondb.commit()
        except Exception, exc:
            if _esterna:
                LogErrori().errore("Errore su apertura file xml:\n\n%s" %(exc))
            else:
                self.info(exc)


    @property
    def cod_ditta_from_nome_file(self):
        if _esterna:
            return self.nome_file[1:8]
        else:
            return self.nome_file[0:7]


    def ricevi(self):
        if not _esterna:
            self._add_to_log()
            #self._post_init_()
        for tagRoot in self.root.getchildren():
            if tagRoot.tag in []:
                continue
            if _esterna:
               print tagRoot.tag
            try:
                if tagRoot.tag == "info":
                    continue
                if tagRoot.tag == "Ditta":
                    self.ditta_id = None
                if tagRoot.tag in ['dipendente', 'collaboratore']:
                    self.dipte_id = None
                    for tag_dipte in tagRoot:
                        getattr(self, "_" + tag_dipte.tag)(tag_dipte)
                else:
                    getattr(self, "_" + tagRoot.tag)(tagRoot)
            except Exception, exc:
                if _esterna:
                    LogErrori().errore("Errore in fase di ricezione:\n\n%s" %(exc))
                else:
                    self.info(exc)

        self._elimina_non_ricevuti()

        os.rename(self.perc_file, self.perc_file.replace(".xml", ".old"))
        self.perc_file = self.perc_file.replace(".xml", ".old")
        self._addMess("Aggiornamento completato con successo")


    """
        R I C E Z I O N E  _  D I T T A
    """
    def _Ditta(self, tag):
        codice = tag.find("codice").text
        rec = self._q(Ditta).filter(Ditta.codice == codice).first()
        if rec:
            for tab in [DittaBanca, DittaCantiere, DittaCentroCosto, DittaFest, DittaOrario, DittaPosInail,
                        DittaSommDist, DittaStraMagg, DittaWACampi, DittaWACausali, DittaWAVoci]:
                self._q(tab).filter(tab.ditta_id == rec.id).delete('evaluate')
            self.sessiondb.commit()
        rec = self._importa_record(tag, rec)[0]
        self.ditta_id = rec.id


    def _DittaBanca(self, tag):
        self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])


    def _DittaCantiere(self, tag):
        self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])


    def _DittaCentroCosto(self, tag):
        self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])


    def _DittaFest(self, tag):
        rec = self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])[0]
        for tagF in tag.findall("DittaFestTab"):
            self._importa_record(tagF, set_campi=[('festivita_id', rec.id)])


    def _DittaOrario(self, tag):
        rec = self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])[0]
        for tagF in tag.findall("DittaOrarioSett"):
            recF = self._importa_record(tagF, set_campi=[('orario_id', rec.id)])[0]
            for tagN in tagF.findall("DittaOrarioSettGG"):
                self._importa_record(tagN, set_campi=[('orario_sett_id', recF.id)])


    def _DittaPosInail(self, tag):
        self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])


    def _DittaSommDist(self, tag):
        rec = self._q(DittaSommDist).filter(and_(DittaSommDist.ditta_id == self.ditta_id,
                                                 DittaSommDist.tipo == tag.find("tipo").text,
                                                 DittaSommDist.cod_fiscale == tag.find("cod_fiscale").text,
                                                 )).first()
        self._importa_record(tag, rec, set_campi=[('ditta_id', self.ditta_id)])


    def _DittaStraMagg(self, tag):
        self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])


    def _DittaWACampi(self, tag):
        self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])


    def _DittaWACausali(self, tag):
        self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])


    def _DittaWAVoci(self, tag):
        self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])


    """
        R I C E Z I O N E  _  D I P E N D E N T I
    """
    def _Dipendente(self, tag):
        if not self.ditta_id: # puo' verificarsi se lo studio non ha inviato anche l'anagrafica ditta
            self.ditta_id = self._q(Ditta.id).filter(Ditta.codice == self.cod_ditta_from_nome_file).first()[0]
        if tag.find("is_collab").text == "S":
            isCollab = True
        else:
            isCollab = False
        rec = self._q(Dipendente).filter(and_(Dipendente.ditta_id == self.ditta_id,
                                              Dipendente.codice == tag.find("codice").text,
                                              Dipendente.is_collab == isCollab
                                              )).first()
        if rec:
            for tab in [DipendenteCarFam, DipendenteFest, DipendenteOrario,
                        DipendenteQual, DipendenteRappPrec, DipendenteStraMagg]:
                self._q(tab).filter(tab.dipendente_id == rec.id).delete('evaluate')
            self.sessiondb.commit()
        rec = self._importa_record(tag, rec, set_campi=[('ditta_id', self.ditta_id)])[0]
        self.dipte_id = rec.id
        self.dipti_id_ricevuti.append(self.dipte_id)



    def _DipendenteCarFam(self, tag):
        self._importa_record(tag, set_campi=[('dipendente_id', self.dipte_id)])


    def _DipendenteFest(self, tag):
        rec = self._importa_record(tag, set_campi=[('dipendente_id', self.dipte_id)])[0]
        for tagF in tag.findall("DipendenteFestTab"):
            self._importa_record(tagF, set_campi=[('festivita_id', rec.id)])


    def _DipendenteOrario(self, tag):
        rec = self._importa_record(tag, set_campi=[('dipendente_id', self.dipte_id)])[0]
        for tagF in tag.findall("DipendenteOrarioSett"):
            recF = self._importa_record(tagF, set_campi=[('orario_id', rec.id)])[0]
            for tagN in tagF.findall("DipendenteOrarioSettGG"):
                self._importa_record(tagN, set_campi=[('orario_sett_id', recF.id)])


    def _DipendenteQual(self, tag):
        self._importa_record(tag, set_campi=[('dipendente_id', self.dipte_id)])


    def _DipendenteRappPrec(self, tag):
        self._importa_record(tag, set_campi=[('dipendente_id', self.dipte_id)])


    def _DipendenteStraMagg(self, tag):
        self._importa_record(tag, set_campi=[('dipendente_id', self.dipte_id)])


    """
        R I C E Z I O N E  _  S T U D I O
    """
    def _Studio(self, tag):
        rec = self._q(Studio).filter(Studio.codice == tag.find("codice").text).first()
        if rec:
            for tab in [StudioCampoPers, StudioCampoWA, StudioVoce]:
                self._q(tab).filter(tab.studio_id == rec.id).delete('evaluate')
            self.sessiondb.commit()
        rec = self._importa_record(tag, rec)[0]
        for tagF in tag.findall("StudioCampoPers"):
            self._importa_record(tagF, set_campi=[('studio_id', rec.id)])
        for tagF in tag.findall("StudioCampoWA"):
            self._importa_record(tagF, set_campi=[('studio_id', rec.id)])
        for tagF in tag.findall("StudioVoce"):
            self._importa_record(tagF, set_campi=[('studio_id', rec.id)])


    """
        R I C E Z I O N E  _   T A B E L L E  _  B
    """
    def _Banca(self, tag):
        rec = self._q(Banca).filter(Banca.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    """
        R I C E Z I O N E  _  T A B E L L E  _  C
    """
    def _Aliquota(self, tag):
        rec = self._q(Aliquota).filter(and_(Aliquota.codice == tag.find("codice").text,
                                            Aliquota.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                            )).first()
        if rec:
            self._q(AliquotaSub).filter(AliquotaSub.aliquota_id == rec.id).delete('evaluate')
            self.sessiondb.commit()
        rec = self._importa_record(tag, rec)[0]
        for tagF in tag.findall("AliquotaSub"):
            self._importa_record(tagF, set_campi=[('aliquota_id', rec.id)])


    def _CassaEdile(self, tag):
        rec = self._q(CassaEdile).filter(and_(CassaEdile.codice == tag.find("codice").text,
                                              CassaEdile.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                              )).first()
        self._importa_record(tag, rec)


    def _Causale(self, tag):
        rec = self._q(Causale).filter(and_(Causale.codice == tag.find("codice").text,
                                           Causale.valida_per == tag.find("valida_per").text,
                                           Causale.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                           )).first()
        self._importa_record(tag, rec)


    def _Contratto(self, tag):
        rec = self._q(Contratto).filter(and_(Contratto.codice == tag.find("codice").text,
                                             Contratto.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                             )).first()
        if rec:
            self._q(ContrattoQual).filter(ContrattoQual.contratto_id == rec.id).delete('evaluate')
            self.sessiondb.commit()
        rec = self._importa_record(tag, rec)[0]
        for tagF in tag.findall("ContrattoQual"):
            self._importa_record(tagF, set_campi=[('contratto_id', rec.id)])


    def _ContrattoApprend(self, tag):
        rec = self._q(ContrattoApprend).filter(and_(ContrattoApprend.contratto == tag.find("contratto").text,
                                                    ContrattoApprend.codice == tag.find("codice").text,
                                                    ContrattoApprend.sudd_contr == tag.find("sudd_contr").text,
                                                    ContrattoApprend.sudd_terr == tag.find("sudd_terr").text,
                                                    ContrattoApprend.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                    )).first()
        self._importa_record(tag, rec)


    def _ContrattoFest(self, tag):
        rec = self._q(ContrattoFest).filter(and_(ContrattoFest.contratto == tag.find("contratto").text,
                                                 ContrattoFest.sudd_contr == tag.find("sudd_contr").text,
                                                 ContrattoFest.sudd_terr == tag.find("sudd_terr").text,
                                                 ContrattoFest.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                 ContrattoFest.aaaa == tag.find("aaaa").text,
                                                 ContrattoFest.mmgg == tag.find("mmgg").text,
                                                 )).first()
        self._importa_record(tag, rec)


    def _ContrattoLivello(self, tag):
        rec = self._q(ContrattoLivello).filter(and_(ContrattoLivello.contratto == tag.find("contratto").text,
                                                    ContrattoLivello.codice == tag.find("codice").text,
                                                    ContrattoLivello.sudd_contr == tag.find("sudd_contr").text,
                                                    ContrattoLivello.sudd_terr == tag.find("sudd_terr").text,
                                                    ContrattoLivello.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                    )).first()
        self._importa_record(tag, rec)


    def _ContrattoOrario(self, tag):
        rec = self._q(ContrattoOrario).filter(and_(ContrattoOrario.contratto == tag.find("contratto").text,
                                                   ContrattoOrario.qualifica == tag.find("qualifica").text,
                                                   ContrattoOrario.sudd_contr == tag.find("sudd_contr").text,
                                                   ContrattoOrario.sudd_terr == tag.find("sudd_terr").text,
                                                   ContrattoOrario.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                   )).first()
        if rec:
            self._q(ContrattoOrarioSett).filter(ContrattoOrarioSett.orario_id == rec.id).delete('evaluate')
            self.sessiondb.commit()
        rec = self._importa_record(tag, rec)[0]
        for tagF in tag.findall("ContrattoOrarioSett"):
            recF = self._importa_record(tagF, set_campi=[('orario_id', rec.id)])[0]
            for tagN in tagF.findall("ContrattoOrarioSettGG"):
                self._importa_record(tagN, set_campi=[('orario_sett_id', recF.id)])


    def _ContrattoStraMagg(self, tag):
        rec = self._q(ContrattoStraMagg).filter(and_(ContrattoStraMagg.contratto == tag.find("contratto").text,
                                                     ContrattoStraMagg.qualifica == tag.find("qualifica").text,
                                                     ContrattoStraMagg.sudd_contr == tag.find("sudd_contr").text,
                                                     ContrattoStraMagg.sudd_terr == tag.find("sudd_terr").text,
                                                     ContrattoStraMagg.cod_straord == tag.find("cod_straord").text,
                                                     ContrattoStraMagg.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                     )).first()
        self._importa_record(tag, rec)


    def _ContrattoSuddContr(self, tag):
        rec = self._q(ContrattoSuddContr).filter(and_(ContrattoSuddContr.contratto == tag.find("contratto").text,
                                                      ContrattoSuddContr.codice == tag.find("codice").text,
                                                      ContrattoSuddContr.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                      )).first()
        self._importa_record(tag, rec)


    def _ContrattoSuddTerr(self, tag):
        rec = self._q(ContrattoSuddTerr).filter(and_(ContrattoSuddTerr.contratto == tag.find("contratto").text,
                                                     ContrattoSuddTerr.codice == tag.find("codice").text,
                                                     ContrattoSuddTerr.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                     )).first()
        self._importa_record(tag, rec)


    def _Voce(self, tag):
        rec = self._q(Voce).filter(and_(Voce.codice == tag.find("codice").text,
                                        Voce.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                        )).first()
        self._importa_record(tag, rec)


    """
        R I C E Z I O N E  _  T A B E L L E  _  L
    """
    def _Comune(self, tag):
        rec = self._q(Comune).filter(Comune.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _Provincia(self, tag):
        rec = self._q(Provincia).filter(Provincia.codice == tag.find("codice").text).first()
        rec = self._importa_record(tag, rec)[0]


    def _Regione(self, tag):
        rec = self._q(Regione).filter(Regione.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    """
        R I C E Z I O N E  _   T A B E L L E  _  P
    """
    def _CausaCessazione(self, tag):
        rec = self._q(CausaCessazione).filter(CausaCessazione.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _Collaborazione(self, tag):
        rec = self._q(Collaborazione).filter(Collaborazione.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _DetrazioniArt13(self, tag):
        rec = self._q(DetrazioniArt13).filter(DetrazioniArt13.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _DetrazioniFiscParent(self, tag):
        rec = self._q(DetrazioniFiscParent).filter(DetrazioniFiscParent.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _DichiarazSpettanza(self, tag):
        rec = self._q(DichiarazSpettanza).filter(DichiarazSpettanza.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _EspoCedolino(self, tag):
        rec = self._q(EspoCedolino).filter(EspoCedolino.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _InpsContratto(self, tag):
        rec = self._q(InpsContratto).filter(InpsContratto.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _Invalidita(self, tag):
        rec = self._q(Invalidita).filter(Invalidita.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _Parentela(self, tag):
        rec = self._q(Parentela).filter(Parentela.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _PartTime(self, tag):
        rec = self._q(PartTime).filter(PartTime.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _Sesso(self, tag):
        rec = self._q(Sesso).filter(Sesso.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _SommDist(self, tag):
        rec = self._q(SommDist).filter(SommDist.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _StatoCivile(self, tag):
        rec = self._q(StatoCivile).filter(StatoCivile.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _TipoContrDeterm(self, tag):
        rec = self._q(TipoContrDeterm).filter(TipoContrDeterm.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _TitoloStudio(self, tag):
        rec = self._q(TitoloStudio).filter(TitoloStudio.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    def _RetribuzioneMov(self, tag):
        rec = self._q(RetribuzioneMov).filter(RetribuzioneMov.codice == tag.find("codice").text).first()
        self._importa_record(tag, rec)


    """
        R I C E Z I O N E  _   T A B E L L E  _  V
    """
    def _ANF(self, tag):
        rec = self._q(ANF).filter(and_(ANF.codice == tag.find("codice").text,
                                       ANF.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                       )).first()
        self._importa_record(tag, rec)


    def _CampoWA(self, tag):
        rec = self._q(CampoWA).filter(and_(CampoWA.codice == tag.find("codice").text)).first()
        self._importa_record(tag, rec)


    def _CausaConguaglio(self, tag):
        rec = self._q(CausaConguaglio).filter(and_(CausaConguaglio.codice == tag.find("codice").text,
                                                   CausaConguaglio.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                   )).first()
        self._importa_record(tag, rec)


    def _CollAttivita(self, tag):
        rec = self._q(CollAttivita).filter(and_(CollAttivita.codice == tag.find("codice").text,
                                                CollAttivita.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                )).first()
        self._importa_record(tag, rec)


    def _CollFormaAssicura(self, tag):
        rec = self._q(CollFormaAssicura).filter(and_(CollFormaAssicura.codice == tag.find("codice").text,
                                                     CollFormaAssicura.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                     )).first()
        self._importa_record(tag, rec)


    def _CollRapporto(self, tag):
        rec = self._q(CollRapporto).filter(and_(CollRapporto.codice == tag.find("codice").text,
                                                CollRapporto.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                )).first()
        self._importa_record(tag, rec)


    def _Costante(self, tag):
        rec = self._q(Costante).filter(and_(Costante.codice == tag.find("codice").text,
                                            Costante.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                            )).first()
        if rec:
            self._q(CostanteCampo).filter(CostanteCampo.costante_id == rec.id).delete('evaluate')
            self.sessiondb.commit()
        rec = self._importa_record(tag, rec)[0]
        for tagF in tag.findall("CostanteCampo"):
            self._importa_record(tagF, set_campi=[('costante_id', rec.id)])


    def _GruppoAliquota(self, tag):
        rec = self._q(GruppoAliquota).filter(and_(GruppoAliquota.codice == tag.find("codice").text,
                                                  GruppoAliquota.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                  )).first()
        self._importa_record(tag, rec)


    def _GruppoContratto(self, tag):
        rec = self._q(GruppoContratto).filter(and_(GruppoContratto.codice == tag.find("codice").text,
                                                   GruppoContratto.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                   )).first()
        self._importa_record(tag, rec)


    def _GruppoVoce(self, tag):
        rec = self._q(GruppoVoce).filter(and_(GruppoVoce.codice == tag.find("codice").text,
                                              GruppoVoce.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                              )).first()
        self._importa_record(tag, rec)


    def _InailPosizione(self, tag):
        rec = self._q(InailPosizione).filter(and_(InailPosizione.codice == tag.find("codice").text,
                                                  InailPosizione.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                                  )).first()
        self._importa_record(tag, rec)


    def _InailSede(self, tag):
        rec = self._q(InailSede).filter(and_(InailSede.codice == tag.find("codice").text,
                                             InailSede.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                             )).first()
        self._importa_record(tag, rec)


    def _Qualifica(self, tag):
        rec = self._q(Qualifica).filter(and_(Qualifica.codice == tag.find("codice").text,
                                             Qualifica.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                             )).first()
        if rec:
            self._q(QualificaRapp).filter(QualificaRapp.qualifica_id == rec.id).delete('evaluate')
            self.sessiondb.commit()
        rec = self._importa_record(tag, rec)[0]
        for tagF in tag.findall("QualificaRapp"):
            self._importa_record(tagF, set_campi=[('qualifica_id', rec.id)])


    def _Rapporto(self, tag):
        rec = self._q(Rapporto).filter(and_(Rapporto.codice == tag.find("codice").text,
                                            Rapporto.periodo_inizio == formatData(tag.find("periodo_inizio").text, "int", "%Y%m"),
                                            )).first()
        self._importa_record(tag, rec)


    """
        M I G R A Z I O N E  _  W A
    """
    def _DittaWAStatus(self, tag):
        try:
            self.ditta_id = self._q(Ditta.id).filter(Ditta.codice == self.cod_ditta_from_nome_file).first()[0]
        except:
            ditta = Ditta()
            ditta.codice = self.cod_ditta_from_nome_file
            self.sessiondb.add(ditta)
            self.sessiondb.commit()
            self.ditta_id = ditta.id

        rec = self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id)])[0]
        for tagF in tag.findall("DittaWAStatusPeriodo"):
            self._importa_record(tagF, set_campi=[('status_id', rec.id)])


    def _Presenze(self, tag):
        try:
            self.dipte_id = self._q(Dipendente.id).filter(Dipendente.codice == tag.attrib.get('cod_dipte')).first()[0]
        except Exception, err:
            dipte = Dipendente()
            dipte.ditta_id = self.ditta_id
            dipte.codice = tag.attrib.get('cod_dipte')
            dipte.is_collab = True
            dipte.matricola = "x"
            dipte.cognome = "x"
            dipte.nome = "x"
            self.sessiondb.add(dipte)
            self.sessiondb.commit()
            self.dipte_id = dipte.id

        rec = self._importa_record(tag, set_campi=[('ditta_id', self.ditta_id), ('dipendente_id', self.dipte_id)])[0]
        for tagF in tag.findall("PresenzeCalend"):
            recF = self._importa_record(tagF, set_campi=[('presenze_id', rec.id)])[0]
            for tagN in tagF.findall("PresenzeCalendCaus"):
                self._importa_record(tagN, set_campi=[('calendario_id', recF.id)])
        for tagF in tag.findall("PresenzeEvento"):
            self._importa_record(tagF, set_campi=[('presenze_id', rec.id)])
        for tagF in tag.findall("PresenzeLavDom"):
            self._importa_record(tagF, set_campi=[('presenze_id', rec.id)])
        for tagF in tag.findall("PresenzeVoce"):
            self._importa_record(tagF, set_campi=[('presenze_id', rec.id)])


    def _UAzienda(self, tag):
        # Inserisce utente UAzienda e Azienda su db WA
        dati = dict2edo(x2d(tag))
        dati.is_sped_anag = dati.is_sped_anag == 'S'
        dati.is_sped_pres = dati.is_sped_pres == 'S'
        dati.is_ricezione = dati.is_ricezione == 'S'
        dati.is_segreteria = dati.is_segreteria == 'S'
        dati.is_sessantesimi = dati.is_sessantesimi == 'S'
        #ipdb.set_trace()

        uazienda = self.user.get_by_username(dati.username)
        if uazienda:
            uazienda.update(**dati)
        else:
            uazienda = self.user.create_user(dati.username, dati.password,
                    **dati)
        for pref in tag.findall("UtentePref"):
            dati = dict2edo(x2d(pref))
            uazienda.add_pref(dati.nome, dati.valore, uazienda.id)

        for azienda in tag.findall("Azienda"):
            dati = dict2edo(x2d(azienda))
            codice = dati.codice
            del dati['codice']
            uazienda.create_ditta(codice, **dati)

    def _UStudio(self, tag):
        dati = dict2edo(x2d(tag))
        utente = self.user.get_by_username(dati.username)
        if utente:
            utente.update(**dati)
        else:
            self.user.create_user(dati.username, dati.password, **dati)

    """
        F U N Z I O N I  -  G E N E R I C H E
    """
    def _importa_record(self, tag, rec=None, set_campi=[]):
        classe = globals()[tag.tag]
        if not rec:
            rec = classe()
        for campo in set_campi:
            try:
                setattr(rec, campo[0], campo[1])
            except: pass
        for campo in tag:
            if not hasattr(rec, campo.tag):
                continue
            try:
                # campi particolari
                campo.text = getattr(self, "_%s_%s" %(tag.tag.lower(), campo.tag))(campo.text)
            except:
                # campi standard
                prop = getattr(classe, campo.tag)
                if campo.text and type(prop) != property:
                    if isinstance(prop.property.columns[0].type, Numeric):
                        campo.text = campo.text.replace(',', '.')
                    if isinstance(prop.property.columns[0].type, Boolean):
                        if campo.text == "S":
                            campo.text = True
                        else:
                            campo.text = False
            setattr(rec, campo.tag, campo.text)
        new = False
        if not rec.id:
            self.sessiondb.add(rec)
            new = True
        try:
            self.sessiondb.commit()
        except Exception, exc:
            self.sessiondb.rollback()

        return [rec, new]


    def _provincia_regione_id(self, codice):
        return self._q(Regione.id).filter(Regione.codice == codice).first()[0]


    def _comune_provincia_id(self, codice):
        return self._q(Provincia.id).filter(Provincia.codice == codice).first()[0]


    def _elimina_non_ricevuti(self):
        # Se lo studio ha inviato l'anagrafica "completa" dei dipendenti elimina dal db quelli non ricevuti
        if not self.dipti_id_ricevuti or \
          (self.root.find('info').attrib.get('dipendenti', 'N') != 'S' and \
           self.root.find('info').attrib.get('collaboratori', 'N') != 'S'):
            return
        if not self.ditta_id:
            self.ditta_id = self._q(Ditta.id).filter(Ditta.codice == self.cod_ditta_from_nome_file).first()[0]
        dipti = self._q(Dipendente.id, Dipendente.codice, Dipendente.is_collab)\
                    .filter(and_(Dipendente.ditta_id == self.ditta_id, ~Dipendente.id.in_(self.dipti_id_ricevuti)))\
                    .all()
        for dipte in dipti:
            try:
                presenze = self._q(Presenze).filter(and_(Presenze.ditta_id == self.ditta_id,
                                                         Presenze.dipendente_id == dipte.id
                                                         )).all()
                for pres in presenze:
                    pres.dipendente_cod = dipte.codice
                    pres.dipendente_is_collab = dipte.is_collab
                self.sessiondb.commit()
                self._q(Dipendente).filter(Dipendente.id == dipte.id).delete('evaluate')
                self.sessiondb.commit()
            except: pass



"""
    G E S T I O N E  -  I N V I A
"""

class Invia(RBase):
    """
        Classe per l'invio delle presenze alle paghe
    """
    def _post_init_(self):
        self.invia_presenze = self.ga("presenze")
        self.invia_somm_dist = self.ga("somm_dist")
        self.invia_note = self.ga("note")


    def invia(self):
        presenze = self._q(Presenze).filter(and_(Presenze.periodo == self._periodo,
                                                 Presenze.ditta_id == self._ditta.id,
                                                 )).all()
        file = "/tmp/DATI_WAO_%s_%s_%s.txt" %(self._ditta.codice,
                                              self._periodo,
                                              datetime.now().strftime("%Y%m%d_%H%M%S")
                                              )
        tr = tracciati.Tracciato(os.path.join(conf.WALL_DIR ,
            "resource", "cpy", "TrMovImport-2009.cpy"), file)

        # record '00': export
        prog = 0
        tr.tiporecimp_00.value = "00"
        tr.progrecimp_00.value = prog
        tr.fileidimp_00.value = "DATI MESE"
        tr.periodoimp_00.value = str(self._periodo)
        tr.prorammapaghe_00.value = "EssePaghe"
        tr.programmapresenze_00.value = "Essepaghe"
        tr.tipocoddipte_00.value = "C"
        tr.formula_00.value = ""
        tr.record_import_00.write()

        if self.invia_presenze:
            for pres in presenze:
                if not pres.dipendente_rel:
                    # dipendente eliminato
                    continue
                if pres.dipendente_rel.is_collab:
                    # collaboratore
                    continue
                # record '10': dipendente
                prog += 1
                tr.tiporecimp_10.value = "10"
                tr.progrecimp_10.value = prog
                tr.keydittaimp_10.value = pres.ditta_rel.codice
                tr.keydipteimp_10.value = pres.dipendente_rel.codice
                tr.nominativoimp_10.value = pres.dipendente_rel.nominativo
                tr.record_import_10.write()

                # record '14'
                for giorno in pres.calendario_rel:
                    # ore teoriche
                    prog += 1
                    tr.tiporecimp_14.value = "14"
                    tr.progrecimp_14.value = prog
                    tr.ggcausale_14.value = giorno.giorno_mese
                    if giorno.ecc == "FG":
                        tr.codcausale_14.value = "FG"
                    else:
                        tr.codcausale_14.value = "OT"
                    if giorno.ecc == "RS":
                        tr.orecausale_14.value = 50
                    elif giorno.ecc == "--":
                        tr.orecausale_14.value = 60
                    else:
                        if self._impostazioni.is_sessantesimi:
                            tr.orecausale_14.value = converti_ore(giorno.ore_teoriche, False)
                        else:
                            tr.orecausale_14.value = giorno.ore_teoriche
                    tr.altrocausale_14.value = ""
                    tr.record_import_14.write()
                    # ore retribuite
                    prog += 1
                    tr.tiporecimp_14.value = "14"
                    tr.progrecimp_14.value = prog
                    tr.ggcausale_14.value = giorno.giorno_mese
                    tr.codcausale_14.value = "OR"
                    if self._impostazioni.is_sessantesimi:
                        tr.orecausale_14.value = converti_ore(giorno.ore_retribuite, False)
                    else:
                        tr.orecausale_14.value = giorno.ore_retribuite
                    tr.altrocausale_14.value = ""
                    tr.record_import_14.write()
                    # causali
                    for causale in giorno.calendarioCaus_rel:
                        prog += 1
                        tr.tiporecimp_14.value = "14"
                        tr.progrecimp_14.value = prog
                        tr.ggcausale_14.value = giorno.giorno_mese
                        tr.codcausale_14.value = causale.codice
                        if self._impostazioni.is_sessantesimi:
                            tr.orecausale_14.value = converti_ore(causale.ore, False)
                        else:
                            tr.orecausale_14.value = causale.ore
                        tr.altrocausale_14.value = ""
                        tr.record_import_14.write()

                # record '12': eventi
                for evento in pres.eventi_rel:
                    prog += 1
                    tr.tiporecimp_12.value = "12"
                    tr.progrecimp_12.value = prog
                    tr.codevento_12.value = evento.codice
                    tr.dinevento_12.value = evento.data_inizio
                    tr.dfievento_12.value = evento.data_fine
                    if self._impostazioni.is_sessantesimi:
                        tr.oreevento_12.value = converti_ore(evento.ore, False)
                    else:
                        tr.oreevento_12.value = evento.ore
                    tr.record_import_12.write()

                # record '13': voci
                for voce in pres.voci_rel:
                    prog += 1
                    tr.tiporecimp_13.value = "13"
                    tr.progrecimp_13.value = prog
                    tr.prefvoce_13.value = voce.prefisso
                    tr.codvoce_13.value = voce.voce
                    tr.suffvoce_13.value = voce.suffisso
                    tr.dbvoce_13.value = voce.dato_base
                    tr.cuvoce_13.value = voce.costo_unitario
                    tr.percvoce_13.value = voce.perc
                    tr.impvoce_13.value = voce.importo
                    tr.record_import_13.write()

                # record '15': lavori a domicilio
                riga = 0
                for lav_dom in pres.lavori_dom_rel:
                    prog += 1
                    riga += 1
                    tr.tiporecimp_15.value = "15"
                    tr.progrecimp_15.value = prog
                    tr.progrigaldom_15.value = riga
                    tr.dataconsldom_15.value = lav_dom.data_consegna
                    tr.oraconsldom_15.value = int(str(lav_dom.ora_consegna).replace('.', ''))
                    tr.dataricldom_15.value = lav_dom.data_riconsegna
                    tr.orariclavdom_15.value = int(str(lav_dom.ora_riconsegna).replace('.', ''))
                    tr.descrlavdom_15.value = lav_dom.descrizione
                    tr.quantitalavdom_15.value = lav_dom.quantita
                    tr.record_import_15.write()

                # record '50': note presenze
                if pres.note:
                    tr.tiporecimp_50.value = "50"
                    tr.tipo_50.value = "N1"
                    for riga_1 in pres.note.split("\n"):
                        for riga_2 in [riga_1[i:i+250] for i in range(0, len(riga_1), 250)]:
                            prog += 1
                            tr.progrecimp_50.value = prog
                            tr.nota_50.value = riga_2
                            tr.record_import_50.write()

        # record '20': somministrati/distaccati
        riga = 0
        if self.invia_somm_dist:
            for somm_dist in self._ditta.somm_dist_rel:
                prog += 1
                riga += 1
                comune_nas = self._q(Comune).filter(Comune.codice == somm_dist.comune_nascita).first()
                comune_res = self._q(Comune).filter(Comune.codice == somm_dist.comune_resid).first()
                tr.tiporecimp_20.value = "20"
                tr.progrecimp_20.value = prog
                tr.keyditta_20.value = self._ditta.codice
                tr.prog_20.value = riga
                tr.tipo_20.value = somm_dist.tipo
                tr.cognome_20.value = somm_dist.cognome
                tr.nome_20.value = somm_dist.nome
                tr.sesso_20.value = somm_dist.sesso
                tr.datanas_20.value = somm_dist.data_nascita
                if comune_nas:
                    tr.comunenas_20.value = comune_nas.descrizione
                    if comune_nas.provincia_rel:
                        tr.provincianas_20.value = comune_nas.provincia_rel.codice
                tr.codfiscale_20.value = somm_dist.cod_fiscale
                tr.indirizzores_20.value = somm_dist.indirizzo_resid
                if comune_res:
                    tr.comuneres_20.value = comune_res.descrizione
                    tr.provinciares_20.value = comune_res.provincia_rel.codice
                    tr.capres_20.value = comune_res.cap
                tr.qualifica_20.value = somm_dist.qualifica
                tr.livello_20.value = somm_dist.livello
                tr.dataingresso_20.value = somm_dist.data_ingresso
                tr.datauscita_20.value = somm_dist.data_uscita
                tr.agenziaprov_20.value = somm_dist.agenzia
                tr.record_import_20.write()

        # record '50': note invio dati
        if self.invia_note:
            tr.tiporecimp_50.value = "50"
            tr.tipo_50.value = "N0"
            for riga_1 in self.invia_note.split("\n"):
                for riga_2 in [riga_1[i:i+250] for i in range(0, len(riga_1), 250)]:
                    prog += 1
                    tr.progrecimp_50.value = prog
                    tr.nota_50.value = riga_2
                    tr.record_import_50.write()

        tr.close()

        # invia il file
        err = ""
        nome_zip = os.path.basename(file).replace(".txt", ".zip")
        perc_zip = "%s/%s/invia/%s" %(conf.DEFAULT_DATA_PATH, self.user.studio_codice, nome_zip)
        try:
            zipfile.ZipFile(perc_zip, "w").write(file, os.path.basename(file), zipfile.ZIP_DEFLATED)
            if socket.gethostname() in ['InazEssepagheWA01']:
                    user = self.user._q(FtpPasswd).filter_by(cod_studio=self.user.studio_codice).one()
                    #user = WaStudioPasswd().get_studio(self.user.studio_codice)
                    #user = self.user.studio
                    ftp = ftplib.FTP('ftp.interno', user.username, user.passwd)
                    ftp.set_pasv(True)
                    f = open(perc_zip, 'rb')
                    fname = os.path.basename(perc_zip)
                    ftp.cwd("WA/FTP")
                    ftp.storbinary('STOR %s' % fname, f)
                    f.close()
                    ftp.quit()
            else:
                cmd = "scp %s webazienda@ftp.interno:/var/WA/Studio%s/FTP" %(perc_zip, self.user.studio_codice)
                err = subprocess.Popen(cmd.split(), shell=False, stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE).communicate()[1]
        except Exception, exc:
            err = str(exc)

        try:
            os.unlink(file)
        except:pass

        if err:
            self._addErr("Errore in fase di invio:<br>%s" %(str(err)))
        else:
            # aggiorna lo status ditta con "inviato"
            if self.invia_presenze:
                for status in self._ditta.status_wa_rel[0].status_periodi_rel:
                    if status.periodo == self._periodo:
                        status.is_inviato = True
                        self.sessiondb.commit()
                        break
            self._addMess("Invio completato.")

        return self.risposta


"""
    G E S T I O N E  -  E R R O R I
"""
class LogErrori():
    def errore(self, errore):
        if _esterna:
            log = open("%s/errori.log" %(conf.DEFAULT_DATA_PATH), "a")
            log.write(">>> %s:\n" %datetime.now().strftime("%d/%m/%Y - %H.%M.%S"))
            traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2], file=log)
            log.write("-" * 100 + "\n")
            log.close()
            sys.exit(-1)
        else:
            return errore



"""
    M A I N
"""
def dogentask(sessiondb, session, request):
    if request.arguments['task'][0] == "ricevi":
        return Ricevi(sessiondb, session, request).doTask()
    elif request.arguments['task'][0] == "invia":
        return Invia(sessiondb, session, request).doTask()


if __name__ == '__main__':

    from waonline.wademon import DatabaseManager
    from waonline.core.users import UsersManager, NewUtente

    #: Creaiamo subito il DatabaseManager e l'UsersManager
    dbm = DatabaseManager()
    user_manager = UsersManager(dbm)

    class Request(dict):
        def __init__(self, *args):
            self.arguments = dict(*args)
            self.db_manager = None
            self.files = []
            self.headers = []
            self.path = []
            self.request = None
            self.utenti_manager = user_manager
            self.user = None
        def save(self):pass
    # sys.argv[1]:
    #   - "STD": ritorna la lista dei codici utenti "studio" nel file specificato in sys.argv[2]
    #   - codice studio: riceve le anagrafiche/tabelle inviate dallo studio o dal Service EssePaghe
    # sys.argv[2]:
    #   - nome del file con lista codici utenti "studio" se sys.argv[1] = "STD"
    #   - nome del file zip se sys.argv[1] != "STD"
    # sys.argv[3]: service (EPV, CPA, CPS, INAZ), usato solo per la migrazione da WA
    try:
        _esterna = True
        par_1 = par_2 = par_3 = None
        try:
            par_1 = sys.argv[1].strip()
            par_2 = sys.argv[2].strip()
            par_3 = sys.argv[3].strip()
        except:pass

        sessiondb = None
        dbm.get_db(None)
        if not dbm.db_exists(par_1):
            dbm.db_create(par_1)
        sessiondb = dbm.get_sessiondb(par_1)
        # lista studi
        if par_1 == 'STD':
            try:
                user = user_manager.get_by_username("esseadmin")
                with open('/tmp/%s' %par_2,'wb') as f:
                    for studio in user._q(NewUtente,
                            NewUtente.livello == NewUtente.USTUDIO).all():
                        f.write('%s;%s\n' %(u.codice, u.descrizione))
            except:pass
            sys.exit(0)

        # ricezione dati
        utente = user_manager.get_by_username('STUDIO%s' % par_1)
        if not utente and par_3:
            # se non trova l'utente, vuol dire che si tratta di una migrazione dal vecchio WA
            # crea l'utente su db WA
            par_3 = sys.argv[3].strip()
            #: utilizzo nuovo sistema utenti
            utente = user_manager.create_user("STUDIO%s" % par_1, #: username
                    'pwd_tmp', #: password temporanea
                    genitore_id=user_manager.get_by_username(par_3).id,
                    livello=NewUtente.USTUDIO, codice=par_1)

            # crea la struttura di cartelle sul server
            path_studio = '%s/%s' %(conf.DEFAULT_DATA_PATH, par_1)
            chk_make_dir(path_studio)
            chk_make_dir(path_studio + '/invia')
            chk_make_dir(path_studio + '/ricevi')

        session = {'user': utente.id}
        path_zip = "%s/%s/ricevi" %(conf.DEFAULT_DATA_PATH, par_1)
        perc_zip = "%s/%s" %(path_zip, par_2)
        zipfile.ZipFile(perc_zip).extractall(path=path_zip)
        os.unlink(perc_zip)
        request = Request({'nome_file': par_2.replace(".zip", ".xml")})
        request.user = utente
        IR = Ricevi(sessiondb, session, request)
        IR.ricevi()
        os.unlink(IR.perc_file)
        sys.exit(0)

    except Exception, exc:
        print "ERRORE", exc
        LogErrori().errore("Errore di importazione:\n\n%s" %(exc))
    finally:
        if sessiondb:
            sessiondb.close()

