# -*- coding: utf-8 -*-
#
# File: menu.py
# Author: Marco Dalla Stella
# Date: 04/06/2010
from xml.etree.cElementTree import tostring, Element, SubElement

from waonline.apps.gestione.tabelle.menu import Menu
from waonline.core.rbase import RBase


class GestioneMenu(RBase):
    """
        Classe per la gestione dei menu
    """
    def _post_init_(self):
        self._task = 'menu'
        self._tabella_autorizzazioni = {'AA': ['auto_anagrafiche',],
                                        'AP': ['auto_presenze',],
                                        'SG': ['is_segreteria',],
                                        'SP': ['is_sped_pres', 'is_sped_anag'],
                                        'RI': ['is_ricezione',]}

    def menu(self):
        query = self.user._q(Menu)
        menu_root = query.order_by(Menu.id).first()

        root = Element('menus')
        for menu_item in menu_root.scorri_menu():
            if not menu_item.id_padre:
                continue

            if self.user.livello < menu_item.min_livello_utente:
                continue

            if not self._controlla_autorizzazioni_utente(menu_item):
                continue

            menu = SubElement(root, 'menu')
            SubElement(menu, 'id').text = str(menu_item.id)
            SubElement(menu, 'genitore_id').text = str(menu_item.id_padre)
            SubElement(menu, 'nome').text = menu_item.nome
            SubElement(menu, 'azione').text = menu_item.azione_js

        self.risposta.contenuto = tostring(root)
        return self.risposta

    def _controlla_autorizzazioni_utente(self, menu_item):
        if not menu_item.autorizzazioni_richieste:
            return True
        # Controllo le autorizzazioni richieste, se ci sono.
        autorizzazioni_richieste = menu_item.autorizzazioni_richieste.split(',')
        autorizzato = 0
        for autorizzazione in autorizzazioni_richieste:
            campi_autorizzazione = self._tabella_autorizzazioni[autorizzazione]
            for campo_autorizzazione in campi_autorizzazione:
                if getattr(self.user, campo_autorizzazione):
                    autorizzato += 1
        return autorizzato > 0

def dogentask(sessiondb, session, request):
    return GestioneMenu(sessiondb,session,request).doTask()
