# coding: latin-1

from sqlalchemy.orm import relationship

from waonline.core.tabelle import *

class Aliquota(Base):
    __tablename__ = "aliquote"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(3))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))
    gruppo = Column(String(2))
    # Relazioni one-to-many
    qualifiche_rel = relationship('AliquotaSub', order_by='AliquotaSub.id')


class AliquotaSub(Base):
    __tablename__ = "aliquote_sub"
    __table_args__ = (UniqueConstraint('aliquota_id', 'sub_qual'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    aliquota_id = Column(Integer, ForeignKey('aliquote.id', onupdate='CASCADE', ondelete='CASCADE'))
    sub_qual = Column(String(4))
    sub_tipo_rapp = Column(String(2))
    sub_rif_cod_aliq = Column(String(3))
    # Relazioni many-to-one
    aliquota_rel = relationship('Aliquota', order_by='Aliquota.id')


class CassaEdile(Base):
    __tablename__ = "casse_edili"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(4))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class Causale(Base):
    __tablename__ = "causali"
    __table_args__ = (UniqueConstraint('codice', 'valida_per', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(4))
    valida_per = Column(String(1))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    tipo = Column(String(1))
    descrizione = Column(String(100))
    is_no_calendario = Column(Boolean)
    is_val_inizio_data = Column(Boolean)
    is_val_fine_data = Column(Boolean)
    condizione_voce = Column(String(5))
    is_unica_giorno = Column(Boolean)
    is_giorno_completo = Column(Boolean)
    valida_in_giorni = Column(String(1))
    is_no_gestione = Column(Boolean)
    is_strao_magg = Column(Boolean)
    is_ass_no_retrib = Column(Boolean)
    is_ore_ordinarie = Column(Boolean)


class Contratto(Base):
    __tablename__ = "contratti"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(3))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))
    gruppo = Column(String(2))
    contr_inps = Column(String(4))


class ContrattoApprend(Base):
    __tablename__ = "contratti_apprend"
    __table_args__ = (UniqueConstraint('contratto', 'codice', 'sudd_contr', 'sudd_terr', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    contratto = Column(String(3))
    codice = Column(String(4))
    sudd_contr = Column(String(2))
    sudd_terr = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))
    data_inizio = Column(Integer)
    data_fine = Column(Integer)
    data_scadenza = Column(Integer)
    livello_rif = Column(String(4))
    is_distinz_eta = Column(Boolean)
    livello_inquad = Column(String(4))


class ContrattoFest(Base):
    __tablename__ = "contratti_fest"
    __table_args__ = (UniqueConstraint('contratto', 'sudd_contr', 'sudd_terr', 'periodo_inizio', 'aaaa', 'mmgg'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    contratto = Column(String(3))
    sudd_contr = Column(String(2))
    sudd_terr = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    aaaa = Column(String(4))
    mmgg = Column(String(4))
    tipo = Column(String(1))
    descrizione = Column(String(100))


class ContrattoLivello(Base):
    __tablename__ = "contratti_livelli"
    __table_args__ = (UniqueConstraint('contratto', 'codice', 'sudd_contr', 'sudd_terr', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    contratto = Column(String(3))
    codice = Column(String(4))
    sudd_contr = Column(String(2))
    sudd_terr = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class ContrattoOrario(Base):
    __tablename__ = "contratti_orari"
    __table_args__ = (UniqueConstraint('contratto', 'qualifica', 'sudd_contr', 'sudd_terr', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    contratto = Column(String(3))
    qualifica = Column(String(4))
    sudd_contr = Column(String(2))
    sudd_terr = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    div_orario = Column(Numeric(precision=5, scale=2))
    div_giorno = Column(Numeric(precision=5, scale=2))
    div_straord = Column(Numeric(precision=5, scale=2))
    div_assenze = Column(Numeric(precision=5, scale=2))
    cod_assenze = Column(String(3))
    rif_div_var = Column(String(5))
    tot_ore_teo = Column(Numeric(precision=4, scale=2))
    tot_ore_retr = Column(Numeric(precision=4, scale=2))
    tot_giorni_teo = Column(Numeric(precision=4, scale=2))
    tot_giorni_retr = Column(Numeric(precision=4, scale=2))
    orario_part = Column(String(1))
    # Relazioni one-to-many
    settimane_rel = relationship('ContrattoOrarioSett', order_by='ContrattoOrarioSett.num_sett')


class ContrattoOrarioSett(Base):
    __tablename__ = "contratti_orari_sett"
    __table_args__ = (UniqueConstraint('orario_id', 'num_sett'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    orario_id = Column(Integer, ForeignKey('contratti_orari.id', onupdate='CASCADE', ondelete='CASCADE'))
    num_sett = Column(Integer)
    # Relazioni many-to-one
    div_orari_rel = relationship('ContrattoOrario', order_by='ContrattoOrario.id')
    # Relazioni one-to-many
    giorni_rel = relationship('ContrattoOrarioSettGG', order_by='ContrattoOrarioSettGG.giorno')


class ContrattoOrarioSettGG(Base):
    __tablename__ = "contratti_orari_sett_gg"
    id = Column(Integer, primary_key=True, autoincrement=True)
    orario_sett_id = Column(Integer, ForeignKey('contratti_orari_sett.id', onupdate='CASCADE', ondelete='CASCADE'))
    giorno = Column(Integer)
    ore_teoriche = Column(Numeric(precision=4, scale=2))
    ore_retribuite = Column(Numeric(precision=4, scale=2))
    is_fest_ng = Column(Boolean)
    cod_causale_1 = Column(String(4))
    ore_causale_1 = Column(Numeric(precision=4, scale=2))
    cod_causale_2 = Column(String(4))
    ore_causale_2 = Column(Numeric(precision=4, scale=2))
    cod_causale_3 = Column(String(4))
    ore_causale_3 = Column(Numeric(precision=4, scale=2))
    cod_causale_4 = Column(String(4))
    ore_causale_4 = Column(Numeric(precision=4, scale=2))
    cod_causale_5 = Column(String(4))
    ore_causale_5 = Column(Numeric(precision=4, scale=2))
    # Relazioni many-to-one
    div_orari_sett_rel = relationship('ContrattoOrarioSett', order_by='ContrattoOrarioSett.num_sett')


class ContrattoQual(Base):
    __tablename__ = "contratti_qual"
    __table_args__ = (UniqueConstraint('contratto_id', 'codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    contratto_id = Column(Integer, ForeignKey('contratti.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(4))


class ContrattoStraMagg(Base):
    __tablename__ = "contratti_stra_magg"
    __table_args__ = (UniqueConstraint('contratto', 'qualifica', 'sudd_contr', 'sudd_terr', 'cod_straord', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    contratto = Column(String(3))
    qualifica = Column(String(4))
    sudd_contr = Column(String(2))
    sudd_terr = Column(String(2))
    cod_straord = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)


class ContrattoSuddContr(Base):
    __tablename__ = "contratti_sudd_contr"
    __table_args__ = (UniqueConstraint('contratto', 'codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    contratto = Column(String(3))
    codice = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class ContrattoSuddTerr(Base):
    __tablename__ = "contratti_sudd_terr"
    __table_args__ = (UniqueConstraint('contratto', 'codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    contratto = Column(String(3))
    codice = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class Voce(Base):
    __tablename__ = "voci"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(4))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))
    gruppo = Column(String(2))
    com_movimenti = Column(String(1))
