from waonline.core.tabelle import *

class ANF(Base):
    __tablename__ = "anf"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(3))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class CampoWA(Base):
    __tablename__ = "campi_wa"
    __table_args__ = (UniqueConstraint('codice'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(5))
    descrizione = Column(String(100))
    tipo_archivio = Column(String(1))
    cod_commento = Column(String(1))
    is_obbligatorio = Column(Boolean)
    is_solo_pres = Column(Boolean)
    funzione = Column(String(2))
    formula = Column(String(150))
    formato = Column(String(2))
    num_caratteri = Column(Integer)
    annotazione = Column(String(500))
    campo_rif = Column(String(50))
    classe_rif = Column(String(50))


class CausaConguaglio(Base):
    __tablename__ = "cause_conguagli"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(1))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class CollAttivita(Base):
    __tablename__ = "coll_attivita"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class CollFormaAssicura(Base):
    __tablename__ = "coll_forme_assicura"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(3))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class CollRapporto(Base):
    __tablename__ = "coll_rapporti"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class Costante(Base):
    __tablename__ = "costanti"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(5))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))
    # Relazioni one-to-many
    campi_rel = relationship('CostanteCampo', order_by='CostanteCampo.id')


class CostanteCampo(Base):
    __tablename__ = "costanti_campi"
    id = Column(Integer, primary_key=True, autoincrement=True)
    costante_id = Column(Integer, ForeignKey('costanti.id', onupdate='CASCADE', ondelete='CASCADE'))
    des_campo = Column(String(50))
    val_campo = Column(Numeric(precision=12, scale=2))
    # Relazioni many-to-one
    costante_rel = relationship('Costante', order_by='Costante.id')


class GruppoAliquota(Base):
    __tablename__ = "gruppi_aliquote"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class GruppoContratto(Base):
    __tablename__ = "gruppi_contratti"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class GruppoVoce(Base):
    __tablename__ = "gruppi_voci"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class InailPosizione(Base):
    __tablename__ = "inail_posizioni"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(1))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class InailSede(Base):
    __tablename__ = "inail_sedi"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(5))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))


class Qualifica(Base):
    __tablename__ = "qualifiche"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(4))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))
    contr_estensione = Column(String(1))
    quarto_dm10 = Column(String(1))
    # Relazioni one-to-many
    tipo_rapp_rel = relationship('QualificaRapp', order_by='QualificaRapp.id')


class QualificaRapp(Base):
    __tablename__ = "qualifiche_rapp"
    __table_args__ = (UniqueConstraint('qualifica_id', 'tipo_rapp'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    qualifica_id = Column(Integer, ForeignKey('qualifiche.id', onupdate='CASCADE', ondelete='CASCADE'))
    tipo_rapp = Column(String(2))
    # Relazioni many-to-one
    qualifica_rel = relationship('Qualifica', order_by='Qualifica.id')


class Rapporto(Base):
    __tablename__ = "rapporti"
    __table_args__ = (UniqueConstraint('codice', 'periodo_inizio'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    codice = Column(String(2))
    periodo_inizio = Column(Integer)
    periodo_fine = Column(Integer)
    descrizione = Column(String(100))
