# coding: latin-1
from datetime           import date

from sqlalchemy.sql     import and_, desc

from waonline.core.rbase     import RBase
from waonline.utils.utility   import converti_ore, formatData, framData, getGiorniMese
from waonline.apps.anagrafiche.tabelle.dipendenti import Dipendente
from waonline.apps.presenze.tabelle.presenze   import Presenze
from waonline.apps.gestione.tabelle.tabelle_c  import Causale, ContrattoFest, ContrattoOrario

GG_SETT = ['LU', 'MA', 'ME', 'GI', 'VE', 'SA', 'DO']

class Assemblaggio(RBase):

    def _post_init_(self):
        self.calend = []
        self.anag = {}
        self.ditta = None
        self.dipte = None
        self.presMP = None
        self.festContr = None
        self.orario = None
        self.ultimaSett = 1
        self.eventoPrec = None
        self.eventi = []


    def assemblaMese(self, periodo, dipte_id):
        self._caricaDati(periodo, dipte_id)
        self._assCalendario(periodo)
        self._assAnagrafica()


    def _caricaDati(self, periodo, dipte_id):
        self.dipte = self._q(Dipendente).filter(Dipendente.id == dipte_id).first()
        self.ditta = self.dipte.ditta_rel
        if self.dipte.is_collab:
            return
        if framData(periodo)[1] == 1:
            periodoMP = ((framData(periodo)[0] - 1) * 100) + 12
        else:
            periodoMP = periodo - 1
        self.presMP = self._q(Presenze).filter(and_(Presenze.periodo == periodoMP,
                                                    Presenze.ditta_id == self.ditta.id,
                                                    Presenze.dipendente_id == self.dipte.id,
                                                    )).first()
        # assembla codice contratto, sudd. contrattuale e territoriale
        codContr = self.dipte.qualifica.contratto or self.ditta.contratto
        suddContr = self.dipte.qualifica.sudd_contr or self.ditta.sudd_contr
        suddTerr = self.dipte.qualifica.sudd_terr or self.ditta.sudd_terr
        # assemla orario...
        if self.dipte.div_orari_rel:
            # ...dipendente, se ci sono settimane
            if self.dipte.div_orari_rel[0].settimane_rel:
                self.orario = self.dipte.div_orari_rel[0]
        if not self.orario and self.ditta.div_orari_rel:
            # ...ditta, se ci sono settimane
            for orario in self.ditta.div_orari_rel:
                if not orario.settimane_rel:
                    continue
                if orario.qualifica == self.dipte.alt_divisori:
                    self.orario = orario
                    break
                if (orario.qualifica[0:1] in ['*', self.dipte.qualifica.codice[0:1]]) and \
                   (orario.qualifica[1:2] in ['*', self.dipte.qualifica.codice[1:2]]) and \
                   (orario.qualifica[2:3] in ['*', self.dipte.qualifica.codice[2:3]]) and \
                   (orario.qualifica[3:4] in ['*', self.dipte.qualifica.codice[3:4]]):
                    self.orario = orario
                    break
        # ...contratto
        if not self.orario:
            orari = self._q(ContrattoOrario).filter(and_(ContrattoOrario.contratto == codContr,
                                                         ContrattoOrario.periodo_inizio <= periodo,
                                                         ContrattoOrario.periodo_fine >= periodo,
                                                         )).order_by(desc(ContrattoOrario.periodo_inizio)).all()
            # se non trova il contratto con il periodo del mese presenze, prende quello valido ad oggi
            if not orari:
                periodo_att = int(self.session['periodo_attuale'].strftime("%Y%m"))
                orari = self._q(ContrattoOrario).filter(and_(ContrattoOrario.contratto == codContr,
                                                             ContrattoOrario.periodo_inizio <= periodo_att,
                                                             ContrattoOrario.periodo_fine >= periodo_att,
                                                             )).order_by(desc(ContrattoOrario.periodo_inizio))
            for orario in orari:
                if (orario.qualifica[0:1] in ['*', self.dipte.qualifica.codice[0:1]]) and \
                   (orario.qualifica[1:2] in ['*', self.dipte.qualifica.codice[1:2]]) and \
                   (orario.qualifica[2:3] in ['*', self.dipte.qualifica.codice[2:3]]) and \
                   (orario.qualifica[3:4] in ['*', self.dipte.qualifica.codice[3:4]]):
                    if (not orario.sudd_contr or orario.sudd_contr == suddContr) and \
                       (not orario.sudd_terr  or orario.sudd_terr  == suddTerr):
                        self.orario = orario
                        break

        # festivita' contratto
        self.festContr = self._q(ContrattoFest).filter(and_(ContrattoFest.contratto == codContr,
                                                            ContrattoFest.periodo_inizio <= periodo,
                                                            ContrattoFest.periodo_fine >= periodo,
                                                            )).order_by(desc(ContrattoFest.periodo_inizio))

    def _assCalendario(self, periodo):
        self.calend = []
        anno = framData(periodo)[0]
        mese = framData(periodo)[1]
        # settimana inizio ciclo
        sett = 1
        if not self.dipte.is_collab and self.orario and len(self.orario.settimane_rel) > 1 and self.presMP:
            sett = self.presMP.ultima_sett
            if date(anno, mese, 1).isoweekday() == 1: # Lunedi'
                sett += 1
            if sett > len(self.orario.settimane_rel):
                sett = 1
        # carica calendario mese:
        ggSett = date(anno, mese, 1).isoweekday()
        for gg in range(getGiorniMese(periodo)):
            giorno = gg + 1
            dizGG = {}
            # - giorno mese
            dizGG['GM'] = giorno
            # - giorno settimana
            dizGG['GS'] = GG_SETT[date(anno, mese, giorno).isoweekday() - 1]
            # - cantiere/centro di costo
            dizGG['cant_cc'] = None
            # carica il calendario vuoto se è un collaboratore o se nessun orario è stato assemblato
            if self.dipte.is_collab or not self.orario:
                dizGG['OT'] = float(0)
                dizGG['OL'] = float(0)
                dizGG['OR'] = float(0)
                self.calend.append(dizGG)
                continue
            orarioGG = None
            for s in self.orario.settimane_rel:
                if s.num_sett == sett:
                    for g in s.giorni_rel:
                        if g.giorno == ggSett:
                            orarioGG = g
                            break
                    break
            if not orarioGG:
                dizGG['OT'] = float(0)
                dizGG['OL'] = float(0)
                dizGG['OR'] = float(0)
                self.calend.append(dizGG)
                continue

            # - ore teoricche...
            codTeo = "OT"
            #...verifica festivita' contratto
            if self.festContr:
                for fest in self.festContr:
                    ris = self._checkFG(fest, anno, mese, giorno, orarioGG.ore_teoriche)
                    if ris:
                        codTeo = ris
                        break
            #...verifica festivita' ditta
            if self.ditta.festivita_rel:
                for fest in self.ditta.festivita_rel[0].festivita_rel:
                    ris = self._checkFG(fest, anno, mese, giorno, orarioGG.ore_teoriche)
                    if ris:
                        codTeo = ris
                        break
            # ...verifica festivita' dipendente
            if self.dipte.festivita_rel:
                for fest in self.dipte.festivita_rel[0].festivita_rel:
                    ris = self._checkFG(fest, anno, mese, giorno, orarioGG.ore_teoriche)
                    if ris:
                        codTeo = ris
                        break

            if orarioGG.ore_teoriche:
                if codTeo == "FG":
                    if int(orarioGG.ore_teoriche) in [50, 60]:
                        dizGG[codTeo] = float(0)
                    else:
                        dizGG[codTeo] = self._formatta_ore(orarioGG.ore_teoriche)
                else:
                    if int(orarioGG.ore_teoriche) == 50:
                        dizGG[codTeo] = "RS"
                    elif int(orarioGG.ore_teoriche) == 60:
                        dizGG[codTeo] = "--"
                    else:
                        dizGG[codTeo] = self._formatta_ore(orarioGG.ore_teoriche)
            else:
                dizGG[codTeo] = "--"
            # - ore retribuite
            dizGG['OR'] = self._formatta_ore(orarioGG.ore_retribuite)
            # - causali predefinite
            if orarioGG.cod_causale_1:
                dizGG['C1'] = orarioGG.cod_causale_1
                dizGG['O1'] = self._formatta_ore(orarioGG.ore_causale_1)
            if orarioGG.cod_causale_2:
                dizGG['C2'] = orarioGG.cod_causale_2
                dizGG['O2'] = self._formatta_ore(orarioGG.ore_causale_2)
            if orarioGG.cod_causale_3:
                dizGG['C3'] = orarioGG.cod_causale_3
                dizGG['O3'] = self._formatta_ore(orarioGG.ore_causale_3)
            if orarioGG.cod_causale_4:
                dizGG['C4'] = orarioGG.cod_causale_4
                dizGG['O4'] = self._formatta_ore(orarioGG.ore_causale_4)
            if orarioGG.cod_causale_5:
                dizGG['C5'] = orarioGG.cod_causale_5
                dizGG['O5'] = self._formatta_ore(orarioGG.ore_causale_5)
            # incremento giorni settimana / settimane
            self.ultimaSett = sett
            if ggSett == 7:
                ggSett = 1
                if sett == len(self.orario.settimane_rel):
                    sett = 1
                else:
                    sett += 1
            else:
                ggSett += 1
            # aggiunge il giorno al calendario
            self.calend.append(dizGG)

        if not self.dipte.is_collab:
            self._caricaEventiMP(periodo)
            self._calcolaOreLav(periodo)


    def _caricaEventiMP(self, periodo):
        if not self.presMP:
            return
        for evento in self.presMP.eventi_rel:
            causale = self._q(Causale).filter(and_(Causale.codice == evento.codice,
                                                   Causale.periodo_inizio <= periodo,
                                                   Causale.periodo_fine >= periodo,
                                                   )).first()
            # salta gli eventi che non cadono nel periodo corrente
            if formatData(evento.data_inizio, "int", "%Y%m") > periodo or \
               formatData(evento.data_fine, "int", "%Y%m")   < periodo:
                continue
            # imposta giorni inizio e fine
            if formatData(evento.data_inizio, "int", "%Y%m") == periodo:
                ggI = int(str(evento.data_inizio)[6:8])
            else:
                ggI = 1
            if formatData(evento.data_fine, "int", "%Y%m") == periodo:
                ggF = int(str(evento.data_fine)[6:8])
            else:
                ggF = getGiorniMese(periodo)
            self.eventoPrec = evento.codice
            self.eventi.append(evento)
            # carica l'evento nel calendario per il periodo interessato
            for gg in range(ggI, ggF + 1):
                giorno = gg - 1
                # se la causale evento non e' di tipo "valida sempre", la inserisce solo nei
                # giorni in cui e' previsto l'orario teorico
                if causale.valida_in_giorni != "S" and (not "OT" in self.calend[giorno] or self.calend[giorno]['OT'] == 0):
                    continue
                for caus in range(5):
                    # inserisce l'evento giornaliero nella prima causale libera
                    if not ("C" + str(caus + 1)) in self.calend[giorno]:
                        self.calend[giorno]['C' + str(caus + 1)] = evento.codice
                        ore = float(0)
                        # se evento senza ore oppure evento "giorno completo", inserisce le ore
                        # teoriche del giorno
                        if not evento.ore or causale.is_giorno_completo:
                            if "OT" in self.calend[giorno]        and \
                                self.calend[giorno]['OT'] != "RS" and \
                                self.calend[giorno]['OT'] != "--":
                                ore = self.calend[giorno]['OT']
                        # se ricovero ospedaliero oppure assenza in un giorno festivo, le ore
                        # non vanno inserite
                        if evento.codice == "MR" or \
                          (causale.tipo == "A" and "FG" in self.calend[giorno]):
                            ore = float(0)
                        self.calend[giorno]['O' + str(caus + 1)] = ore
                        break


    def _calcolaOreLav(self, periodo):
        for giorno in range(getGiorniMese(periodo)):
            if "FG" in self.calend[giorno] or self.calend[giorno]['OT'] in ['RS', '--']:
                ore_lav = float(0)
            else:
                ore_lav = float(self.calend[giorno]['OT'])
                for num_cau in range(5):
                    cod_cau = "C" + str(num_cau + 1)
                    ore_cau = "O" + str(num_cau + 1)
                    if cod_cau in self.calend[giorno]:
                        try:
                            tipo = self._q(Causale.tipo).filter(and_(Causale.codice == self.calend[giorno][cod_cau],
                                                                     Causale.periodo_inizio <= periodo,
                                                                     Causale.periodo_fine >= periodo,
                                                                     )).first()[0]
                        except: continue
                        if tipo == "A":
                            if self._impostazioni.is_sessantesimi:
                                # 60 > 100
                                ore_lav = converti_ore(ore_lav, False) - converti_ore(self.calend[giorno][ore_cau], False)
                            else:
                                ore_lav -= float(self.calend[giorno][ore_cau])
                            if self._impostazioni.is_sessantesimi:
                                # 100 > 60
                                ore_lav =  float(converti_ore(ore_lav, True))

            self.calend[giorno]['OL'] = ore_lav


    def _assAnagrafica(self):
        self.anag['qualifica'] = self.dipte.qualifica.codice
        self.anag['livello'] = self.dipte.qualifica.livello
        self.anag['centro_costo'] = None
        self.anag['pos_inail'] = None
        if self.dipte.qualifica.num_pos_inail:
            self.anag['pos_inail'] = self.dipte.qualifica.num_pos_inail
            if self.dipte.qualifica.cc_pos_inail:
                self.anag['pos_inail'] += self.dipte.qualifica.cc_pos_inail
        else:
            if self.dipte.qualifica.rif_inail:
                try:
                    posInail = [p for p in self.ditta.pos_inail_rel if p.cod_pos == int(self.dipte.qualifica.rif_inail[0:2])][0]
                    if posInail:
                        self.anag['pos_inail'] = posInail.num_pos + posInail.cc_pos
                except:
                    self.anag['pos_inail'] = None
        self.anag['tipo_pm'] = self.dipte.qualifica.part_time
        self.anag['perc_pm'] = self.dipte.qualifica.perc_part_time
        self.anag['mansione'] = self.dipte.qualifica.mansione
        self.anag['ultima_sett'] = self.ultimaSett
        self.anag['evento_mp'] = self.eventoPrec
        if self.orario and self.orario.tot_ore_teo:
            self.anag['ore_sett'] = self._formatta_ore(self.orario.tot_ore_teo)
        else:
            self.anag['ore_sett'] = float(0)


    def _checkFG(self, fest, anno, mese, giorno, ore_teoriche):
        if fest.tipo == "G"                             and \
          (fest.aaaa == None or int(fest.aaaa) == anno) and \
           int(fest.mmgg[0:2]) == mese                  and \
           int(fest.mmgg[2:4]) == giorno                and \
           ore_teoriche > 0:
            if isinstance(fest, ContrattoFest):
                return "FG"
            else:
                if fest.is_ignora:
                    return "OT"
                else:
                    return "FG"
        return None


    def _stampaCalend(self):
        for g in self.calend:
            gg = "GM:" + str(g['GM']).zfill(2) + "  GS:" + str(g['GS'])
            if "OT" in g: gg += "  OT:" + str(g['OT'])
            if "FG" in g: gg += "  FG:" + str(g['FG'])
            gg += "  OR:" + str(g['OR']) + "  OL:" + str(g['OL'])
            if "C1" in g: gg += "  C1:" + str(g['C1']) + '  O1:' + str(g['O1'])
            if "C2" in g: gg += "  C2:" + str(g['C2']) + '  O2:' + str(g['O2'])
            if "C3" in g: gg += "  C3:" + str(g['C3']) + '  O3:' + str(g['O3'])
            if "C4" in g: gg += "  C4:" + str(g['C4']) + '  O4:' + str(g['O4'])
            if "C5" in g: gg += "  C5:" + str(g['C5']) + '  O5:' + str(g['O5'])


    def _formatta_ore(self, ore):
        if not ore:
            ore = float(0)
        if ore and self._impostazioni.is_sessantesimi:
            return converti_ore(ore, True)
        else:
            return ore
