# coding: latin-1
import json
from xml.etree import cElementTree as ET
from sqlalchemy.sql     import and_

from .assemblaggio import Assemblaggio
from .calendario import Calendario, CausaleCal, EventoCal,\
        GeneraDatiCal, LavoroDomCal, VoceCal
#from waonline.gestione.gest_valoriLista import GestioneListe
from waonline.apps.gestione.gest_valoriLista import carica_causali_in_session
from waonline.core.rbase    import RBase
from waonline.utils.utility import framData, formatData, isDataOK,\
        isOrarioOK, converti_ore, getGiorniMese, doppieInLista
#from waonline.core.tabelle import *
from waonline.apps.anagrafiche.tabelle.dipendenti import Dipendente
from waonline.apps.anagrafiche.tabelle.ditte import DittaWAStatus,\
        DittaWAStatusPeriodo
from waonline.apps.presenze.tabelle.presenze import Presenze, PresenzeCalend,\
    PresenzeCalendCaus, PresenzeEvento, PresenzeLavDom, PresenzeVoce
from waonline.apps.gestione.tabelle.tabelle_c import Causale, Voce
from waonline.core import sessions

MAX_CAUSALI = 5


class GestionePresenze(RBase):

    def _post_init_(self):
        self.cella_id = 0
        self.calendario = None
        if 'calendario' in self.session:
            self.calendario = self.session['calendario']
        if self._preferenze.AUTOCOMP_ORE_ASS == "S":
            self.is_auto_comp = True
        else:
            self.is_auto_comp = False
        try:
            self.causali_db = self.session['causali_db']
        except:
            self.causali_db = carica_causali_in_session(self)

    """
        G E S T I O N E  -  S T A T U S
    """
    def apri_mese(self):
        self._add_to_log()
        if self._ditta.status_wa_rel:
            status = self._ditta.status_wa_rel[0]
        else:
            status = DittaWAStatus()
            status.ditta_id = self._ditta.id
            self._ditta.status_wa_rel.append(status)
            self.sessiondb.commit()

        root = ET.Element("apriMese")
        if self._periodo:
            # L'utente ha confermato l'apertura di un nuovo mese
            if [p for p in status.status_periodi_rel if p.periodo == self._periodo]:
                self._addErr("Periodo gia' presente: %s" % formatData(self._periodo, "str", "%m/%Y"))
            else:
                # crea il nuovo periodo
                newPeriodo = DittaWAStatusPeriodo()
                newPeriodo.status_id = status.id
                newPeriodo.periodo = self._periodo
                newPeriodo.stato = "A"
                newPeriodo.is_inviato = False
                status.status_periodi_rel.append(newPeriodo)
                self.sessiondb.commit()
                # esegue assemblaggio presenze per tutti i dipendenti in forza
                dipti = self._q(Dipendente).filter(Dipendente.ditta_id == self._ditta.id).order_by(Dipendente.codice)
                for dipte in dipti:
                    #if formatData(dipte.data_ass,  "int", "%Y%m") > self._periodo or \
                    #  (formatData(dipte.data_cess, "int", "%Y%m") < self._periodo and dipte.data_cess > 0):
                    #    continue
                    pres_ass = self._assembla_presenze(dipte)
                    if pres_ass:
                        self.sessiondb.add(pres_ass)
                        self.sessiondb.commit()
                self._addStato("Periodo aperto: %s" %(formatData(self._periodo, "str", "%m/%Y")), 5000)
        else:
            # L'utente chiede di aprire un nuovo mese
            if len(status.status_periodi_rel) > 0:
                periodo = status.status_periodi_rel[-1].periodo
                if framData(periodo)[1] == 12:
                    periodo = ((framData(periodo)[0] + 1) * 100) + 1
                else:
                    periodo += 1
            else:
                periodo = str(self.session['periodo_attuale'])[0:10]
                periodo = periodo.split("-")[0] + periodo.split("-")[1] + periodo.split("-")[2]
            ET.SubElement(root, "periodo").text = formatData(periodo, "str", "%m/%Y")

        self._repHead("Content-Type","text/xml")
        self.risposta.contenuto = ET.tostring(root)


    def chiudi_mese(self):
        self._add_to_log()
        status = self._ditta.status_wa_rel[0]
        root = ET.Element("chiudiMese")
        try:
            periodo = [p for p in status.status_periodi_rel if p.periodo == self._periodo][0]
            if periodo.stato == "C":
                self._addErr("Periodo gia' chiuso: %s" % formatData(self._periodo, "str", "%m/%Y"))
            else:
                if not self._verifica_calendario_ok():
                    return
                periodo.stato = "C"
                self.sessiondb.commit()
            self._addStato("Periodo chiuso: %s" %(formatData(self._periodo, "str", "%m/%Y")), 5000)
        except:
            self._addErr("Periodo non presente: %s" % formatData(self._periodo, "str", "%m/%Y"))

        self._repHead("Content-Type","text/xml")
        self.risposta.contenuto = ET.tostring(root)


    def cambia_stato(self):
        self._add_to_log()
        status = self._ditta.status_wa_rel[0]
        stato_ga = self.ga("statoStatus")
        inviato_ga = self.ga("inviatoStatus")
        periodo_ga = formatData(self.ga("periodoStatus"), "int", "%Y%m")
        if stato_ga:
            stato_ga = "C" if stato_ga == "Chiuso" else "A"
        try:
            periodo = [p for p in status.status_periodi_rel if p.periodo == periodo_ga][0]
            periodo.stato = stato_ga
            periodo.is_inviato = (inviato_ga == "S")
            self.sessiondb.commit()
        except:pass

        self._addStato("Status aggiornato: %s" %(formatData(periodo_ga, "str", "%m/%Y")), 5000)


    """
        G E S T I O N E  -  C A L E N D A R I O
    """
    @sessions.require_lock
    def carica_calendario(self):
        if self.calendario:
            calendario = self.calendario
            # aggiunge in sessione eventuali dipendenti non presenti
            dipti_id = [id for id in self._dipti_id if id not in calendario.id_dipendenti]
            if dipti_id:
                presenze = self._carica_presenze(dipti_id)
                for pres in presenze:
                    calendario.aggiungi_presenze_dipe(pres)
            # elimina dalla sessione eventuali dipendenti non richiesti
            dipti_id = [id for id in calendario.id_dipendenti if id not in self._dipti_id]
            for dipte_id in dipti_id:
                if dipte_id in calendario.presenze:
                    calendario.presenze.pop(dipte_id)
        else:
            causali_no_calendario = [k for k, v in self.causali_db.items() if v['is_no_calendario'] == True]
            presenze = self._carica_presenze(self._dipti_id)
            # carica il calendario
            calendario = Calendario(self._ditta.id, self._periodo, causali_no_calendario)
            for pres in presenze:
                calendario.aggiungi_presenze_dipe(pres)
            self.session['salva_calendario'] = False

        self.calendario = calendario
        self.session['calendario'] = calendario

        self.risposta.contenuto = GeneraDatiCal(self, calendario).genera_calendario()
        self.session.to_save = True


    def _carica_presenze(self, dipti_id):
        lpresenze = self._q(Presenze).filter(and_(Presenze.periodo == self._periodo,
                                                  Presenze.ditta_id == self._ditta.id,
                                                  Presenze.dipendente_id.in_(dipti_id)))
        lpresenze = lpresenze.all()
        presenze = []
        for id in dipti_id:
            nl = [l for l in lpresenze if l.dipendente_id == id]
            if nl:
                presenze.append(nl[0])
        id_caricati = [p.dipendente_id for p in presenze]
        id_mancanti = [id for id in dipti_id if id not in id_caricati]
        for dipte_id in id_mancanti:
            dipte = self._q(Dipendente).filter(Dipendente.id == dipte_id).first()
            if dipte:
                pre = self._assembla_presenze(dipte)
                if pre:
                    self.sessiondb.add(pre)
        self.sessiondb.commit()
        return presenze


    @sessions.require_lock
    def gestione_giorno(self, verifica_mese=False):
        #self.args = {'cod_ditta':'000C036', 'type':'gest_presenze', 'task':'gestione_giorno',
        #             'id_dipte':'715', 'giorno':'1', 'periodo':'201107', 'causale':'2', 'valore':'PR0.25'}
        sw_errore = False
        gg_idx = int(self.ga("giorno")) - 1
        presenze = self.calendario.presenze[self._dipti_id[0]]
        pres_giorno = presenze.giorni[gg_idx]
        riga = int(self.ga("causale"))
        #valore = None
        cod_cau = None
        ore_cau = float(0)
        if self.ga("valore"):
            causale = self._cod_ore_causale(self.ga("valore"))
            cod_cau = causale[0]
            ore_cau = causale[1]
            try:
                ore_cau = float(ore_cau)
            except:
                if riga > 0 or ore_cau not in ['RS', '--']:
                    ore_cau = float(0)
            if presenze.is_collab:
                ore_cau = float(0)

        # - ore teoriche
        if riga == 0:
            if cod_cau == "FG":
                pres_giorno.ecc = cod_cau
                if ore_cau in ['RS', '--']:
                    pres_giorno.ore_teoriche = float(0)
                else:
                    pres_giorno.ore_teoriche = ore_cau
            else:
                if ore_cau in ['RS', '--']:
                    pres_giorno.ecc = ore_cau
                    pres_giorno.ore_teoriche = float(0)
                else:
                    pres_giorno.ecc = None
                    pres_giorno.ore_teoriche = ore_cau

        # - causali
        if riga >= 2 and riga <=6:
            cau_idx = riga - 2
            if cod_cau:
                causale = pres_giorno.ricava_causale_giorno(cau_idx)
                if causale:
                    causale.codice = cod_cau
                    causale.ore = ore_cau
                else:
                    causale = CausaleCal(None, cod_cau, ore_cau)
                    pres_giorno.causali_giorno.append(causale)
                if self._causale_ok(presenze, gg_idx + 1, causale):
                    if self.is_auto_comp and not presenze.is_collab:
                        causale.ore = self._compila_ore(pres_giorno, causale)
                else:
                    sw_errore = True
            else:
                try:
                    pres_giorno.causali_giorno.pop(cau_idx)
                except:pass
        if not sw_errore:
            self._calcola_ore_lav_retr(presenze, gg_idx + 1)

        # Salva il calendario nella sessione utente
        self.session['calendario'] = self.calendario
        self.session['salva_calendario'] = True
        self.session.to_save = True

        # Risposta
        self.risposta.contenuto = GeneraDatiCal(
            self, self.calendario).genera_calendario(
            presenze.id_dipe, pres_giorno.giorno, pres_giorno.giorno)


    @sessions.require_lock
    def gestione_eventi(self, dipte_id=None):
        gg_inizio = None
        gg_fine = None
        gg_inizio_old = None
        gg_fine_old = None
        is_ok = True
        if not self.calendario:
            self.carica_calendario()
        if dipte_id:
            presenze = self.calendario.presenze[dipte_id]
        else:
            presenze = self.calendario.presenze[self._dipti_id[0]]

        # Eliminazione
        if self.ga("stato") == "salva" or self.ga("stato") == "elimina":
            try:
                evento = presenze.eventi[int(self.ga("id_riga"))] # il vecchio evento
                # dal calendario
                if evento.data_inizio:
                    gg_inizio = self._ricava_giorno_data(evento.data_inizio, "i")
                if evento.data_inizio:
                    gg_fine = self._ricava_giorno_data(evento.data_fine, "f")
                for giorno in presenze._ricava_giorni(gg_inizio, gg_fine):
                    for causale in [c for c in giorno.causali_giorno if c.codice == evento.codice]:
                        giorno.causali_giorno.remove(causale)
                # dalla tabella
                if self.ga("stato") == "elimina":
                    presenze.eventi.pop(presenze.eventi.index(evento))

                gg_inizio_old = gg_inizio
                gg_fine_old = gg_fine
                for giorno in range(gg_inizio, gg_fine + 1):
                    self._calcola_ore_lav_retr(presenze, giorno)
            except:
                is_ok = False

        # Modifica/Inserimento
        if self.ga("stato") == "salva":
            causale_db = self._leggi_causale(self.ga("codice_evento").upper().strip())
            # in tabella
            evento = EventoCal()
            evento.codice = self.ga("codice_evento").upper().strip()
            evento.data_inizio = formatData(self.ga("data_inizio"), "int", "%Y%m%d")
            evento.data_fine = formatData(self.ga("data_fine"), "int", "%Y%m%d")
            evento.ore = float(0)
            if self.ga("ore_evento") and evento.codice != "MN" and evento.codice != "MR":
                evento.ore = float(self.ga("ore_evento").replace(",", "."))
            if causale_db:
                if causale_db.is_no_calendario or formatData(evento.data_fine, "int", "%Y%m") > self._periodo:
                    try:
                        # modifica
                        presenze.eventi[int(self.ga("id_riga"))] = evento
                    except:
                        # inserimento
                        presenze.eventi.append(evento)
                else:
                    # sostituzione causale no calendario con causale calendario
                    try:
                        evento_old = presenze.eventi[int(self.ga("id_riga"))]
                        presenze.eventi.pop(presenze.eventi.index(evento_old))
                    except:pass
            else:
                try:
                    # codice errato (modifica)
                    presenze.eventi[int(self.ga("id_riga"))] = evento
                except:
                    # codice errato (inserimento)
                    presenze.eventi.append(evento)

            # sul calendario
            is_ok = self._evento_ok(presenze, evento)
            if is_ok:
                aaaamm_inizio = formatData(evento.data_inizio, "int", "%Y%m")
                aaaamm_fine = formatData(evento.data_fine, "int", "%Y%m")
                gg_inizio = self._ricava_giorno_data(evento.data_inizio, "i")
                gg_fine = self._ricava_giorno_data(evento.data_fine, "f")
                if aaaamm_inizio <= self._periodo and aaaamm_fine >= self._periodo:
                    for giorno in range(gg_inizio, gg_fine + 1):
                        pres_giorno = self.calendario.ricava_giorno_dipe(presenze.id_dipe, giorno)
                        if not self._giorno_in_forza(presenze, giorno):
                            continue
                        if len(pres_giorno.causali_giorno) < MAX_CAUSALI:
                            if causale_db.valida_in_giorni == "A": # solo se OT > 0 e no FG
                                if pres_giorno.ore_teoriche == 0 or pres_giorno.ecc == "FG":
                                    continue
                            if causale_db.valida_in_giorni == "B": # solo se OR > 0 e no FG
                                if pres_giorno.ore_retribuite == 0 or pres_giorno.ecc == "FG":
                                    continue
                            if causale_db.valida_in_giorni == "C": # no FG
                                if pres_giorno.ecc == "FG":
                                    continue
                            if causale_db.valida_in_giorni == "O": # solo se OT > 0
                                if pres_giorno.ore_teoriche == 0:
                                    continue
                            if causale_db.valida_in_giorni == "R": # solo se OR > 0
                                if pres_giorno.ore_retribuite == 0:
                                    continue
                            causale = CausaleCal(None, evento.codice, evento.ore)
                            if evento.codice in ['IN', 'ML'] and evento.ore > 0 and \
                               giorno == gg_inizio and aaaamm_inizio == self._periodo:
                                causale.ore = evento.ore
                            else:
                                causale.ore = self._compila_ore(pres_giorno, causale)
                            if causale_db.is_no_calendario:
                                causale.abilitata = False
                            else:
                                causale.abilitata = True
                            pres_giorno.causali_giorno.append(causale)
                            self._calcola_ore_lav_retr(presenze, giorno)

        # Crea la risposta solo per il singolo dipendente
        if dipte_id:
            self.risposta.contenuto = None
        else:
            if is_ok and gg_inizio and gg_fine and (self.ga("stato") == "salva" or self.ga("stato") == "elimina"):
                if gg_inizio_old:
                    if gg_inizio_old < gg_inizio:
                        gg_inizio = gg_inizio_old
                    if gg_fine_old > gg_fine:
                        gg_fine = gg_fine_old
                self.risposta.contenuto = GeneraDatiCal(self, self.calendario).genera_calendario(presenze.id_dipe,
                                                                                                 gg_inizio,
                                                                                                 gg_fine)
            else:
                self.risposta.contenuto = GeneraDatiCal(self, self.calendario).genera_eventi(presenze.id_dipe)

        # Salva il calendario nella sessione utente
        self.session['calendario'] = self.calendario
        if self.ga("stato"):
            self.session['salva_calendario'] = True
        self.session.to_save = True


    @sessions.require_lock
    def gestione_eventi_multi(self):

        self.carica_calendario()
        for dipte_id in self._dipti_id:
            self.gestione_eventi(dipte_id)
        self.salva_calendario()


    @sessions.require_lock
    def gestione_voci(self, dipte_id=None):
        if not self.calendario:
            self.carica_calendario()
        if dipte_id:
            presenze = self.calendario.presenze[dipte_id]
        else:
            presenze = self.calendario.presenze[self._dipti_id[0]]

        # Eliminazione
        if self.ga("stato") == "elimina":
            try:
                voce = presenze.voci[int(self.ga("id_riga"))]
                presenze.voci.pop(presenze.voci.index(voce))
            except: pass

        # Modifica/Inserimento
        if self.ga("stato") == "salva":
            voce = VoceCal()
            if self.ga("prefisso"):
                voce.prefisso = self.ga("prefisso").upper()
            if self.ga("voce"):
                voce.voce = self.ga("voce").upper().strip().rjust(4, "0")
            if self.ga("suffisso"):
                voce.suffisso = self.ga("suffisso").upper()
            if self.ga("dato_base"):
                voce.dato_base = float(self.ga("dato_base").replace(",", "."))
            if self.ga("costo_unitario"):
                voce.costo_unitario = float(self.ga("costo_unitario").replace(",", "."))
            if self.ga("perc"):
                voce.perc = float(self.ga("perc").replace(",", "."))
            if self.ga("importo"):
                voce.importo = float(self.ga("importo").replace(",", "."))
            try:
                # modifica
                presenze.voci[int(self.ga("id_riga"))] = voce
            except:
                # inserimento
                presenze.voci.append(voce)

            # controlli
            self._voce_ok(presenze, voce)

        # Crea la risposta solo per il singolo dipendente
        if dipte_id:
            self.risposta.contenuto = None
        else:
            self.risposta.contenuto = GeneraDatiCal(self, self.calendario).genera_voci(presenze.id_dipe)

        # Salva il calendario nella sessione utente
        self.session['calendario'] = self.calendario
        if self.ga("stato"):
            self.session['salva_calendario'] = True
        self.session.to_save = True


    @sessions.require_lock
    def gestione_voci_multi(self):
        self.carica_calendario()
        for dipte_id in self._dipti_id:
            self.gestione_voci(dipte_id)
        self.salva_calendario()


    @sessions.require_lock
    def gestione_note(self, dipte_id=None):
        if not self.calendario:
            self.carica_calendario()
        if dipte_id:
            presenze = self.calendario.presenze[dipte_id]
        else:
            presenze = self.calendario.presenze[self._dipti_id[0]]

        # Aggiorna la classe
        if self.ga("stato") == "salva":
            presenze.note = self.ga("nota")

        # Crea la risposta solo per il singolo dipendente
        if dipte_id:
            self.risposta.contenuto = None
        else:
            self.risposta.contenuto = GeneraDatiCal(self, self.calendario).genera_note(presenze.id_dipe)

        # Salva il calendario nella sessione utente
        self.session['calendario'] = self.calendario
        if self.ga("stato"):
            self.session['salva_calendario'] = True
        self.session.to_save = True


    @sessions.require_lock
    def gestione_note_multi(self):
        self.carica_calendario()
        for dipte_id in self._dipti_id:
            self.gestione_note(dipte_id)
        self.salva_calendario()


    @sessions.require_lock
    def gestione_lav_dom(self, dipte_id=None):
        if dipte_id:
            presenze = self.calendario.presenze[dipte_id]
        else:
            presenze = self.calendario.presenze[self._dipti_id[0]]

        # Eliminazione
        if self.ga("stato") == "elimina":
            try:
                lav_dom = presenze.lavdom[int(self.ga("id_riga"))]
                presenze.lavdom.pop(presenze.lavdom.index(lav_dom))
            except: pass

        # Modifica/Inserimento
        if self.ga("stato") == "salva":
            lav_dom = LavoroDomCal()
            if self.ga("data_consegna"):
                lav_dom.data_consegna = formatData(self.ga("data_consegna"), "int", "%Y%m%d")
            if self.ga("ora_consegna"):
                lav_dom.ora_consegna = float(self.ga("ora_consegna").replace(":", "."))
            if self.ga("data_riconsegna"):
                lav_dom.data_riconsegna = formatData(self.ga("data_riconsegna"), "int", "%Y%m%d")
            if self.ga("ora_riconsegna"):
                lav_dom.ora_riconsegna = float(self.ga("ora_riconsegna").replace(":", "."))
            lav_dom.descrizione = self.ga("descrizione")
            if self.ga("quantita"):
                lav_dom.quantita = self.ga("quantita")
            try:
                # modifica
                presenze.lavdom[int(self.ga("id_riga"))] = lav_dom
            except:
                # inserimento
                presenze.lavdom.append(lav_dom)

            # controlli
            self._lav_dom_ok(presenze, lav_dom)

        # Crea la risposta solo per il singolo dipendente
        if not dipte_id:
            self.risposta.contenuto = GeneraDatiCal(self, self.calendario).genera_lavdom(presenze.id_dipe)

        # Salva il calendario nella sessione utente
        self.session['calendario'] = self.calendario
        if self.ga("stato"):
            self.session['salva_calendario'] = True
        self.session.to_save = True

    def gestione_anagrafche(self, dipte_id=None):pass


    def _verifica_calendario_ok(self):
        return self.verifica_calendario()


    def verifica_calendario(self):
        if self.calendario:
            # ... da calendario
            calendario = self.calendario
        else:
            # ... da chiusura mese
            presenze = self._q(Presenze).filter(and_(Presenze.periodo == self._periodo,
                                                     Presenze.ditta_id == self._ditta.id,
                                                     ))
            calendario = Calendario(self._ditta.id, self._periodo, None)
            for p in presenze:
                calendario.aggiungi_presenze_dipe(p)

        for dipte_id in calendario.ricava_id_dipendenti():
            presenze = calendario.presenze[dipte_id]
            # Esegue la verifica...
            for pres_giorno in presenze.giorni:
                causali = [c for c in pres_giorno.causali_giorno]
                # ...causale doppia
                if doppieInLista([c.codice for c in causali]):
                    self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                                 "Giorno %d: causale doppia nel giorno" %(pres_giorno.giorno)
                                 )
                    return False
                # ...causale unica nel giorno
                for causale in causali:
                    if causale.codice in ['AA', 'Y0', 'ZZ']:
                        continue
                    if presenze.is_collab and causale.codice == 'P':
                        continue
                    causale_db = self._leggi_causale(causale.codice)
                    if causale_db:
                        if causale_db.is_unica_giorno and len(causali) > 1:
                            if len(causali) == 2 and len([c for c in causali if c.codice in ['MC', 'ML']]):
                                continue
                            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                                         "Giorno %d: causale '%s' unica nel giorno" %(pres_giorno.giorno, causale.codice)
                                         )
                            return False
                    else:
                        self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                                     "Giorno %d: codice causale '%s' errato" %(pres_giorno.giorno, causale.codice)
                                     )
                        return False
                # ...malattia ricovero/non indennizzata
                codCau = None
                if "MR" in [c.codice for c in causali]:
                    codCau = "MR"
                elif "MN" in [c.codice for c in causali]:
                    codCau = "MN"
                if codCau:
                    if not [c for c in causali if c.codice in ['MC', 'ML']]:
                        self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                                     "Giorno %d: causale '%s' in assenza di evento malattia" %(pres_giorno.giorno, codCau)
                                     )
                        return False
                # ...ore assenze > ore teoriche
                ore_lav = float(0)
                if pres_giorno.ore_lavorate:
                    ore_lav = pres_giorno.ore_lavorate
                    if ore_lav < 0:
                        ore_lav *= -1
                ore_teo = float(0)
                if pres_giorno.ore_teoriche and pres_giorno.ecc != "FG":
                    ore_teo = pres_giorno.ore_teoriche
                if self._giorno_in_forza(presenze, pres_giorno.giorno):
                    if round(ore_teo, 2) <> round(self._operazione("+", ore_lav, self._totale_ore_assenza(causali)), 2):
                        self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                                     "Giorno %d: la somma delle ore assenza piu' ore lavorate non corrisponde all'orario teorico" %(pres_giorno.giorno)
                                     )
                        return False

        if self.calendario:
            self._addMess("Verifica completata con successo")
        return True


    @sessions.require_lock
    def salva_calendario(self):
        self._add_to_log()
        for dipte_id, presenze in self.calendario.presenze.iteritems():
            # Controlli
            # - calendario
            for pres_giorno in presenze.giorni:
                for causale in pres_giorno.causali_giorno:
                    if not self._causale_ok(presenze, pres_giorno.giorno, causale):
                        return
            # - eventi
            for evento in presenze.eventi:
                if not self._evento_ok(presenze, evento):
                    return
            # - voci
            for voce in presenze.voci:
                if not self._voce_ok(presenze, voce):
                    return
            # - lavori a domicilio
            for lav_dom in presenze.lavdom:
                if not self._lav_dom_ok(presenze, lav_dom):
                    return

            presenze_db = self._q(Presenze).filter(Presenze.id == presenze.id).first()

            # Salvataggio
            # - anagrafica
            presenze_db.qualifica = presenze.qualifica
            presenze_db.livello = presenze.livello
            presenze_db.centro_costo = presenze.centro_costo
            presenze_db.pos_inail = presenze.pos_inail
            presenze_db.tipo_pm = presenze.tipo_pm
            presenze_db.perc_pm = presenze.perc_pm
            presenze_db.mansione = presenze.mansione
            presenze_db.note = presenze.note

            # -calendario
            for pres_giorno in presenze.giorni:
                # - aggiorna l'orario
                pres_giorno_db = [g for g in presenze_db.calendario_rel if g.giorno_mese == pres_giorno.giorno][0]
                pres_giorno_db.ore_teoriche = pres_giorno.ore_teoriche
                pres_giorno_db.ore_lavorate = pres_giorno.ore_lavorate
                pres_giorno_db.ore_retribuite = pres_giorno.ore_retribuite
                pres_giorno_db.ecc = pres_giorno.ecc
                # - aggiunge/aggiorna le nuove causali presenti in sessione
                for causale in pres_giorno.causali_giorno:
                    try:
                        causale_db = [c for c in pres_giorno_db.calendarioCaus_rel if c.id == causale.id][0]
                    except:
                        causale_db = PresenzeCalendCaus()
                    causale_db.codice = causale.codice
                    causale_db.ore = causale.ore
                    if not causale_db.id:
                        pres_giorno_db.calendarioCaus_rel.append(causale_db)
                        self.sessiondb.commit()
                        causale.id = causale_db.id
                # - elimina le causali non piu' presenti in sessione (verifica solo quelle con "id" valorizzato)
                causali_id = [c.id for c in pres_giorno.causali_giorno]
                for causale_db in [c for c in pres_giorno_db.calendarioCaus_rel if c.id not in causali_id]:
                    pres_giorno_db.calendarioCaus_rel.pop(pres_giorno_db.calendarioCaus_rel.index(causale_db))
                    self._q(PresenzeCalendCaus).filter(PresenzeCalendCaus.id == causale_db.id).delete()

            # Eventi
            # - aggiunge/aggiorna i nuovi eventi presenti in sessione
            # - eventi
            # --- aggiunge/aggiorna i nuovi eventi presenti in sessione
            for evento in presenze.eventi:
                try:
                    evento_db = [e for e in presenze_db.eventi_rel if e.id == evento.id_evento][0]
                except:
                    evento_db = PresenzeEvento()
                evento_db.codice = evento.codice
                evento_db.data_inizio = evento.data_inizio
                evento_db.data_fine = evento.data_fine
                evento_db.ore = evento.ore
                if not evento_db.id:
                    presenze_db.eventi_rel.append(evento_db)
                    self.sessiondb.commit()
                    evento.id_evento = evento_db.id
            # --- elimina gli eventi non piu' presenti in sessione (verifica solo quelli con "id" valorizzato)
            eventi_id = [e.id_evento for e in presenze.eventi]
            for evento_db in [e for e in presenze_db.eventi_rel if e.id not in eventi_id]:
                presenze_db.eventi_rel.pop(presenze_db.eventi_rel.index(evento_db))
                self._q(PresenzeEvento).filter(PresenzeEvento.id == evento_db.id).delete()

            # - voci
            # --- aggiunge/aggiorna le nuove voci presenti in sessione
            for voce in presenze.voci:
                try:
                    voce_db = [v for v in presenze_db.voci_rel if v.id == voce.id_voce][0]
                except:
                    voce_db = PresenzeVoce()
                voce_db.prefisso = voce.prefisso
                voce_db.voce = voce.voce
                voce_db.suffisso = voce.suffisso
                voce_db.dato_base = voce.dato_base
                voce_db.costo_unitario = voce.costo_unitario
                voce_db.perc = voce.perc
                voce_db.importo = voce.importo
                if not voce_db.id:
                    presenze_db.voci_rel.append(voce_db)
                    self.sessiondb.commit()
                    voce.id_voce = voce_db.id
            # --- elimina le voci non piu' presenti in sessione (verifica solo quelle con "id" valorizzato)
            voci_id = [v.id_voce for v in presenze.voci]
            for voce_db in [v for v in presenze_db.voci_rel if v.id not in voci_id]:
                presenze_db.voci_rel.pop(presenze_db.voci_rel.index(voce_db))
                self._q(PresenzeVoce).filter(PresenzeVoce.id == voce_db.id).delete()

            # - lavori a domicilio
            # --- aggiunge/aggiorna i nuovi lavori a domicilio presenti in sessione
            for lav_dom in presenze.lavdom:
                try:
                    lav_dom_db = [l for l in presenze_db.lavori_dom_rel if l.id == lav_dom.id][0]
                except:
                    lav_dom_db = PresenzeLavDom()
                lav_dom_db.data_consegna = lav_dom.data_consegna
                lav_dom_db.ora_consegna = lav_dom.ora_consegna
                lav_dom_db.data_riconsegna = lav_dom.data_riconsegna
                lav_dom_db.ora_riconsegna = lav_dom.ora_riconsegna
                lav_dom_db.descrizione = lav_dom.descrizione
                lav_dom_db.quantita =lav_dom.quantita
                if not lav_dom_db.id:
                    presenze_db.lavori_dom_rel.append(lav_dom_db)
                    self.sessiondb.commit()
                    lav_dom.id = lav_dom_db.id
            # --- elimina i lavori a domicilio non piu' presenti in sessione (verifica solo quelli con "id" valorizzato)
            lav_dom_id = [l.id for l in presenze.lavdom]
            for lav_dom_db in [l for l in presenze_db.lavori_dom_rel if l.id not in lav_dom_id]:
                presenze_db.lavori_dom_rel.pop(presenze_db.lavori_dom_rel.index(lav_dom_db))
                self._q(PresenzeLavDom).filter(PresenzeLavDom.id == lav_dom_db.id).delete()

        self.sessiondb.commit()
        self.calendario = None
        del self.session.data['calendario']
        self.session['salva_calendario'] = False
        self.session.to_save = True
        self._addStato("Aggiornamento completato", 5000)


    def verifica_salvataggio(self):
        if self.session['salva_calendario']:
            self._addConfermacalendario("Salvare le modifiche?", 'SNA')
            self.risposta.contenuto = json.dumps(False)
        else:
            self.risposta.contenuto = json.dumps(True)
            self.annulla_modifiche()


    @sessions.require_lock
    def annulla_modifiche(self):

        self.calendario = None
        try:
            del self.session.data['calendario']
        except KeyError:
            pass
        self.session['salva_calendario'] = False
        self.session.to_save = True


    """
        G E N E R I C H E
    """
    def _assembla_presenze(self, dipte):
        data_cess = dipte.data_cess_fut or dipte.data_cess
        if formatData(dipte.data_ass,  "int", "%Y%m") > self._periodo:
            return None
        if data_cess and formatData(data_cess, "int", "%Y%m") < self._periodo:
            return None
        ass = Assemblaggio(self)
        ass.assemblaMese(self._periodo, dipte.id)
        # Anagrafica
        presenze = Presenze()
        presenze.periodo = self._periodo
        presenze.ditta_id = self._ditta.id
        presenze.dipendente_id = dipte.id
        presenze.qualifica = ass.anag['qualifica']
        presenze.livello = ass.anag['livello']
        presenze.centro_costo = ass.anag['centro_costo']
        presenze.pos_inail = ass.anag['pos_inail']
        presenze.tipo_pm = ass.anag['tipo_pm']
        presenze.perc_pm = ass.anag['perc_pm']
        presenze.ultima_sett = ass.anag['ultima_sett']
        presenze.evento_mp = ass.anag['evento_mp']
        presenze.ore_sett = ass.anag['ore_sett']
        presenze.mansione = ass.anag['mansione']
        # Calendario
        for giorno in ass.calend:
            presenzeCal = PresenzeCalend()
            presenzeCal.presenze_id = presenze.id
            presenzeCal.giorno_mese = giorno['GM']
            presenzeCal.giorno_sett = giorno['GS']
            if "FG" in giorno:
                presenzeCal.ore_teoriche = giorno['FG']
                presenzeCal.ecc = "FG"
            else:
                if giorno['OT'] == "RS":
                    presenzeCal.ore_teoriche = float(0)
                    presenzeCal.ecc = "RS"
                elif giorno['OT'] == "--":
                    presenzeCal.ore_teoriche = float(0)
                    presenzeCal.ecc = "--"
                else:
                    presenzeCal.ore_teoriche = giorno['OT']
                    presenzeCal.ecc = None
            presenzeCal.ore_lavorate = giorno['OL']
            presenzeCal.ore_retribuite = giorno['OR']
            presenze.calendario_rel.append(presenzeCal)
            # causali
            for c in range(1, MAX_CAUSALI + 1):
                if ("C%d" %c) in giorno:
                    presenzeCalCaus = PresenzeCalendCaus()
                    presenzeCalCaus.calendario_id = presenzeCal.id
                    presenzeCalCaus.codice = giorno['C%d' %c]
                    presenzeCalCaus.ore = float(giorno['O%d' %c])
                    presenzeCal.calendarioCaus_rel.append(presenzeCalCaus)
        # Eventi mese precedente
        for evento_prec in ass.eventi:
            evento = PresenzeEvento()
            evento.codice = evento_prec.codice
            evento.data_inizio = evento_prec.data_inizio
            evento.data_fine = evento_prec.data_fine
            evento.ore = evento_prec.ore
            presenze.eventi_rel.append(evento)

        return presenze


    def _calcola_ore_lav_retr(self, presenze, giorno):
        pres_giorno = presenze.giorni[giorno - 1]

        # Calcola le ore lavorate
        pres_giorno.ore_lavorate = float(0)
        if pres_giorno.ecc == None and pres_giorno.ore_teoriche:
            pres_giorno.ore_lavorate = self._operazione("-", pres_giorno.ore_teoriche, self._totale_ore_assenza(pres_giorno.causali_giorno))

        # Calcola le ore retribuite
        if presenze.qualifica.endswith(('ID', 'IN', 'SD', 'SN')):
            pres_giorno.ore_retribuite = pres_giorno.ore_teoriche
        else:
            if pres_giorno.ecc == 'RS':
                try:
                    pres_giorno.ore_retribuite = presenze.pgu_ore_retribuite_rs
                except:
                    pres_giorno.ore_retribuite = float(0)
            elif pres_giorno.ecc != '--' and pres_giorno.ore_teoriche:
                try:
                    ore_ret = self._operazione("*", pres_giorno.ore_teoriche, presenze.pgu_ore_retribuite)
                    ore_ret = self._operazione("/", ore_ret, presenze.pgu_ore_teoriche)
                except:
                    ore_ret = float(0)
                pres_giorno.ore_retribuite = ore_ret
            else:
                pres_giorno.ore_retribuite = float(0)


    def _carica_risposta(self, xml):
        self._addHead("Content-Type","text/xml")
        self.risposta.contenuto = ET.tostring(xml)


    def _causale_ok(self, presenze, giorno, causale):
        if causale.codice in ['AA', 'Y0', 'ZZ']:
            return True
        pres_giorno = presenze.giorni[giorno - 1]
        diz_err = {'id':presenze.id_dipe, 'g':(giorno), 'c':pres_giorno.causali_giorno.index(causale) + 2}
        if presenze.is_collab:
            if causale.codice == 'P':
                return True
            else:
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: per i collaboratori / tirocinanti e' ammesso solo il codice 'P' (Presenza)"
                              %(pres_giorno.giorno), diz_err
                            )
                return False
        if self._ditta.filtro_caus_wa == "N":
            causale_db = None
        else:
            causale_db = self._leggi_causale(causale.codice)
        if not causale_db or (self._ditta.filtro_caus_wa == "S" and not [c for c in self._ditta.abilit_caus_rel if c.codice == causale.codice]):
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Giorno %d: codice causale '%s' errato" %(pres_giorno.giorno, causale.codice),
                         diz_err
                        )
            return False

        if causale_db.is_no_calendario:
            data = int("%s%02d" %(self._periodo, pres_giorno.giorno))
            if not [e for e in presenze.eventi if e.codice == causale.codice and data >= e.data_inizio and data <= e.data_fine]:
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: causale '%s' non gestita a calendario" %(pres_giorno.giorno, causale.codice),
                             diz_err
                             )
                return False
        else:
            if causale_db.tipo == "A" and causale_db.valida_in_giorni != "S" and not pres_giorno.ore_teoriche:
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: causale '%s' non prevista in assenza di ore teoriche" %(pres_giorno.giorno, causale.codice),
                             diz_err
                             )
                return False

            if causale_db.valida_in_giorni == "A" and (pres_giorno.ecc == "FG" or pres_giorno.ore_teoriche <= 0):
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: causale '%s' non prevista in assenza di ore teoriche" %(pres_giorno.giorno, causale.codice),
                             diz_err
                             )
                return False

            if causale_db.valida_in_giorni == "B" and (pres_giorno.ecc == "FG" or pres_giorno.ore_retribuite <= 0):
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: causale '%s' non prevista in assenza di ore retribuite" %(pres_giorno.giorno, causale.codice),
                              diz_err
                             )
                return False

            if causale_db.valida_in_giorni == "C" and pres_giorno.ecc == "FG":
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: causale '%s' non prevista in assenza di ore retribuite" %(pres_giorno.giorno, causale.codice),
                              diz_err
                             )
                return False

            if causale_db.valida_in_giorni == "O" and pres_giorno.ore_teoriche <= 0:
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: causale '%s' non prevista in assenza di ore teoriche" %(pres_giorno.giorno, causale.codice),
                              diz_err
                             )
                return False

            if causale_db.valida_in_giorni == "R" and pres_giorno.ore_retribuite <= 0:
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: causale '%s' non prevista in assenza di ore retribuite" %(pres_giorno.giorno, causale.codice),
                              diz_err
                             )
                return False

            if self._impostazioni.is_sessantesimi and str(causale.ore)[2:] > "59":
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Giorno %d: causale '%s' con formato ore errato" %(pres_giorno.giorno, causale.codice),
                             diz_err
                             )
                return False

        return True


    def _cod_ore_causale(self, causale):
        cod_cau = None
        ore_cau = None
        if causale:
            causale = causale.upper().replace(",", ".").strip()
            cod_cau = causale[0:2].strip()
            ore_cau = causale[2:].strip()
            if ore_cau not in ['RS', '--']:
                try:
                    ore_cau = float(ore_cau)
                except:
                    ore_cau = float(0)
        return [cod_cau, ore_cau]


    def _compila_ore(self, pres_giorno, causale):
        causale_db = self._leggi_causale(causale.codice)
        if causale_db.tipo != "A":
            return causale.ore

        if causale.codice in ['MN', 'MR']:
            return 0

        if causale_db.is_giorno_completo:
            if pres_giorno.ecc in ['FG', 'RS', '--']:
                return 0
            else:
                return pres_giorno.ore_teoriche
        else:
            if causale.ore == 0:
                tot_ore_ass = self._totale_ore_assenza(pres_giorno.causali_giorno)
                return self._operazione("-", pres_giorno.ore_teoriche, tot_ore_ass)
            else:
                if pres_giorno.ecc in ['FG', 'RS', '--']:
                    return 0
                else:
                    return causale.ore


    def _evento_ok(self, presenze, evento):
        if self._ditta.filtro_caus_wa == "N":
            causale_db = None
        else:
            causale_db = self._leggi_causale(evento.codice)
        if not causale_db or (self._ditta.filtro_caus_wa == "S" and not [c for c in self._ditta.abilit_caus_rel if c.codice == evento.codice]):
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Evento %s: codice errato" %(evento.codice))
            return False

        if evento.data_inizio:
            if not isDataOK(evento.data_inizio):
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Evento %s: data inizio errata" %(evento.codice))
                return False
        else:
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Evento %s: data inizio obbligatoria" %(evento.codice))
            return False

        if evento.data_fine:
            if not isDataOK(evento.data_fine):
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Evento %s: data fine errata" %(evento.codice))
                return False
        else:
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Evento %s: data fine obbligatoria" %(evento.codice))
            return False

        if evento.data_inizio > evento.data_fine:
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Evento %s: data inizio > data fine" %(evento.codice))
            return False

        return True


    def _lav_dom_ok(self, presenze, lav_dom):
        if not lav_dom.descrizione:
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Lavoro a domicilio: descrizione obbligatoria")
            return False

        if lav_dom.data_consegna and not isDataOK(lav_dom.data_consegna):
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Lavoro a domicilio: data consegna errata")
            return False

        if lav_dom.ora_consegna and not isOrarioOK(lav_dom.ora_consegna):
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Lavoro a domicilio: orario consegna errato")
            return False

        if lav_dom.data_riconsegna and not isDataOK(lav_dom.data_riconsegna):
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Lavoro a domicilio: data riconsegna errata")
            return False

        if lav_dom.ora_riconsegna and not isOrarioOK(lav_dom.ora_riconsegna):
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Lavoro a domicilio: orario riconsegna errato")
            return False

        if lav_dom.data_consegna > lav_dom.data_riconsegna:
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Lavoro a domicilio: data consegna > data riconsegna")
            return False

        if lav_dom.data_consegna == lav_dom.data_riconsegna and lav_dom.ora_consegna > lav_dom.ora_riconsegna:
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Lavoro a domicilio: ora consegna > ora riconsegna")
            return False

        return True


    def _leggi_causale(self, codice, periodo=None):
        try:
            causale = Causale()
            causale.codice = codice
            causale.tipo = self.causali_db[codice]['tipo']
            causale.is_no_calendario = self.causali_db[codice]['is_no_calendario']
            causale.is_unica_giorno = self.causali_db[codice]['is_unica_giorno']
            causale.is_giorno_completo = self.causali_db[codice]['is_giorno_completo']
            causale.valida_in_giorni = self.causali_db[codice]['valida_in_giorni']
            return causale
        except:
            return None


    def _operazione(self, ope, add_1, add_2):
        if not add_1:
            add_1 = float(0)
        if not add_2:
            add_2 = float(0)
        if self._impostazioni.is_sessantesimi:
            add_1 = converti_ore(add_1, False)
            add_2 = converti_ore(add_2, False)
        if ope == "+":
            ore = add_1 + add_2
        elif ope == "-":
            ore = add_1 - add_2
        elif ope == "*":
            ore = add_1 * add_2
        elif ope == "/":
            ore =  add_1 / add_2
        elif ope == "%":
            ore = add_1 % add_2
        else:
            return None
        if self._impostazioni.is_sessantesimi:
            return float(converti_ore(ore, True))
        else:
            return ore


    def _ricava_giorno_data(self, data, tipo):
        giorno = int(str(data)[6:8])
        periodo = int(str(data)[0:6])
        if tipo == "i":
            if periodo < self._periodo:
                giorno = 1
            elif periodo > self._periodo:
                giorno = 0
        else:
            if periodo > self._periodo:
                giorno = getGiorniMese(self._periodo)
            elif periodo < self._periodo:
                giorno = 0
        return giorno


    def _totale_ore_assenza(self, causali):
        tot_ore_ass = float(0)
        for causale in causali:
            causale_db = self._leggi_causale(causale.codice)
            if causale_db and causale_db.tipo == "A" and causale.ore:
                tot_ore_ass = self._operazione("+", tot_ore_ass, causale.ore)
        return tot_ore_ass


    def _voce_ok(self, presenze, voce):
        if not voce.voce:
            self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                         "Codice voce obbligatorio (riga %s)" %(int(self.ga("id_riga")) + 1))
        else:
            if self._ditta.filtro_voci_wa == "N":
                voce_db = None
            else:
                voce_db = self._q(Voce).filter(and_(Voce.codice == voce.voce,
                                                    Voce.periodo_inizio <= self._periodo,
                                                    Voce.periodo_fine >= self._periodo,
                                                    Voce.com_movimenti != None,
                                                    )).first()
            if not voce_db or (self._ditta.filtro_voci_wa == "S" and not [v for v in self._ditta.abilit_voci_rel if v.codice == voce.voce]):
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Voce %s: codice errato" %(voce.voce))
                return False

            if voce.prefisso and not voce.prefisso in ['A', 'R']:
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Voce %s: prefisso voce errato; ammessi solo 'A'(Arretrato), 'R'(Recupero) o nulla" %(voce.voce))
                return False

            if voce.suffisso and not voce.suffisso in ['A', 'R']:
                self._addErr("Dipendente %s: %s<br></br>" %(presenze.codice, presenze.nominativo) +
                             "Voce %s: suffisso voce errato; ammessi solo 'A'(Accantonamento) o nulla" %(voce.voce))
                return False

        return True


    def _giorno_in_forza(self, presenze, giorno):
        da_giorno = 1
        a_giorno = getGiorniMese(presenze.periodo)
        if formatData(presenze.data_ass, "int", "%Y%m") == presenze.periodo:
            da_giorno = formatData(presenze.data_ass, "int", "%d")
        if presenze.data_cess and formatData(presenze.data_cess, "int", "%Y%m") == presenze.periodo:
            a_giorno = formatData(presenze.data_cess, "int", "%d")
        if giorno in range(da_giorno, a_giorno + 1):
            return True
        else:
            return False


#@sessions.require_lock
#def carica_causali_in_session(rbase):
    ## carica le causali in session
    #causali_db = GestioneListe(rbase).getCausali(campi=['codice', 'descrizione', 'tipo', 'is_no_calendario',
                                                        #'valida_in_giorni', 'is_unica_giorno','is_giorno_completo',])
    #causali = {}
    #for causale in causali_db:
        #causali[causale['codice']] = {'descrizione':causale['descrizione'],
                                      #'tipo':causale['tipo'],
                                      #'is_no_calendario':causale['is_no_calendario'],
                                      #'valida_in_giorni':causale['valida_in_giorni'],
                                      #'is_unica_giorno':causale['is_unica_giorno'],
                                      #'is_giorno_completo':causale['is_giorno_completo'],
                                      #}
    #rbase.session['causali_db'] = causali
    #rbase.session.to_save = True

#def dogentask(sessiondb, session, request):
    #return GestionePresenze(sessiondb, session, request).doTask()


#Main = GestionePresenze
