# coding: latin-1
from os                        import remove

from reportlab.lib             import colors
from reportlab.lib.pagesizes   import A4, landscape
from reportlab.lib.styles      import getSampleStyleSheet
from reportlab.lib.units       import inch
from reportlab.platypus        import PageBreak, SimpleDocTemplate
from reportlab.platypus.tables import Table as TableRL, TableStyle
from sqlalchemy.sql            import and_

from waonline.core.rbase            import RBase
from waonline.apps.presenze.tabelle.presenze          import Presenze
from waonline.apps.gestione.tabelle.tabelle_c         import Causale, ContrattoApprend
from waonline.apps.gestione.tabelle.tabelle_l         import Comune
from waonline.apps.gestione.tabelle.tabelle_p         import Collaborazione
from waonline.apps.gestione.tabelle.tabelle_v         import Qualifica
from waonline.utils.utility                   import *

#pdfmetrics.registerFont(TTFont("Vera", "Vera.ttf"))
#pdfmetrics.registerFont(TTFont("VeraBd", "VeraBd.ttf"))
mesi = ['GENNAIO', 'FEBBRAIO', 'MARZO', 'APRILE', 'MAGGIO', 'GIUGNO', 'LUGLIO', 'AGOSTO', 'SETTEMBRE', 'OTTOBRE', 'NOVEMBRE', 'DICEMBRE']

class StampaPresenze(RBase):
    def _post_init_(self):
        self.pagina = []
        self.perc_file = "/tmp/%s" %(os.urandom(32).encode('hex'))
        self.doc = SimpleDocTemplate(self.perc_file,
                                     pagesize=landscape(A4), # A4 orizzontale
                                     topMargin=0.1*inch,
                                     bottomMargin=0.1*inch,
                                     leftMargin=0.01*inch,
                                     rightMargin=0.1*inch,
                                     )
        self.stili = getSampleStyleSheet()
        self.tot_dipti = 0


    def stampa(self):
        tab_presenze = []
        for dipte_id in self._dipti_id:
            presenze = self._q(Presenze).filter(and_(Presenze.periodo == self._periodo,
                                                     Presenze.ditta_id == self._ditta.id,
                                                     Presenze.dipendente_id == dipte_id,
                                                     )).first()
            if not presenze:
                continue
            data_ass_cess = self._data_ass_cess(presenze.dipendente_rel, True)
            if data_ass_cess[0] > self._periodo:
                continue
            if data_ass_cess[1] and data_ass_cess[1] < self._periodo:
                continue
            tab_presenze.append(presenze)
            if len(tab_presenze) == 3:
                self.stampa_pagina(tab_presenze, 3)
                tab_presenze = []
        if len(tab_presenze) > 0 and len(tab_presenze) < 3:
            self.stampa_pagina(tab_presenze, len(tab_presenze))
        self.doc.build(self.pagina)
        # Risposta
        file_pdf = open(self.perc_file, 'rb')
        self.risposta.contenuto = file_pdf.read()
        self._addHead("Content-Disposition", "inline; filename=pres_%s_%s.pdf" %(self._ditta.codice, str(self._periodo)))
        self._repHead("Content-Type", "application/pdf")
        file_pdf.close()
        remove(self.perc_file)
        #except Exception, exc:
            #self.info(exc)


    def stampa_pagina(self, tab_presenze, num_dipti):
        self.pagina.append(self.stampa_ditta())
        self.pagina.append(self.stampa_dipendenti(tab_presenze, num_dipti))
        self.pagina.append(self.stampa_colonne(num_dipti))
        self.pagina.append(self.stampa_calendario(tab_presenze, num_dipti))
        self.pagina.append(self.stampa_totali(num_dipti))
        self.pagina.append(PageBreak())


    def stampa_ditta(self):
        comune_att = self._q(Comune).filter(Comune.codice == self._ditta.comune_att).first()
        dati = [ # riga 1
                ['FOGLIO PRESENZE DEL MESE', 'Ditta', self._ditta.descrizione, 'Attivita\'', self._ditta.descriz_att, '', '',],
                 # riga 2
                ['%s %s' %(mesi[framData(self._periodo)[1] - 1], framData(self._periodo, True)[0]), '',
                 '%s - %s - %s' %(self._ditta.indirizzo_att, comune_att.cap, comune_att.descrizione),
                 'Cod. fiscale', self._ditta.cod_fiscale, 'Posiz. Inps', '%s/%s' %(self._ditta.pos_inps, self._ditta.zona_inps)
                 ],
                ]
        tab_style = [
                    ('BOX', (0, 0),(0, -1), 1, colors.black),
                    ('BOX', (1, 0),(-1, -1), 1, colors.black),
                    ('SPAN', (4, 0), (-1, 0)),
                    ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                    # 'Foglio presenze del mese'
                    ('FONT', (0, 0), (0, 0), 'Helvetica', 9),
                    # Mese - Anno
                    ('FONT', (0, -1), (0, -1), 'Helvetica-Bold', 9),
                    # 'Ditta'
                    ('FONT', (1, 0), (1, 0), 'Helvetica', 9),
                    ('BACKGROUND', (1, 0), (1, 0), (0.5, 0.5, 0.5)),
                    ('TEXTCOLOR', (1, 0), (1, 0), colors.white),
                    # Ragione sociale
                    ('FONT', (2, 0), (2, 0), 'Helvetica-Bold', 9),
                    # Indirizzo
                    ('FONT', (2, -1), (2, -1), 'Helvetica-Bold', 9),
                    # 'Attività'
                    ('FONT', (3, 0), (3, 0), 'Helvetica', 8),
                    # Descrizione attività
                    ('FONT', (4, 0), (4, 0), 'Helvetica-Bold', 9),
                    # 'Cod. Fiscale'
                    ('FONT', (3, -1), (3, -1), 'Helvetica', 8),
                    # Codice fiscale
                    ('FONT', (4, -1), (4, -1), 'Helvetica-Bold', 9),
                    # 'Posiz. Inps'
                    ('FONT', (5, -1), (5, -1), 'Helvetica', 8),
                    # Posizione Inps
                    ('FONT', (6, -1), (6, -1), 'Helvetica-Bold', 9),
                    ]

        tabella = TableRL(dati,
                          1*[2.3*inch] + 1*[0.5*inch] + 1*[4.0*inch] + 1*[0.7*inch] + 1*[1.7*inch] + 1*[0.65*inch] + 1*[1.55*inch],
                          2*[0.2*inch],
                          )
        tabella.setStyle(TableStyle(tab_style))
        return tabella


    def stampa_dipendenti(self, presenze, num_dipti):
        dati = []
        riga1 = ['']
        riga2 = ['']
        riga3 = ['']
        riga4 = ['']
        riga5 = ['']
        riga6 = ['']
        riga7 = ['']
        for d in range(num_dipti):
            dipte = presenze[d].dipendente_rel
            des_qual = ""
            if dipte.is_collab:
                des_qual = self._q(Collaborazione.descrizione).filter(Collaborazione.codice == dipte.qualifica.codice,
                                                                      ).first()[0]
            else:
                des_qual = self._q(Qualifica.descrizione).filter(and_(Qualifica.codice == dipte.qualifica.codice,
                                                                      Qualifica.periodo_inizio <= self._periodo,
                                                                      Qualifica.periodo_fine >= self._periodo,
                                                                      )).first()[0]
            livello = ""
            if not dipte.is_collab:
                if dipte.qualifica.apprendistato:
                    try:
                        livello =  self._q(ContrattoApprend.livello_inquad)\
                                            .filter(and_(ContrattoApprend.contratto == self._ditta.contratto,
                                                         ContrattoApprend.codice == dipte.qualifica.apprendistato,
                                                         ContrattoApprend.periodo_inizio <= self._periodo,
                                                         ContrattoApprend.periodo_fine >= self._periodo,
                                                         )).first()[0]
                    except:pass
                else:
                    livello = presenze[d].livello
                if dipte.qualifica.perc_part_time:
                    livello += "/%d%%" %(dipte.qualifica.perc_part_time)
            self.tot_dipti += 1
            riga1.extend(['Dip.te', '%s.  %s %s' %(str(self.tot_dipti), dipte.cognome, dipte.nome), '', ''])
            des_qual_25 = ""
            for riga in [des_qual[i:i+25] for i in range(0, len(des_qual), 25)]:
                if des_qual_25:
                    des_qual_25 += "_\n"
                des_qual_25 += riga
            riga2.extend(['Codice', dipte.codice, 'Qualifica', des_qual_25])
            riga3.extend(['Codice ID', dipte.matricola, 'Pos.Inail', presenze[d].pos_inail])
            riga4.extend(['Gestione', '.', 'C/Costo', presenze[d].centro_costo])
            riga5.extend(['Livello', livello, 'Data ass.', formatData(dipte.data_ass, "str", "%d/%m/%Y")])
            riga6.extend(['Data nascita', formatData(dipte.data_nascita, "str", "%d/%m/%Y"),
                          'Data cess.', formatData(dipte.data_cess_fut or dipte.data_cess, "str", "%d/%m/%Y")])
            riga7.extend(['', '', '', ''])

        for d in range(num_dipti, 3):
            riga1.extend(['Dip.te', '', '', ''])
            riga2.extend(['Codice', '', 'Qualifica', ''])
            riga3.extend(['Codice ID', '', 'Pos.Inail', ''])
            riga4.extend(['Gestione', '', 'C/Costo', ''])
            riga5.extend(['Livello', '', 'Data ass.', ''])
            riga6.extend(['Data nascita', '', 'Data cess.', ''])
            riga7.extend(['', '', '', ''])

        dati.extend([riga1, riga2, riga3, riga4, riga5, riga6, riga7])

        tab_style = [
                    ('BOX', (0, 0),(0, -1), 1, colors.black),
                    ('BOX', (1, 0),(4, 0), 1, colors.black),
                    ('BOX', (5, 0),(8, 0), 1, colors.black),
                    ('BOX', (9, 0),(-1, 0), 1, colors.black),
                    ('BOX', (1, 1),(4, -1), 1, colors.black),
                    ('BOX', (5, 1),(8, -1), 1, colors.black),
                    ('BOX', (9, 1),(-1, -1), 1, colors.black),
                    ('SPAN', (2, 0),(4, 0)),
                    ('SPAN', (6, 0),(8, 0)),
                    ('SPAN', (10, 0),(-1, 0)),
                    ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                    # 'Dip.te' 1
                    ('FONT', (1, 0), (1, 0), 'Helvetica', 8),
                    ('BACKGROUND', (1, 0), (1, 0), (0.5, 0.5, 0.5)),
                    ('TEXTCOLOR', (1, 0), (1, 0), colors.white),
                    # Dipendente 1
                    ('FONT', (2, 0), (2, 0), 'Helvetica-Bold', 9),
                    # 'Numero' 1
                    ('FONT', (1, 1), (1, 1), 'Helvetica', 8),
                    # Numero 1
                    ('FONT', (2, 1), (2, 1), 'Helvetica-Bold', 9),
                    # 'Qualifica' 1
                    ('FONT', (3, 1), (3, 1), 'Helvetica', 8),
                    # Qualifica 1
                    ('FONT', (4, 1), (4, 1), 'Helvetica-Bold', 9),
                    # 'Codice' 1
                    ('FONT', (1, 2), (1, 2), 'Helvetica', 8),
                    # Codice 1
                    ('FONT', (2, 2), (2, 2), 'Helvetica-Bold', 9),
                    # 'Pos. Inail' 1
                    ('FONT', (3, 2), (3, 2), 'Helvetica', 8),
                    # Posizione Inail 1
                    ('FONT', (4, 2), (4, 2), 'Helvetica-Bold', 9),
                    # 'Matricola' 1
                    ('FONT', (1, 3), (1, 3), 'Helvetica', 8),
                    # Matricola 1
                    ('FONT', (2, 3), (2, 3), 'Helvetica-Bold', 9),
                    # 'C/Costo' 1
                    ('FONT', (3, 3), (3, 3), 'Helvetica', 8),
                    # Centro di costo 1
                    ('FONT', (4, 3), (4, 3), 'Helvetica-Bold', 9),
                    # 'Gestione' 1
                    ('FONT', (1, 4), (1, 4), 'Helvetica', 8),
                    # Gestione 1
                    ('FONT', (2, 4), (2, 4), 'Helvetica-Bold', 9),
                    # 'Data nascita' 1
                    ('FONT', (3, 4), (3, 4), 'Helvetica', 8),
                    # Data di nascita 1
                    ('FONT', (4, 4), (4, 4), 'Helvetica-Bold', 9),
                    # 'Livello' 1
                    ('FONT', (1, 5), (1, 5), 'Helvetica', 8),
                    # Livello 1
                    ('FONT', (2, 5), (2, 5), 'Helvetica-Bold', 9),
                    # 'Data assunzione' 1
                    ('FONT', (3, 5), (3, 5), 'Helvetica', 8),
                    # Data di assunzione 1
                    ('FONT', (4, 5), (4, 5), 'Helvetica-Bold', 9),
                    # 'Data cessazione' 1
                    ('FONT', (3, 6), (3, 6), 'Helvetica', 8),
                    # Data di cessazione 1
                    ('FONT', (4, 6), (4, 6), 'Helvetica-Bold', 9),

                    # 'Dip.te' 2
                    ('FONT', (5, 0), (5, 0), 'Helvetica', 8),
                    ('BACKGROUND', (5, 0), (5, 0), (0.5, 0.5, 0.5)),
                    ('TEXTCOLOR', (5, 0), (5, 0), colors.white),
                    # Dipendente 2
                    ('FONT', (6, 0), (6, 0), 'Helvetica-Bold', 9),
                    # 'Numero' 2
                    ('FONT', (5, 1), (5, 1), 'Helvetica', 8),
                    # Numero 2
                    ('FONT', (6, 1), (6, 1), 'Helvetica-Bold', 9),
                    # 'Qualifica' 2
                    ('FONT', (7, 1), (7, 1), 'Helvetica', 8),
                    # Qualifica 2
                    ('FONT', (8, 1), (8, 1), 'Helvetica-Bold', 9),
                    # 'Codice' 2
                    ('FONT', (5, 2), (5, 2), 'Helvetica', 8),
                    # Codice 2
                    ('FONT', (6, 2), (6, 2), 'Helvetica-Bold', 9),
                    # 'Pos. Inail' 2
                    ('FONT', (7, 2), (7, 2), 'Helvetica', 8),
                    # Posizione Inail 2
                    ('FONT', (8, 2), (8, 2), 'Helvetica-Bold', 9),
                    # 'Matricola' 2
                    ('FONT', (5, 3), (5, 3), 'Helvetica', 8),
                    # Matricola 2
                    ('FONT', (6, 3), (6, 3), 'Helvetica-Bold', 9),
                    # 'C/Costo' 2
                    ('FONT', (7, 3), (7, 3), 'Helvetica', 8),
                    # Centro di costo 2
                    ('FONT', (8, 3), (8, 3), 'Helvetica-Bold', 9),
                    # 'Gestione' 2
                    ('FONT', (5, 4), (5, 4), 'Helvetica', 8),
                    # Gestione 2
                    ('FONT', (6, 4), (6, 4), 'Helvetica-Bold', 9),
                    # 'Data nascita' 2
                    ('FONT', (7, 4), (7, 4), 'Helvetica', 8),
                    # Data di nascita 2
                    ('FONT', (8, 4), (8, 4), 'Helvetica-Bold', 9),
                    # 'Livello' 2
                    ('FONT', (5, 5), (5, 5), 'Helvetica', 8),
                    # Livello 2
                    ('FONT', (6, 5), (6, 5), 'Helvetica-Bold', 9),
                    # 'Data assunzione' 2
                    ('FONT', (7, 5), (7, 5), 'Helvetica', 8),
                    # Data di assunzione 2
                    ('FONT', (8, 5), (8, 5), 'Helvetica-Bold', 9),
                    # 'Data cessazione' 2
                    ('FONT', (7, 6), (7, 6), 'Helvetica', 8),
                    # Data di cessazione 2
                    ('FONT', (8, 6), (8, 6), 'Helvetica-Bold', 9),

                    # 'Dip.te' 3
                    ('FONT', (9, 0), (9, 0), 'Helvetica', 8),
                    ('BACKGROUND', (9, 0), (9, 0), (0.5, 0.5, 0.5)),
                    ('TEXTCOLOR', (9, 0), (9, 0), colors.white),
                    # Dipendente 3
                    ('FONT', (10, 0), (10, 0), 'Helvetica-Bold', 9),
                    # 'Numero' 3
                    ('FONT', (9, 1), (9, 1), 'Helvetica', 8),
                    # Numero 3
                    ('FONT', (10, 1), (10, 1), 'Helvetica-Bold', 9),
                    # 'Qualifica' 3
                    ('FONT', (11, 1), (11, 1), 'Helvetica', 8),
                    # Qualifica 3
                    ('FONT', (12, 1), (12, 1), 'Helvetica-Bold', 9),
                    # 'Codice' 3
                    ('FONT', (9, 2), (9, 2), 'Helvetica', 8),
                    # Codice 3
                    ('FONT', (10, 2), (10, 2), 'Helvetica-Bold', 9),
                    # 'Pos. Inail' 3
                    ('FONT', (11, 2), (11, 2), 'Helvetica', 8),
                    # Posizione Inail 3
                    ('FONT', (12, 2), (12, 2), 'Helvetica-Bold', 9),
                    # 'Matricola' 3
                    ('FONT', (9, 3), (9, 3), 'Helvetica', 8),
                    # Matricola 3
                    ('FONT', (10, 3), (10, 3), 'Helvetica-Bold', 9),
                    # 'C/Costo' 3
                    ('FONT', (11, 3), (11, 3), 'Helvetica', 8),
                    # Centro di costo 3
                    ('FONT', (12, 3), (12, 3), 'Helvetica-Bold', 9),
                    # 'Gestione' 3
                    ('FONT', (9, 4), (9, 4), 'Helvetica', 8),
                    # Gestione 3
                    ('FONT', (10, 4), (10, 4), 'Helvetica-Bold', 9),
                    # 'Data nascita' 3
                    ('FONT', (11, 4), (11, 4), 'Helvetica', 8),
                    # Data di nascita 3
                    ('FONT', (12, 4), (12, 4), 'Helvetica-Bold', 9),
                    # 'Livello' 3
                    ('FONT', (9, 5), (9, 5), 'Helvetica', 8),
                    # Livello 3
                    ('FONT', (10, 5), (10, 5), 'Helvetica-Bold', 9),
                    # 'Data assunzione' 3
                    ('FONT', (11, 5), (11, 5), 'Helvetica', 8),
                    # Data di assunzione 3
                    ('FONT', (12, 5), (12, 5), 'Helvetica-Bold', 9),
                    # 'Data assunzione' 3
                    ('FONT', (11, 6), (11, 6), 'Helvetica', 8),
                    # Data di assunzione 3
                    ('FONT', (12, 6), (12, 6), 'Helvetica-Bold', 9),
                    ]

        tabella = TableRL(dati,
                          1*[0.6*inch] + (1*[0.7*inch] + 1*[0.7*inch] + 1*[0.6*inch] + 1*[1.6*inch]) * 3,
                          7*[0.2*inch],
                          )
        tabella.setStyle(TableStyle(tab_style))
        return tabella


    def stampa_colonne(self, num_dipti):
        dati = [["Giorno"] + (["Ore teo."] + ["Causale/ore"]*5)*3]
        tab_style = [
                    ('GRID', (0, 0), (-1, 0), 1, colors.black),
                    ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
                    ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                    ('FONT', (0, 0), (-1, 0), 'Helvetica', 7),
                    ]

        tabella = TableRL(dati,
                          1*[0.6*inch] + (1*[0.45*inch] + 1*[0.63*inch] * 5) * 3,
                          1*[0.2*inch],
                          )
        tabella.setStyle(TableStyle(tab_style))
        return tabella


    def stampa_calendario(self, presenze, num_dipti):
        self.tot_ore_ord = [0]*3
        self.tot_ore_lav = [0]*3
        self.tot_ore_ass = [0]*3
        self.tot_ore_str = [0]*3
        self.tot_gg_ass = [0]*3
        self.tot_gg_lav = [0]*3
        dati = []
        for g in range(getGiorniMese(self._periodo)):
            dati_gg = []
            for d in range(num_dipti):
                giorno_ass = 1
                giorno_cess = getGiorniMese(presenze[d].periodo)
                data_ass_cess = self._data_ass_cess(presenze[d].dipendente_rel)
                if formatData(data_ass_cess[0], "int", "%Y%m") == presenze[d].periodo:
                    giorno_ass = formatData(data_ass_cess[0], "int", "%d")
                if data_ass_cess[1] and formatData(data_ass_cess[1], "int", "%Y%m") == presenze[d].periodo:
                    giorno_cess = formatData(data_ass_cess[1], "int", "%d")
                giorno = presenze[d].calendario_rel[g]
                if d == 0:
                    # giorno mese e settimana
                    dati_gg.extend([str(giorno.giorno_mese).zfill(2), giorno.giorno_sett])
                # ore teoriche
                if g in range(giorno_ass - 1, giorno_cess):
                    if giorno.ecc:
                        ecc = giorno.ecc
                        # per giustificare a sinistra rispetto alle ore
                        if giorno.ecc == "--":
                            ecc += " " * 4
                        else:
                            ecc += " " * 2
                        dati_gg.append(ecc)
                    else:
                        if giorno.ore_lavorate:
                            dati_gg.append(("%.2f" % float(giorno.ore_lavorate)).replace(".", ","))
                            self.tot_ore_ord[d] = self._operazione("+", self.tot_ore_ord[d], giorno.ore_lavorate)
                        else:
                            dati_gg.append("")
                else:
                    dati_gg.append("")
                    if g < giorno_ass - 1:
                        dati_gg.extend(['AA'] + ['']*9)
                        continue
                    elif g > giorno_cess - 1:
                        dati_gg.extend(['ZZ'] + ['']*9)
                        continue

                # causali
                is_ass = False
                is_str = False
                num_cau_gg = 0
                for c in giorno.calendarioCaus_rel:
                    causale_db = self._q(Causale).filter(and_(Causale.codice == c.codice,
                                                              Causale.periodo_inizio <= self._periodo,
                                                              Causale.periodo_fine >= self._periodo,
                                                              )).first()
                    if causale_db:
                        if causale_db.tipo == "A":
                            if c.ore:
                                self.tot_ore_ass[d] = self._operazione("+", self.tot_ore_ass[d], c.ore)
                            if giorno.ore_lavorate <= 0:
                                is_ass = True
                        else:
                            if c.ore:
                                if causale_db.is_ore_ordinarie:
                                    self.tot_ore_str[d] = self._operazione("+", self.tot_ore_str[d], c.ore)
                                is_str = True

                    dati_gg.append(c.codice)
                    if c.ore:
                        dati_gg.append(str(("%.2f" % float(c.ore)).replace(".", ",")))
                    else:
                        dati_gg.append("")
                    num_cau_gg += 1

                for c in range(num_cau_gg, 5):
                    dati_gg.extend(['']*2)

                if is_ass:
                    self.tot_gg_ass[d] += 1
                if is_str or giorno.ore_lavorate > 0:
                    self.tot_gg_lav[d] += 1

            for d in range(num_dipti, 3):
                dati_gg.extend(['']*11)
            dati.append(dati_gg)

        for g in range(getGiorniMese(self._periodo), 31):
            dati.append(['']*35)

        for d in range(num_dipti):
            self.tot_ore_lav[d] = self.tot_ore_ord[d] + self.tot_ore_str[d]

        tab_style = [
                    ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
                    ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                    ('FONT', (0, 0), (-1, -1), 'Helvetica-Bold', 9),
                    ('BOX', (0, 0),(1, -1), 1, colors.black),
                    ('LINEABOVE', (0, 1), (-1, -1), 0, (0.5, 0.5, 0.5)),
                    # Dipendente 1
                    ('BOX', (2, 0),(2, -1), 1, colors.black),
                    ('BOX', (3, 0),(4, -1), 1, colors.black),
                    ('BOX', (5, 0),(6, -1), 1, colors.black),
                    ('BOX', (7, 0),(8, -1), 1, colors.black),
                    ('BOX', (9, 0),(10, -1), 1, colors.black),
                    ('BOX', (11, 0),(12, -1), 1, colors.black),
                    ('LINEBEFORE', (4, 0), (4, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (6, 0), (6, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (8, 0), (8, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (10, 0), (10, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (12, 0), (12, -1), 0, (0.5, 0.5, 0.5)),
                    # Dipendente 2
                    ('BOX', (13, 0),(13, -1), 1, colors.black),
                    ('BOX', (14, 0),(15, -1), 1, colors.black),
                    ('BOX', (16, 0),(17, -1), 1, colors.black),
                    ('BOX', (18, 0),(19, -1), 1, colors.black),
                    ('BOX', (20, 0),(21, -1), 1, colors.black),
                    ('BOX', (22, 0),(23, -1), 1, colors.black),
                    ('LINEBEFORE', (15, 0), (15, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (17, 0), (17, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (19, 0), (19, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (21, 0), (21, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (23, 0), (23, -1), 0, (0.5, 0.5, 0.5)),
                    # Dipendente 3
                    ('BOX', (24, 0),(24, -1), 1, colors.black),
                    ('BOX', (25, 0),(26, -1), 1, colors.black),
                    ('BOX', (27, 0),(28, -1), 1, colors.black),
                    ('BOX', (29, 0),(30, -1), 1, colors.black),
                    ('BOX', (31, 0),(32, -1), 1, colors.black),
                    ('BOX', (33, 0),(34, -1), 1, colors.black),
                    ('LINEBEFORE', (26, 0), (26, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (28, 0), (28, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (30, 0), (30, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (32, 0), (32, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (34, 0), (34, -1), 0, (0.5, 0.5, 0.5)),
                    # per rimarcare le righe sovrapposte da quelle grigie
                    ('LINEABOVE', (0, 0), (-1, 0), 1, colors.black),  # linea sopra
                    ('LINEBELOW', (0, -1), (-1, -1), 1 , colors.black),  # linea sotto
                    ('LINEBEFORE', (0, 0), (0, -1), 1, colors.black),  # colonna prima
                    ]

        tabella = TableRL(dati,
                          2*[0.3*inch] + (1*[0.45*inch] + (1*[0.23*inch] + 1*[0.4*inch]) * 5) * 3,
                          31*[0.17*inch],
                          )
        tabella.setStyle(TableStyle(tab_style))
        return tabella


    def stampa_totali(self, num_dipti):
        dati = []
        riga1 = ['TOTALI']
        riga2 = ['']
        riga3 = ['']*19
        for d in range(num_dipti):
            ore_ord = "%.2f" %(self.tot_ore_ord[d]) if self.tot_ore_ord[d] else ""
            ore_ord.replace(".", ",")
            ore_lav = "%.2f" %(self.tot_ore_lav[d]) if self.tot_ore_lav[d] else ""
            ore_lav.replace(".", ",")
            gg_lav = "%d" %(self.tot_gg_lav[d]) if self.tot_gg_lav[d] else ""
            ore_str = "%.2f" %(self.tot_ore_str[d]) if self.tot_ore_str[d] else ""
            ore_str.replace(".", ",")
            ore_ass = "%.2f" %(self.tot_ore_ass[d]) if self.tot_ore_ass[d] else ""
            ore_ass.replace(".", ",")
            gg_ass = "%d" %(self.tot_gg_ass[d]) if self.tot_gg_ass[d] else ""
            riga1.extend(['Ore ord.', ore_ord, 'Tot. ore. lav.', ore_lav, 'GG lavorati', gg_lav])
            riga2.extend(['Ore straord.', ore_str, 'Ore assenza', ore_ass, 'GG assenza', gg_ass])

        dati.extend([riga1, riga2, riga3])

        tab_style = [
                    ('BOX', (0, 0),(0, -1), 1, colors.black),
                    ('BOX', (1, 0),(6, -1), 1, colors.black),
                    ('BOX', (7, 0),(12, -1), 1, colors.black),
                    ('BOX', (13, 0),(18, -1), 1, colors.black),
                    ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                    ('FONT', (0, 0), (0, -1), 'Helvetica-Bold', 9),
                    # per rimarcare le righe sovrapposte da quelle grigie
                    ('LINEABOVE', (1, 1), (-1, -1), 0, (0.5, 0.5, 0.5)),
                    ('LINEBEFORE', (1, 0), (1, -1), 1, colors.black),
                    ('LINEBEFORE', (7, 0), (7, -1), 1, colors.black),
                    ('LINEBEFORE', (13, 0), (13, -1), 1, colors.black),
                    ('LINEBEFORE', (19, 0), (19, -1), 1, colors.black),

                    # Dipendente 1
                    # 'Ore ord.' 1
                    ('FONT', (1, 0), (1, 0), 'Helvetica', 8),
                    # Ore ordinarie 1
                    ('FONT', (2, 0), (2, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (2, 0), (2, 0), 'RIGHT'),
                    # 'Tot. ore. lav.' 1
                    ('FONT', (3, 0), (3, 0), 'Helvetica', 8),
                    # Totale ore lavorate 1
                    ('FONT', (4, 0), (4, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (4, 0), (4, 0), 'RIGHT'),
                    # 'GG lavorati' 1
                    ('FONT', (5, 0), (5, 0), 'Helvetica', 8),
                    # Totale giorni lavorati 1
                    ('FONT', (6, 0), (6, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (6, 0), (6, 0), 'RIGHT'),
                    # 'Ore straord.' 1
                    ('FONT', (1, 1), (1, 1), 'Helvetica', 8),
                    # Ore straordinarie 1
                    ('FONT', (2, 1), (2, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (2, 1), (2, 1), 'RIGHT'),
                    # 'Ore assenza' 1
                    ('FONT', (3, 1), (3, 1), 'Helvetica', 8),
                    # Totale ore assenza 1
                    ('FONT', (4, 1), (4, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (4, 1), (4, 1), 'RIGHT'),
                    # 'GG assenza' 1
                    ('FONT', (5, 1), (5, 1), 'Helvetica', 8),
                    # Totale giorni assenza 1
                    ('FONT', (6, 1), (6, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (6, 1), (6, 1), 'RIGHT'),

                    # Dipendente 2
                    # 'Ore ord.' 2
                    ('FONT', (7, 0), (7, 0), 'Helvetica', 8),
                    # Ore ordinarie 2
                    ('FONT', (8, 0), (8, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (8, 0), (8, 0), 'RIGHT'),
                    # 'Tot. ore. lav.' 2
                    ('FONT', (9, 0), (9, 0), 'Helvetica', 8),
                    # Totale ore lavorate 2
                    ('FONT', (10, 0), (10, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (10, 0), (10, 0), 'RIGHT'),
                    # 'GG lavorati' 2
                    ('FONT', (11, 0), (11, 0), 'Helvetica', 8),
                    # Totale giorni lavorati 2
                    ('FONT', (12, 0), (12, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (12, 0), (12, 0), 'RIGHT'),
                    # 'Ore straord.' 2
                    ('FONT', (7, 1), (7, 1), 'Helvetica', 8),
                    # Ore straordinarie 2
                    ('FONT', (8, 1), (8, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (8, 1), (8, 1), 'RIGHT'),
                    # 'Ore assenza' 2
                    ('FONT', (9, 1), (9, 1), 'Helvetica', 8),
                    # Totale ore assenza 2
                    ('FONT', (10, 1), (10, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (10, 1), (10, 1), 'RIGHT'),
                    # 'GG assenza' 2
                    ('FONT', (11, 1), (11, 1), 'Helvetica', 8),
                    # Totale giorni assenza 2
                    ('FONT', (12, 1), (12, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (12, 1), (12, 1), 'RIGHT'),

                    # Dipendente 3
                    # 'Ore ord.' 3
                    ('FONT', (13, 0), (13, 0), 'Helvetica', 8),
                    # Ore ordinarie 3
                    ('FONT', (14, 0), (14, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (14, 0), (14, 0), 'RIGHT'),
                    # 'Tot. ore. lav.' 3
                    ('FONT', (15, 0), (15, 0), 'Helvetica', 8),
                    # Totale ore lavorate 3
                    ('FONT', (16, 0), (16, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (16, 0), (16, 0), 'RIGHT'),
                    # 'GG lavorati' 3
                    ('FONT', (17, 0), (17, 0), 'Helvetica', 8),
                    # Totale giorni lavorati 3
                    ('FONT', (18, 0), (18, 0), 'Helvetica-Bold', 9),
                    ('ALIGN', (18, 0), (18, 0), 'RIGHT'),
                    # 'Ore straord.' 3
                    ('FONT', (13, 1), (13, 1), 'Helvetica', 8),
                    # Ore straordinarie 3
                    ('FONT', (14, 1), (14, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (14, 1), (14, 1), 'RIGHT'),
                    # 'Ore assenza' 3
                    ('FONT', (15, 1), (15, 1), 'Helvetica', 8),
                    # Totale ore assenza 3
                    ('FONT', (16, 1), (16, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (16, 1), (16, 1), 'RIGHT'),
                    # 'GG assenza' 3
                    ('FONT', (17, 1), (17, 1), 'Helvetica', 8),
                    # Totale giorni assenza 3
                    ('FONT', (18, 1), (18, 1), 'Helvetica-Bold', 9),
                    ('ALIGN', (18, 1), (18, 1), 'RIGHT'),
                    ]

        tabella = TableRL(dati,
                          1*[0.6*inch] + (1*[0.7*inch] + 1*[0.55*inch] + 1*[0.75*inch] + 1*[0.55*inch] + 1*[0.75*inch] + 1*[0.3*inch]) * 3,
                          3*[0.2*inch],
                          )

        tabella.setStyle(TableStyle(tab_style))
        return tabella


    def _data_ass_cess(self, dipendente, periodo=False):
        data_ass = dipendente.data_ass
        if dipendente.data_cess_fut:
            data_cess = dipendente.data_cess_fut
        elif dipendente.data_cess:
            data_cess = dipendente.data_cess
        else:
            data_cess = None

        if periodo:
            data_ass = formatData(data_ass, "int", "%Y%m")
            if data_cess:
                data_cess = formatData(data_cess, "int", "%Y%m")

        return (data_ass, data_cess)


    def _operazione(self, ope, add_1=0, add_2=0):
        if self._impostazioni.is_sessantesimi:
            add_1 = converti_ore(add_1, False)
            add_2 = converti_ore(add_2, False)
        if ope == "+":
            ore = add_1 + add_2
        elif ope == "-":
            ore = add_1 - add_2
        elif ope == "*":
            ore = add_1 * add_2
        elif ope == "/":
            ore =  add_1 / add_2
        elif ope == "%":
            ore = add_1 % add_2
        else:
            return None
        if self._impostazioni.is_sessantesimi:
            return converti_ore(ore, True)
        else:
            return ore


def dogentask(sessiondb, session, request):
    return StampaPresenze(sessiondb,session,request).doTask()

