# coding: latin-1
from os import remove
from types import TupleType

from reportlab.lib import colors
from reportlab.lib.enums import TA_CENTER
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.units import inch
from reportlab.platypus import PageBreak, Paragraph, SimpleDocTemplate, Spacer
from reportlab.platypus.flowables import Flowable, KeepTogether
from reportlab.platypus.tables import Table as TableRL
from sqlalchemy.sql import and_

from waonline.core.rbase import RBase
from waonline.utils.utility import *
from waonline.core.wadecorators import WaAuth
from waonline.apps.anagrafiche.tabelle.dipendenti import Dipendente
from waonline.apps.anagrafiche.tabelle.ditte import Ditta
from waonline.apps.presenze.tabelle.presenze import *
from waonline.apps.gestione.tabelle.tabelle_c import Causale, Voce

mesi = ['Gennaio', 'Febbraio', 'Marzo', 'Aprile', 'Maggio', 'Giugno', 'Luglio', 'Agosto', 'Settembre', 'Ottobre', 'Novembre', 'Dicembre']


class StampaPresenzeRiass(RBase):
    def _post_init_(self):
        self.presenze = None
        self.dipte = None
        self.calendario = None
        if 'calendario' in self.session:
            self.calendario = self.session['calendario']
        #if self.ga("applica") == "dipe":
            #self.applica = "singolo"
        #else:
            #self.applica = self.ga("applica")
        #self.dal = int(self.ga("dal", None))
        #self.al = int(self.ga("al", None))
        #self.causali_si = self.ga("causali")
        #self.voci_si = self.ga("voci")
        #self.lav_dom_si = self.ga("lav_dom")
        #self.note_si = self.ga("note")
        #self.firma_si = self.ga("firma")
        self.tot_ditta = {'dipti_pre':0, 'dipti_ass':0, 'tot_ore_lav':0, 'tot_ore_ass':0, 'causali':{}, 'voci':{}}
        self.pdf = []
        self.perc_file = "/tmp/%s" %(os.urandom(32).encode('hex'))
        self.doc = SimpleDocTemplate(self.perc_file,
                                     topMargin=0.1*inch,
                                     bottomMargin=0.1*inch,
                                     leftMargin=0.1*inch,
                                     rightMargin=0.1*inch,
                                     )
        self.stili = getSampleStyleSheet()
        self.tot_dipti = 0


    @WaAuth()
    def stampa_riassuntivo(self, dal=WaAuth.INT, al=WaAuth.INT, applica="tutti",
            causali=False, voci=False, lav_dom=False, note=False, firma=False,
            sessione=False):
        self.applica = applica
        self.dal = dal
        self.al = al
        self.causali_si = causali
        self.voci_si = voci
        self.lav_dom_si = lav_dom
        self.note_si = note
        self.firma_si = firma
        for dipte_id in self._dipti_id:
            if sessione:
                if self.calendario:
                    self.presenze = self._get_presenze_from_session(self.calendario.presenze[dipte_id])
            else:
                self.presenze = self._q(Presenze).filter(and_(Presenze.periodo == self._periodo,
                                                              Presenze.ditta_id == self._ditta.id,
                                                              Presenze.dipendente_id == dipte_id,
                                                              )).first()
            if not self.presenze:
                continue

            self.dipte = self.presenze.dipendente_rel
            self.data_ass = self.dipte.data_ass
            if self.dipte.data_cess_fut:
                self.data_cess = self.dipte.data_cess_fut
            elif self.dipte.data_cess:
                self.data_cess = self.dipte.data_cess
            else:
                self.data_cess = None
            if formatData(self.data_ass, "int", "%Y%m") > self._periodo or \
               (self.data_cess > 0 and formatData(self.data_cess, "int", "%Y%m") < self._periodo):
                continue
            self.giorni_in_forza = None
            da_giorno = 1
            a_giorno = getGiorniMese(self.presenze.periodo)
            if formatData(self.data_ass, "int", "%Y%m") == self.presenze.periodo:
                da_giorno = formatData(self.data_ass, "int", "%d")
            if self.data_cess and formatData(self.data_cess, "int", "%Y%m") == self.presenze.periodo:
                a_giorno = formatData(self.data_cess, "int", "%d")
            self.giorni_in_forza = range(da_giorno, a_giorno + 1)

            if len(self.pdf) == 0 or self.firma_si:
                if len(self.pdf) > 1:
                   self.pdf.append(PageBreak())
                self._stampa_testa()
            else:
                self.pdf.append(Spacer(0*inch, 0.2*inch))
            self._stampa_dipendente()
        if len(self.pdf) == 0:
            return
        self._stampa_totali()
        self.doc.build(self.pdf)

        # Risposta
        file_pdf = open(self.perc_file, 'rb')
        self.risposta.contenuto = file_pdf.read()
        self._addHead("Content-Disposition", "inline; filename=pres_%s_%s_r.pdf" %(self._ditta.codice, str(self._periodo)))
        self._repHead("Content-Type", "application/pdf")
        file_pdf.close()
        remove(self.perc_file)


    def _get_presenze_from_session(self, pres_session):
        presenze = Presenze()
        presenze.periodo = pres_session.periodo
        presenze.ditta_id = self._ditta.id
        presenze.dipendente_id = pres_session.id_dipe
        presenze.qualifica = pres_session.qualifica
        presenze.livello = pres_session.livello
        presenze.centro_costo = pres_session.centro_costo
        #presenze.cantiere = pres_session.cantiere
        presenze.pos_inail = pres_session.pos_inail
        presenze.tipo_pm = pres_session.tipo_pm
        presenze.perc_pm = pres_session.perc_pm
        presenze.evento_mp = pres_session.evento_mp
        presenze.mansione = pres_session.mansione
        presenze.note = pres_session.note
        presenze.dipendente_rel = self._q(Dipendente).filter(and_(Dipendente.id == pres_session.id_dipe, Dipendente.ditta_id == self._ditta.id)).first()
        presenze.ditta_rel = self._q(Ditta).filter(Ditta.id == self._ditta.id).first()

        for giorno_ss in pres_session.giorni:
            preg_gg = PresenzeCalend()
            preg_gg.giorno_mese = giorno_ss.giorno
            preg_gg.ore_teoriche = giorno_ss.ore_teoriche
            preg_gg.ore_lavorate = giorno_ss.ore_lavorate
            preg_gg.ore_retribuite = giorno_ss.ore_retribuite
            preg_gg.ecc = giorno_ss.ecc
            for causale_ss in giorno_ss.causali_giorno:
                causale_gg = PresenzeCalendCaus()
                causale_gg.codice = causale_ss.codice
                causale_gg.ore = causale_ss.ore
                preg_gg.calendarioCaus_rel.append(causale_gg)
            presenze.calendario_rel.append(preg_gg)

        for evento_ss in pres_session.eventi:
            evento = PresenzeEvento()
            evento.codice = evento_ss.codice
            evento.data_inizio = evento_ss.data_inizio
            evento.data_fine = evento_ss.data_fine
            evento.ore = evento_ss.ore
            presenze.eventi_rel.append(evento)

        for voce_ss in pres_session.voci:
            voce = PresenzeVoce()
            voce.prefisso = voce_ss.prefisso
            voce.voce = voce_ss.voce
            voce.suffisso = voce_ss.suffisso
            voce.dato_base = voce_ss.dato_base
            voce.costo_unitario = voce_ss.costo_unitario
            voce.perc = voce_ss.perc
            voce.importo = voce_ss.importo
            presenze.voci_rel.append(voce)

        for lav_dom_ss in pres_session.lavdom:
            lav_dom = PresenzeLavDom()
            lav_dom.data_consegna = lav_dom_ss.data_consegna
            lav_dom.ora_consegna = lav_dom_ss.ora_consegna
            lav_dom.data_riconsegna = lav_dom_ss.data_riconsegna
            lav_dom.ora_riconsegna = lav_dom_ss.ora_riconsegna
            lav_dom.descrizione = lav_dom_ss.descrizione
            lav_dom.quantita = lav_dom_ss.quantita
            presenze.lavori_dom_rel.append(lav_dom)

        return presenze


    def _stampa_testa(self):
        sezione = [Paragraph("RIASSUNTIVO &nbsp;DIPENDENT%s" %("E" if self.firma_si else "I"),
                             ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=16,
                                            alignment=TA_CENTER, spaceAfter=0.1*inch)),
                   Paragraph("dal %d al %d %s %s" %(self.dal, self.al, mesi[framData(self._periodo)[1] - 1],
                             framData(self._periodo, True)[0]),
                             ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=14,
                                            alignment=TA_CENTER, spaceAfter=0.5*inch)),
                   Paragraph("%s - %s" %(self._ditta.codice, self._ditta.descrizione),
                             ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=14, spaceAfter=0.1*inch)),
                   ]

        self.pdf.append(KeepTogether(sezione))


    def _stampa_dipendente(self):
        totali = self._calcola()
        sezione = self._stampa_nominativo(totali)
        sezione.extend(self._stampa_calendario(totali))
        self.pdf.append(KeepTogether(sezione))
        sezione = self._stampa_voci(totali)
        if sezione:
            self.pdf.append(KeepTogether(sezione))
        sezione = self._stampa_lav_dom()
        if sezione:
            self.pdf.append(KeepTogether(sezione))
        sezione = self._stampa_note()
        if sezione:
            self.pdf.extend(sezione)
        sezione = self._stampa_firma()
        if sezione:
            self.pdf.append(KeepTogether(sezione))

        # totali per ditta
        if totali['tot_gg_pres'] > 0:
            self.tot_ditta['dipti_pre'] += 1
        else:
            self.tot_ditta['dipti_ass'] += 1
        self.tot_ditta['tot_ore_lav'] += totali['tot_ore_lav']
        self.tot_ditta['tot_ore_ass'] += totali['tot_ore_ass']
        for k, v in totali['causali'].iteritems():
            if k in ['AA', 'ZZ']:
                continue
            if self.tot_ditta['causali'].has_key(k):
                self.tot_ditta['causali'][k]['ore'] += v['ore']
            else:
                self.tot_ditta['causali'][k] = {'descrizione':v['descrizione'], 'ore':v['ore']}
        for v in totali['voci']:
            key = "%s_%s_%s" %(v['codice'], str(v['costo_unitario']), str(v['perc']))
            if self.tot_ditta['voci'].has_key(key):
                self.tot_ditta['voci'][key]['dato_base'] += v['dato_base']
                self.tot_ditta['voci'][key]['importo'] += v['importo']
            else:
                self.tot_ditta['voci'][key] = {'codice':v['codice'], 'descrizione':v['descrizione'],
                                               'dato_base':v['dato_base'], 'costo_unitario':v['costo_unitario'],
                                               'perc':v['perc'], 'importo':v['importo']
                                               }


    def _calcola(self):
        causali = {}
        voci = []
        tot_ore_lav = 0
        tot_ore_ass = 0
        tot_gg_ass = 0
        tot_gg_pres = 0
        giorni = [g for g in self.presenze.calendario_rel if g.giorno_mese in range(self.dal, self.al + 1)]

        # causali
        for giorno in giorni:
            if giorno.giorno_mese not in self.giorni_in_forza:
                continue
            tot_ore_str = 0
            for causale in giorno.calendarioCaus_rel:
                if causale.codice in ['AA', 'ZZ']:
                    causali[causale.codice] = {}
                    continue
                causale_db = self._q(Causale).filter(and_(Causale.codice == causale.codice,
                                                          Causale.periodo_inizio <= self._periodo,
                                                          Causale.periodo_fine >= self._periodo,
                                                          )).first()
                if not causale_db:
                    continue
                # totali ore/giorni causali in calendario
                if causali.has_key(causale.codice):
                    causali[causale.codice]['ore'] += causale.ore or 0
                    if giorno.giorno_mese not in causali[causale.codice]['giorni']:
                        causali[causale.codice]['giorni'].append(giorno.giorno_mese)
                else:
                    causali[causale.codice] = {'descrizione':causale_db.descrizione,
                                               'tipo':causale_db.tipo,
                                               'ore':causale.ore or 0,
                                               'giorni':[]}
                # totale ore straordinari
                if causale_db.tipo == "A":
                    tot_ore_ass = self._operazione("+", tot_ore_ass, causale.ore)
                elif causale_db.is_ore_ordinarie:
                    tot_ore_str = self._operazione("+", tot_ore_str, causale.ore)
            # totale ore lavorate
            tot_ore_lav = self._operazione("+", tot_ore_lav, giorno.ore_lavorate)
            tot_ore_lav = self._operazione("+", tot_ore_lav, tot_ore_str)
            # totale giorni lavorati e assenti
            if giorno.ore_lavorate or tot_ore_str:
                tot_gg_pres += 1
            elif giorno.ore_teoriche and giorno.ecc != "FG":
                tot_gg_ass += 1

        # voci
        for voce in self.presenze.voci_rel:
            codice = (voce.prefisso or ' ') + voce.voce + (voce.suffisso or ' ')
            try:
                descrizione = self._q(Voce.descrizione).filter(and_(Voce.codice == voce.voce,
                                                                    Voce.periodo_inizio <= self._periodo,
                                                                    Voce.periodo_fine >= self._periodo
                                                                    )).first()[0]
            except:
                continue
            voci.append({'codice':codice,
                         'descrizione':descrizione,
                         'dato_base':voce.dato_base or 0,
                         'costo_unitario':voce.costo_unitario or 0,
                         'perc':voce.perc or 0,
                         'importo':voce.importo or 0,
                        })
        voci.sort()
        return {'causali':causali, 'tot_ore_lav':tot_ore_lav, 'tot_ore_ass':tot_ore_ass,
                'tot_gg_ass':tot_gg_ass, 'tot_gg_pres':tot_gg_pres, 'voci':voci}


    def _stampa_nominativo(self, totali):
        sezione = []
        sezione.extend([_Linea((0, 0, 7.9*inch, 0)),
                        Spacer(0*inch, 0.1*inch),
                        Paragraph('%s - %s %s' %(self.dipte.codice, self.dipte.cognome, self.dipte.nome),
                                  ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=11, spaceAfter=0.1*inch)),
                        ])
        if 'AA' in [k for k in totali['causali'].iterkeys()]:
            sezione.append(Paragraph('%sassunt%s il %s' %(8*"&nbsp;", "a" if self.dipte.sesso == "F" else "o", formatData(self.data_ass, "str", "%d/%m/%Y")),
                                     ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=11, spaceAfter=0.1*inch)))
        if 'ZZ' in [k for k in totali['causali'].iterkeys()]:
            sezione.append(Paragraph('%scessat%s il %s' %(8*"&nbsp;", "a" if self.dipte.sesso == "F" else "o", formatData(self.data_cess, "str", "%d/%m/%Y")),
                                     ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=11, spaceAfter=0.1*inch)))
        return sezione


    def _stampa_calendario(self, totali):
        if isinstance(self._preferenze.TOT_GG_GIUST, list):
            cau_in_gg = self._preferenze.TOT_GG_GIUST
        else:
            cau_in_gg = [self._preferenze.TOT_GG_GIUST]
        sezione = []
        # causali
        if self.causali_si and totali['causali']:
            dati = [['Calendario:'],
                    ['Codice', 'Descrizione', 'Ore']]
            for k in sorted(totali['causali'].iterkeys()):
                if k not in ['AA', 'ZZ'] and k not in cau_in_gg:
                    cau_diz = totali['causali'][k]
                    dati.append([k, cau_diz['descrizione'], ('%.2f' % cau_diz['ore']).replace(".", ",")])
            # tabellamento causali
            stile = [('GRID', (0, 0), (-1, -1), 0, colors.black),
                     ('SPAN', (0, 0), (-1, 0)),
                     ('LINEBEFORE', (0, 0), (3, 0), 0, colors.white),
                     ('LINEABOVE', (0, 0), (-1, 0), 0, colors.white),
                     ('LINEABOVE', (0, 1), (-1, 1), 0, colors.black),
                     ('LEFTPADDING', (0, 0), (-1, -1), 2),
                     ('RIGHTPADDING', (0, 0), (-1, -1), 2),
                     ('ALIGN', (0, 1), (-1, 1), 'CENTER'),
                     ('ALIGN', (0, 2), (0, -1), 'CENTER'),
                     ('ALIGN', (2, 2), (2, -1), 'RIGHT'),
                     ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                     ('FONT', (0, 0), (0, 0), 'Times-Italic', 11),
                     ('FONT', (0, 1), (-1, 1), 'Helvetica-Bold', 10),
                     ('FONT', (0, 2), (-1, -1), 'Helvetica', 10),
                    ]
            cau_diz = ""
            for k in sorted(totali['causali'].iterkeys()):
                if k not in ['AA', 'ZZ'] and k in cau_in_gg:
                    if not cau_diz:
                        dati.append(['', '', 'Giorni'])
                        stile.extend([('ALIGN', (2, (len(dati) - 1)), (2, (len(dati) - 1)), 'CENTER'),
                                      ('FONT', (2, (len(dati) - 1)), (2, (len(dati) - 1)), 'Helvetica-Bold', 10),
                                     ])
                    cau_diz = totali['causali'][k]
                    dati.append([k, cau_diz['descrizione'], len(cau_diz['giorni'])])
            tabella = TableRL(dati,
                              [0.7*inch, 5.8*inch, 0.7*inch],
                              len(dati)*[0.2*inch],
                              hAlign = "CENTER",
                              style = stile
                              )
            sezione.extend([tabella, Spacer(0*inch, 0.1*inch)])

        # totali
        tot_ore_lav = ""
        if totali['tot_ore_lav']:
            tot_ore_lav = ("%.2f" % totali['tot_ore_lav']).replace(".", ",")
        tot_gg_pres = ""
        if totali['tot_gg_pres']:
            tot_gg_pres = ("%.2f" % totali['tot_gg_pres']).replace(".", ",")
        tot_gg_ass = ""
        if totali['tot_gg_ass']:
            tot_gg_ass = ("%.2f" % totali['tot_gg_ass']).replace(".", ",")
        dati = [['Totale ore lavorate', tot_ore_lav],
                ['Totale giorni presenza', tot_gg_pres],
                ['Totale giorni assenza', tot_gg_ass],
                ]
        # tabellamento totali
        tabella = TableRL(dati,
                          [6.5*inch, 0.7*inch],
                          len(dati)*[0.2*inch],
                          hAlign = "CENTER",
                          style = [('GRID', (0, 0), (-1, -1), 0, colors.black),
                                   ('LEFTPADDING', (0, 0), (-1, -1), 2),
                                   ('RIGHTPADDING', (0, 0), (-1, -1), 2),
                                   ('ALIGN', (1, 0), (1, -1), 'RIGHT'),
                                   ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                                   ('FONT', (0, 0), (-1, -1), 'Helvetica', 10),
                                   ]
                          )
        sezione.append(tabella)

        return sezione


    def _stampa_voci(self, totali):
        if not self.voci_si or not totali['voci']:
            return
        sezione = [Spacer(0*inch, 0.2*inch)]

        # voci
        dati = [['Voci:'],
                ['Codice', 'Descrizione', 'Dato base', 'C. Unitario', '%', 'Importo']]
        for v in totali['voci']:
            dati.append([v['codice'], v['descrizione'],
                         ("%.2f" %(v['dato_base'])).replace(".", ",") if v['dato_base'] else '',
                         ("%.2f" %(v['costo_unitario'])).replace(".", ",") if v['costo_unitario'] else '',
                         ("%.2f" %(v['perc'])).replace(".", ",") if v['perc'] else '',
                         ("%.2f" %(v['importo'])).replace(".", ",") if v['importo'] else '',
                        ])
        # tabellamento voci
        tabella = TableRL(dati,
                          1*[0.7*inch] + 1*[3.1*inch] + 2*[0.9*inch] + 1*[0.6*inch] + 1*[1*inch],
                          len(dati)*[0.2*inch],
                          hAlign = "CENTER",
                          style = [('GRID', (0, 0), (-1, -1), 0, colors.black),
                                   ('LINEBEFORE', (0, 0), (6, 0), 0, colors.white),
                                   ('LINEABOVE', (0, 0), (-1, 0), 0, colors.white),
                                   ('LINEABOVE', (0, 1), (-1, 1), 0, colors.black),
                                   ('LEFTPADDING', (0, 0), (-1, -1), 2),
                                   ('RIGHTPADDING', (0, 0), (-1, -1), 2),
                                   ('ALIGN', (0, 1), (-1, 1), 'CENTER'),
                                   ('ALIGN', (0, 2), (0, -1), 'CENTER'),
                                   ('ALIGN', (2, 2), (3, -1), 'RIGHT'),
                                   ('ALIGN', (4, 2), (4, -1), 'CENTER'),
                                   ('ALIGN', (5, 2), (5, -1), 'RIGHT'),
                                   ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                                   ('FONT', (0, 0), (0, 0), 'Times-Italic', 11),
                                   ('FONT', (0, 1), (-1, 1), 'Helvetica-Bold', 10),
                                   ('FONT', (0, 2), (-1, -1), 'Helvetica', 10),
                                   ]
                          )
        sezione.append(tabella)

        return sezione


    def _stampa_lav_dom(self):
        if not self.lav_dom_si or not self.presenze.lavori_dom_rel:
            return
        sezione = [Spacer(0*inch, 0.2*inch)]

        # lavori a domicilio
        dati = [['Lavori a domicilio:'],
                ['Data - Ora cons.', 'Data - Ora ricons.', 'Descrizione', 'Quantita\'']]
        for lavoro in self.presenze.lavori_dom_rel:
            dati.append(['%s - %.2f' %(formatData(lavoro.data_consegna, "str", "%d/%m/%Y"), lavoro.ora_consegna),
                         '%s - %.2f' %(formatData(lavoro.data_riconsegna, "str", "%d/%m/%Y"), lavoro.ora_riconsegna),
                         lavoro.descrizione,
                         lavoro.quantita])
        # tabellamento lavori a domicilio
        tabella = TableRL(dati,
                          1*[1.4*inch] + 1*[1.4*inch] + 1*[3*inch] + 1*[1.4*inch],
                          len(dati)*[0.2*inch],
                          hAlign = "CENTER",
                          style = [('GRID', (0, 0), (-1, -1), 0, colors.black),
                                   ('LINEBEFORE', (0, 0), (4, 0), 0, colors.white),
                                   ('LINEABOVE', (0, 0), (-1, 0), 0, colors.white),
                                   ('LINEABOVE', (0, 1), (-1, 1), 0, colors.black),
                                   ('LEFTPADDING', (0, 0), (-1, -1), 2),
                                   ('RIGHTPADDING', (0, 0), (-1, -1), 2),
                                   ('ALIGN', (0, 1), (-1, 1), 'CENTER'),
                                   ('ALIGN', (0, 2), (1, -1), 'CENTER'),
                                   ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                                   ('FONT', (0, 0), (0, 0), 'Times-Italic', 11),
                                   ('FONT', (0, 1), (-1, 1), 'Helvetica-Bold', 10),
                                   ('FONT', (0, 2), (-1, -1), 'Helvetica', 10),
                                   ]
                          )
        sezione.append(tabella)

        return sezione


    def _stampa_note(self):
        if not self.note_si or not self.presenze.note:
            return
        sezione = [Spacer(0*inch, 0.2*inch)]
        prima = True
        for riga in self.presenze.note.split("\n"):
            sezione.append(Paragraph("%s %s" %("*" if prima else "&nbsp"*2, riga),
                                     ParagraphStyle(name='', fontName='Times-Italic', fontSize=11, spaceAfter=0.1*inch))
                          )
            prima = False

        return sezione


    def _stampa_firma(self):
        if not self.firma_si:
            return
        sezione = [Paragraph("Firma per accettazione e controllo",
                             ParagraphStyle(name='', fontName='Times-Italic', fontSize=11, leftIndent=5*inch,
                                            spaceBefore=1*inch, spaceAfter=0.4*inch))
                   ]
        linee = []
        x1 = x2 = 0
        for i in range(1, 50):
            if x1 == 0:
                x1 = 5
            else:
                x1 = x2 + 0.03
            x2 = x1 + 0.02
            linee.append((x1*inch, 0, x2*inch, 0))
        sezione.append(_Linea(linee))

        return sezione


    def _stampa_totali(self):
        self.pdf.append(PageBreak())
        sezione = [Paragraph("RIASSUNTIVO &nbsp;DITTA",
                             ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=16,
                                            alignment=TA_CENTER, spaceAfter=0.1*inch)),
                   Paragraph("dal %d al %d %s %s" %(self.dal, self.al, mesi[framData(self._periodo)[1] - 1],
                             framData(self._periodo, True)[0]),
                             ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=14,
                                            alignment=TA_CENTER, spaceAfter=0.5*inch)),
                   Paragraph("%s - %s" %(self._ditta.codice, self._ditta.descrizione),
                             ParagraphStyle(name='', fontName='Helvetica-Bold', fontSize=14, spaceAfter=0.1*inch)),
                   _Linea((0, 0, 7.9*inch, 0)),
                   Spacer(0*inch, 0.1*inch),
                   Paragraph("Dipendenti elaborati: %s (%s)" %(str(len(self._dipti_id)), self.applica),
                             ParagraphStyle(name='', fontName='Helvetica', fontSize=12, spaceAfter=0.2*inch)),
                   ]

        # causali
        if self.causali_si and self.tot_ditta['causali']:
            dati = [['Calendario:'],
                    ['Codice', 'Descrizione', 'Ore']]
            for k in sorted(self.tot_ditta['causali'].iterkeys()):
                if k not in ['AA', 'ZZ']:
                    cau_diz = self.tot_ditta['causali'][k]
                    dati.append([k, cau_diz['descrizione'], ('%.2f' % cau_diz['ore']).replace(".", ",")])
            # tabellamento causali
            tabella = TableRL(dati,
                              [0.7*inch, 5.8*inch, 0.7*inch],
                              len(dati)*[0.2*inch],
                              hAlign = "CENTER",
                              style = [('GRID', (0, 0), (-1, -1), 0, colors.black),
                                       ('SPAN', (0, 0), (-1, 0)),
                                       ('LINEBEFORE', (0, 0), (3, 0), 0, colors.white),
                                       ('LINEABOVE', (0, 0), (-1, 0), 0, colors.white),
                                       ('LINEABOVE', (0, 1), (-1, 1), 0, colors.black),
                                       ('LEFTPADDING', (0, 0), (-1, -1), 2),
                                       ('RIGHTPADDING', (0, 0), (-1, -1), 2),
                                       ('ALIGN', (0, 1), (-1, 1), 'CENTER'),
                                       ('ALIGN', (0, 2), (0, -1), 'CENTER'),
                                       ('ALIGN', (2, 2), (2, -1), 'RIGHT'),
                                       ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                                       ('FONT', (0, 0), (0, 0), 'Times-Italic', 11),
                                       ('FONT', (0, 1), (-1, 1), 'Helvetica-Bold', 10),
                                       ('FONT', (0, 2), (-1, -1), 'Helvetica', 10),
                                       ]
                              )
            sezione.extend([tabella, Spacer(0*inch, 0.1*inch)])

        # totali
        dati = [['N. dipendenti presenti', self.tot_ditta['dipti_pre'] or ''],
                ['N. dipendenti assenti', self.tot_ditta['dipti_ass'] or ''],
                ]
        ore = ""
        if self.tot_ditta['tot_ore_lav']:
            ore = ('%.2f' % self.tot_ditta['tot_ore_lav']).replace(".", ",")
        dati.append(['Totale ore lavorate', ore])
        ore = ""
        if self.tot_ditta['tot_ore_ass']:
            ore = ('%.2f' % self.tot_ditta['tot_ore_ass']).replace(".", ",")
        dati.append(['Totale ore assenza', ore])
        # tabellamento totali
        tabella = TableRL(dati,
                          [6.5*inch, 0.7*inch],
                          len(dati)*[0.2*inch],
                          hAlign = "CENTER",
                          style = [('GRID', (0, 0), (-1, -1), 0, colors.black),
                                   ('LEFTPADDING', (0, 0), (-1, -1), 2),
                                   ('RIGHTPADDING', (0, 0), (-1, -1), 2),
                                   ('ALIGN', (1, 0), (1, -1), 'RIGHT'),
                                   ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                                   ('FONT', (0, 0), (-1, -1), 'Helvetica', 10),
                                   ]
                          )
        sezione.append(tabella)

        # voci
        if self.voci_si and self.tot_ditta['voci']:
            sezione.append(Spacer(0*inch, 0.2*inch))
            dati = [['Voci:'],
                    ['Codice', 'Descrizione', 'Dato base', 'C. Unitario', '%', 'Importo']]
            for k in sorted(self.tot_ditta['voci'].iterkeys()):
                v = self.tot_ditta['voci'][k]
                dati.append([v['codice'], v['descrizione'],
                            ("%.2f" %(v['dato_base'])).replace(".", ",") if v['dato_base'] else '',
                            ("%.2f" %(v['costo_unitario'])).replace(".", ",") if v['costo_unitario'] else '',
                            ("%.2f" %(v['perc'])).replace(".", ",") if v['perc'] else '',
                            ("%.2f" %(v['importo'])).replace(".", ",") if v['importo'] else '',
                            ])
            # tabellamento voci
            tabella = TableRL(dati,
                              1*[0.7*inch] + 1*[3.1*inch] + 2*[0.9*inch] + 1*[0.6*inch] + 1*[1*inch],
                              len(dati)*[0.2*inch],
                              hAlign = "CENTER",
                              style = [('GRID', (0, 0), (-1, -1), 0, colors.black),
                                       ('LINEBEFORE', (0, 0), (6, 0), 0, colors.white),
                                       ('LINEABOVE', (0, 0), (-1, 0), 0, colors.white),
                                       ('LINEABOVE', (0, 1), (-1, 1), 0, colors.black),
                                       ('LEFTPADDING', (0, 0), (-1, -1), 2),
                                       ('RIGHTPADDING', (0, 0), (-1, -1), 2),
                                       ('ALIGN', (0, 1), (-1, 1), 'CENTER'),
                                       ('ALIGN', (0, 2), (0, -1), 'CENTER'),
                                       ('ALIGN', (2, 2), (3, -1), 'RIGHT'),
                                       ('ALIGN', (4, 2), (4, -1), 'CENTER'),
                                       ('ALIGN', (5, 2), (5, -1), 'RIGHT'),
                                       ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
                                       ('FONT', (0, 0), (0, 0), 'Times-Italic', 11),
                                       ('FONT', (0, 1), (-1, 1), 'Helvetica-Bold', 10),
                                       ('FONT', (0, 2), (-1, -1), 'Helvetica', 10),
                                       ]
                              )
            sezione.append(tabella)

        self.pdf.append(KeepTogether(sezione))


    def _operazione(self, ope, add_1=0, add_2=0):
        if not add_1:
            add_1 = 0
        if not add_2:
            add_2 = 0
        if self._impostazioni.is_sessantesimi:
            add_1 = converti_ore(add_1, False)
            add_2 = converti_ore(add_2, False)
        if ope == "+":
            ore = add_1 + add_2
        elif ope == "-":
            ore = add_1 - add_2
        elif ope == "*":
            ore = add_1 * add_2
        elif ope == "/":
            ore =  add_1 / add_2
        elif ope == "%":
            ore = add_1 % add_2
        else:
            return None
        if self._impostazioni.is_sessantesimi:
            return converti_ore(ore, True)
        else:
            return ore


class _Linea(Flowable):
    def __init__(self, linee):
        if type(linee) == TupleType:
            linee = [linee]
        self.linee = []
        for linea in linee:
            if type(linea) == TupleType:
                self.linee.append(linea)
        if not self.linee:
            return None

    def wrap(self, *args):
        return (0*inch, 0*inch)

    def draw(self):
        canvas = self.canv
        canvas.lines(self.linee)


def dogentask(sessiondb, session, request):
    return StampaPresenzeRiass(sessiondb, session, request).doTask()

