# -*- coding: utf-8 -*-
from waonline.core.tabelle import *
from sqlalchemy.types import Float


class Presenze(Base):
    """
        Classe per la tabella delle presenze
    """
    __tablename__ = "presenze"
    __table_args__ = (UniqueConstraint('periodo', 'ditta_id', 'dipendente_id'), {'useexisting':True})
    id = Column(Integer, primary_key=True, autoincrement=True)
    periodo = Column(Integer)
    ditta_id = Column(Integer, ForeignKey('ditte.id', onupdate='CASCADE', ondelete='SET NULL'))
    dipendente_id = Column(Integer, ForeignKey('dipendenti.id', onupdate='CASCADE', ondelete='SET NULL'))
    dipendente_cod = Column(String(5))
    dipendente_is_collab = Column(Boolean)
    qualifica = Column(String(4))
    livello = Column(String(4))
    centro_costo = Column(String(12))
    pos_inail = Column(String(10))
    tipo_pm = Column(String(1))
    perc_pm = Column(Float())
    ultima_sett = Column(Integer)
    evento_mp = Column(String(4))
    ore_sett = Column(Float())
    mansione = Column(String(100))
    note = Column(String(1000))
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    dipendente_rel = relationship('Dipendente', order_by='Dipendente.id')
    ditta_rel = relationship('Ditta', order_by='Ditta.id')
    # Relazioni one-to-many
    calendario_rel = relationship('PresenzeCalend', order_by='PresenzeCalend.id', lazy='joined')
    eventi_rel = relationship('PresenzeEvento', order_by='PresenzeEvento.id', lazy='joined')
    lavori_dom_rel = relationship('PresenzeLavDom', order_by='PresenzeLavDom.id')
    voci_rel = relationship('PresenzeVoce', order_by='PresenzeVoce.id')


class PresenzeCalend(Base):
    __tablename__ = "presenze_calend"
    __table_args__ = (UniqueConstraint('presenze_id', 'giorno_mese'), {})
    id = Column(Integer, primary_key=True, autoincrement=True)
    presenze_id = Column(Integer, ForeignKey('presenze.id', onupdate='CASCADE', ondelete='CASCADE'))
    giorno_mese = Column(Integer)
    giorno_sett = Column(String(2))
    ore_teoriche = Column(Float())
    ore_lavorate = Column(Float())
    ore_retribuite = Column(Float())
    ecc = Column(String(2))
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    presenze_rel = relationship('Presenze', order_by='Presenze.id')
    # Relazioni one-to-many
    calendarioCaus_rel = relationship('PresenzeCalendCaus', order_by='PresenzeCalendCaus.id', lazy='joined')


class PresenzeCalendCaus(Base):
    __tablename__ = "presenze_calend_caus"
    id = Column(Integer, primary_key=True, autoincrement=True)
    calendario_id = Column(Integer, ForeignKey('presenze_calend.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(4))
    ore = Column(Float())
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    calendario_rel = relationship('PresenzeCalend', order_by='PresenzeCalend.id')


class PresenzeEvento(Base):
    __tablename__ = "presenze_eventi"
    id = Column(Integer, primary_key=True, autoincrement=True)
    presenze_id = Column(Integer, ForeignKey('presenze.id', onupdate='CASCADE', ondelete='CASCADE'))
    codice = Column(String(4))
    data_inizio = Column(Integer)
    data_fine = Column(Integer)
    ore = Column(Float())
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    presenze_rel = relationship('Presenze', order_by='Presenze.id')


class PresenzeLavDom(Base):
    __tablename__ = "presenze_lav_dom"
    id = Column(Integer, primary_key=True, autoincrement=True)
    presenze_id = Column(Integer, ForeignKey('presenze.id', onupdate='CASCADE', ondelete='CASCADE'))
    data_consegna = Column(Integer)
    ora_consegna = Column(Float())
    data_riconsegna = Column(Integer)
    ora_riconsegna = Column(Float())
    descrizione = Column(String(100))
    quantita = Column(String(50))
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    presenze_rel = relationship('Presenze', order_by='Presenze.id')


class PresenzeVoce(Base):
    __tablename__ = "presenze_voci"
    id = Column(Integer, primary_key=True, autoincrement=True)
    presenze_id = Column(Integer, ForeignKey('presenze.id', onupdate='CASCADE', ondelete='CASCADE'))
    prefisso = Column(String(1))
    voce = Column(String(4))
    suffisso = Column(String(1))
    dato_base = Column(Float())
    costo_unitario = Column(Float())
    perc = Column(Float())
    importo = Column(Float())
    data_modifica = Column(DateTime, onupdate=datetime.now)
    utente_modifica = Column(String(50))
    # Relazioni many-to-one
    presenze_rel = relationship('Presenze', order_by='Presenze.id')
