# -*- coding: utf-8 -*-
"""
Utenti
=======================================

vedi :ref:`tabelle <tbl-user>`

"""
__author__ = "Luca"
__email__ = "luca.z@essepaghe.it"
__web_site__ = "http://waonline.essepaghe.it"
__version__ = "1.0.0"
__date__ = "2011-12-20"
__app_name__ = "utenti"

import os
import smtplib
import time
from datetime import datetime
from email.mime.text import MIMEText

from waonline.core.rbase import RBase
from waonline.utils import objutils
from waonline.utils import utility
from waonline.core.wadecorators import WaAuth
from waonline.core.ete_dumbobj import EteDumbObj
from waonline.core.users.tabelle.new_utenti import NewUtente, StudioCodeError,\
        NewUtentePref
from waonline import conf
from waonline.core import sessions

from waonline.apps.anagrafiche.tabelle.ditte import Ditta

def get_passwd():
    """ Recupera la password nel file indicato, se presente """
    try:
        return open(conf.PASSWD_PATH).read()
    except:
        return None


class Main(RBase):
    """ Gestione Utenti """

    def _post_init_(self):
        self._repHead("Content-Type", "application/javascript")

    def _imposta_session(self):
        """ Imposta le variabili di sessione per il login """
        self.session['user'] = self.user.id
        if self.user.studio_codice:
            self.session['cod_studio'] = self.user.studio_codice
        else:
            try:
                del self.session['cod_studio']
            except:
                pass
        self.session['periodo_attuale'] = datetime.now()
        self.session['login_datetime'] = datetime.now()

    @sessions.require_lock
    @WaAuth()
    def login(self, username, passwd):
        """ Funzione per il login """
        self._repHead("Content-Type", "text/html")
        self.info("Utente:%s" % username)
        self.info("Password:%s" % passwd)
        if not self.user:
            if get_passwd() == passwd.upper():
                self.user = self._um.get_by_username(username)
            else:
                self.user = self._um.login(username, passwd.upper())
            if self.user:
                prefs = self._preferenze
                self.session['prev_user'] = []
                self.session['tema'] = prefs.get('tema', conf.DEFAULT_TEMPLATE)
                self.session['skin'] = prefs.get('skin', conf.SKIN_NAME)
                self._imposta_session()
                self._addHead("X-Wao-Auth", "OK")
            else:
                self._addHead("X-Wao-Auth", "Failed")
        else:
            self._addHead("X-Wao-Auth", "Present")
        self.risposta.contenuto = '<script type="text/javascript">'\
            'document.location="/";</script>'

    @sessions.require_lock
    @WaAuth()
    def trasformazione(self, id_utente=WaAuth.INT):
        """ Funzione per cambio utente """
        if not id_utente in self.user.id_children:
            self._addErr("Non si hanno i diritti per accedere con questo "\
                "utente")
            self.risposta.contenuto = objutils.toJSON(False)
            return
        self.session['prev_user'].append(self.user.id)
        self.user = self.user.child_by_id(id_utente)
        self._imposta_session()
        self.risposta.contenuto = objutils.toJSON(True)
        self._addStato("Passaggio a %s" % self.user.username)

    @sessions.require_lock
    def ritorna(self):
        """ Ritorna dal cambio utente """
        if len(self.session['prev_user']):
            self.user = self.user.get_by_id(self.session['prev_user'].pop())
            self._imposta_session()
            self.risposta.contenuto = objutils.toJSON(True)
            self._addStato("Passaggio a %s" % self.user.username)
        else:
            self._addErr("Impossibile ritornare all'utente originale")
            self.risposta.contenuto = objutils.toJSON(False)

    @property
    def is_from_admin(self):
        """ verifica se uno degli utenti usati e' amministratore """
        return self.user.ADMIN in [u.livello for u in \
                self.user.get_by_id(self.session['prev_user'])] +\
                [self.user.livello]

    @WaAuth()
    def cambio_password(self, npass=WaAuth.UPPER, cpass=WaAuth.UPPER,
            apass=WaAuth.UPPER, id_utente=WaAuth.INT):
        """ cambia password all'utente """
        user = self.user.child_by_id(id_utente)
        if not user and id_utente == self.user.id:
            user = self.user
        if not user:
            self._addErr("Utente inesistente o non discendente di %s" %\
                    self.user.username)
            return
        if npass != cpass:
            self._addErr("La nuova password e la password di conferma non "\
                "corrispondono")
        elif not self.user.check_password(apass.upper()) and \
                not self.is_from_admin:
            self._addErr("La password attuale non corrisponde")
        else:
            user.password = npass.upper()
            user.commit()
            self._addStato("Password per l'utente %s aggiornata" %\
                    user.username)

    @WaAuth()
    def crea_utente(self, username=WaAuth.UPPER, descrizione=WaAuth.STR,
            passwd=WaAuth.UPPER, cpasswd=WaAuth.UPPER, livello=WaAuth.INT,
            genitore_id=WaAuth.INT, codice_utente=None):
        """ Crea un nuovo utente """
        if passwd != cpasswd:
            self._addErr('La password e la password di conferma non "\
                "corrispondono.')
            return
        if codice_utente:
            codice_utente = codice_utente.upper().zfill(3)
        try:
            user = self.user.create_user(username, passwd, livello=livello,
                    genitore_id=genitore_id, codice=codice_utente,
                    descrizione=descrizione)
        except StudioCodeError, err:
            self._addErr(str(err))
            return
        if isinstance(user, NewUtente):
            if user.livello == user.USTUDIO:
                path_studio = os.path.join(conf.DEFAULT_DATA_PATH,
                        user.studio_codice)
                utility.chk_make_dir(path_studio)
                utility.chk_make_dir(path_studio + '/invia')
                utility.chk_make_dir(path_studio + '/ricevi')
            self._addStato("Aggiunto l'utente %s" % user.username)
        elif user==self.user.USER_EXISTS:
            self._addErr('Esiste gia un utente %s' % username)
        elif user==self.user.USER_PARENT_ERROR:
            self._addErr('Genitore non valido')
        elif user==self.user.USER_LEVEL_ERROR:
            self._addErr("Il livello dell'utente deve essere inferiore a "\
                "quello del genitore.")
        elif user==self.user.USER_NO_LEVEL:
            self._addErr('Il livello utente non è valido.')

    @WaAuth()
    def elimina_utente(self, id_utente=WaAuth.INT):
        """ elimina un utente """
        user = self.user.child_by_id(id_utente)
        if user:
            self.user.delete_user(user)
            self._addStato("Utente eliminato")
        else:
            self._addErr("Utente %s non trovato" % id_utente)

    @WaAuth()
    def crea_ditta(self, descrizione, codice_ditta=WaAuth.UPPER,
            id_utente=WaAuth.INT):
        """ Crea una nuova ditta """
        genitore = self.user.child_by_id(id_utente)
        if not genitore:
            self._addErr("L'utente indicato come genitore per la ditta non "\
                "è valido.")
            return
        if genitore.livello != self.user.UAZIENDA:
            self._addErr("Il genitore deve essere un <b>Utente Ditta</b>.")
            return
        ditta = genitore.ditta_by_codice(codice_ditta)
        if ditta:
            self._addErr("Esiste gia' una ditta con il codice <b>%s</b> "\
                "(%s) per l'utente <b>%s</b>" % (ditta.codice,
                    ditta.descrizione, genitore.username))
            return
        new_ditta = genitore.create_ditta(codice=codice_ditta,
                descrizione=descrizione)
        if new_ditta == self.user.USER_CREATED:
            self._addStato('Ditta <b>%s</b> (%s) inserita con successo' % \
                    (new_ditta.obj.codice, new_ditta.obj.descrizione))
            return
        elif new_ditta == self.user.USER_EXISTS:
            html = "<ul>"
            for user in new_ditta.obj.genitori_rel:
                html += "<li>%s - %s</li>" % (user.username, user.descrizione)
            html += "</ul>"
            msg = ("<p>Esiste gia una ditta con codice <b>%s</b> (%s) "\
                "legata agli utenti: %s</p><p>Si desidera legarla anche "\
                "all'utente <b>%s</b> ?</p>" % (new_ditta.obj.codice,
                    new_ditta.obj.descrizione, html, genitore.username))
            self._addConferma(msg, "sn")
            data = {"id_ditta": new_ditta.obj.id,
                    "id_utente": genitore.id,
                    "username_ditta": new_ditta.obj.username,
                    "username_utente": genitore.username}
            self.risposta.contenuto = data
        elif new_ditta == self.user.USER_PARENT_ERROR:
            #: qui non dovrebbe mai arrivare
            self._addErr("Errore recuperando il genitore.")

    @WaAuth()
    def elimina_ditta(self, id_ditta=WaAuth.INT, id_utente=WaAuth.INT):
        """ Elimina una ditta """
        user = self.user.child_by_id(id_utente)
        self.info("ELIMINA DITTA")
        self.info("user %s" % str(user))
        if not user:
            self._addErr("Utente di riferimento non valido")
            return
        ditta = user.ditta_by_id(id_ditta)
        self.info("id_ditta:%s" % id_ditta)
        self.info("ditta:%s" % str(ditta))
        if not ditta:
            self._addErr("Ditta non valida")
            return
        try:
            ditta.genitori_rel.remove(user)
            ditta.commit()
            if not ditta.genitori_rel:
                self.user.delete_user(ditta)
            #: TODO: eliminare le ditte orfane
            # self.user.delete_user([d.id for d in self.user.get_orphans(0,0)])
            self._addStato("Ditta eliminata con successo")
        except:
            self._addErr("Errore eliminando la ditta.")

    @WaAuth()
    def copia_ditte(self, id_utente=WaAuth.INT,
            id_aziende={'err': True, 'default': [], 'forcelist': True, 'tipo': int}):
        """ Copia ditte """
        user = self.user.child_by_id(id_utente)
        ca = False
        if user:
            if user.livello != user.UAZIENDA:
                self._addErr("Impossibile inserire ditte per questo tipo "\
                        "di utente")
                return
            genitore = user.get_genitore()
            for id_azienda in id_aziende:
                #azienda = self.utenti_manager.get_user(id_azienda)
                ditta = genitore.ditta_by_id(id_azienda)
                ca = False
                if ditta:
                    if not ditta.id in user.id_ditte :
                        user.figli_rel.append(ditta)
                        ca = True
                    else:
                        self._addMess("L'azienda <b>%s - %s</b> esiste gia' "\
                            "per l'untente <b>%s</b>" % (
                                ditta.codice, ditta.descrizione,
                            user.username))
            if ca:
                user.commit()
                self._addStato('Ditte copiate con successo')
            else:
                self._addStato('Nessuna ditta copiata')
        else:
            self._addErr("L'utente non esiste")

    @WaAuth()
    def aggiorna_utente(self, dati=WaAuth.JEDO):
        """ Aggiorna i dati per l'utente """
        if dati:
            dati_utente = dati.get('utente')
            if dati_utente:
                if not dati_utente.id in self.user.id_children + \
                        [self.user.id]:
                    self._addErr("Non si dispongono dei diritti per "\
                            "modificare questo utente.")
                    return
                user = self.user.child_by_id(dati_utente.id) or self.user
                dati_utente.update(dati.get('autorizzazioni',{}))
                user.update(**dati_utente)
                ditte = EteDumbObj(dati.get('aziende', {}))
                if ditte:
                    for ditta in ditte:
                        try:
                            user.child_by_id(ditta.id).update(**ditta)
                        except:
                            pass
                if dati.azienda_id in user.id_children + [user.id]:
                    ditta = user.child_by_id(dati.azienda_id) or self.user
                    if 'preferenze' in dati:
                        l = [int(l.get('id', 0)) for l in dati.preferenze]
                        for apref in ditta.preferenze:
                            if not apref.id in l:
                                ditta.preferenze_rel.remove(apref)
                        for preferenza in dati.preferenze:
                            if 'id' in preferenza:
                                record_preferenza = ditta._q(NewUtentePref).get(
                                                preferenza.id)
                                record_preferenza.nome = preferenza.nome
                                record_preferenza.valore = preferenza.valore
                            else:
                                record_preferenza = NewUtentePref(
                                        preferenza.nome, preferenza.valore)
                                record_preferenza.genitore_id = user.id
                                ditta.preferenze_rel.append(record_preferenza)
                            try:
                                ditta.sessiondb.commit()
                            except Exception, e:
                                self._addErr("Errore %s" % e)
                else:
                    if user:
                        self._addErr("L'azienda non appartiene "\
                            "all'utente <b>%s</b> impossibile apportare "\
                            "modifiche" % user.username)
                try:
                    info = objutils.getObjInfo(user,
                        includi=['username', 'codice', 'descrizione', 'id',
                            'livello', 'genitore_id'])
                    self._addStato('Utente aggiornato.')
                    self.risposta.contenuto = info
                except Exception, e:
                    self._addErr('Errore: %s' % e)
            else:
                self._addErr("L'utente non ti appartiene, "\
                        "impossibile apportare modifiche")
        else:
            self._addErr("Dati dell'utente mancanti")

    def user_logout(self):
        """ logout dell'utente """
        if  'user' in self.session:
            self.session.delete()
        self.risposta.contenuto = '<script type="text/javascript">'\
            'document.location="/";</script>'
        self._remHead('Content-Type')
        self._addHead('Location', '/')

    def getUsersListJ(self):
        self.risposta.contenuto = objutils.getJsonObj(
            [f for f in self.user.children if f.livello > self.user.AZIENDA],
            includi = ['username', 'codice', 'descrizione', 'genitore_id',
                'id', 'livello'])
        if self.risposta.contenuto == 'null':
            self.risposta.contenuto = objutils.toJSON([])

    @WaAuth()
    def getUserDetails(self, utente_id=WaAuth.INT):
        """ Ritorna i dettagli dell'utente """
        info = {}
        user = self.user.child_by_id(utente_id)
        if user:
            info = objutils.getObjInfo(user, includi = ['username', 'codice',
                'descrizione', 'genitore_id', 'id', 'livello', 'mesi_password',
                'is_segreteria', 'auto_presenze', 'is_ricezione',
                'auto_anagrafiche', 'is_sped_anag', 'is_sped_pres'])
            info.auto_anagrafiche = info.auto_anagrafiche or 0
            info.auto_presenze = info.auto_presenze or 0
        self.risposta.contenuto = objutils.toJSON(info)

    @WaAuth()
    def getAziendeListJ(self, id_utente=WaAuth.INT):
        """ Ritorna la lista delle aziende per l'utente specificato """
        info = []
        if self.user:
            if id_utente == self.user.id:
                user = self.user
            else:
                user = self.user.child_by_id(id_utente)
            info = objutils.getObjInfo([user],
                includi=['id', 'username', 'livello', 'descrizione',
                    'genitore_id'],
                custom={'codice': "-" * 7})
            if user.ditte:
                info += objutils.getObjInfo(user.ditte,
                    includi=['id', 'codice', 'username', 'livello',
                        'descrizione', 'genitore_id'])
        self.risposta.contenuto = objutils.toJSON(info)
        return info

    def getAziendeListUser(self):
        """ Ritorna la lista delle aziende per l'utente attuale """
        self.getAziendeListJ(self.user.id)

    def get_user(self):
        """ Ritorna le informazioni per l'utente """
        if self.user:
            user_info = self.user.info
            user_info.preferenze = self.user.get_all_prefs()
            user_info.impostazioni = self.user.get_all_impostazioni()
            user_info.impostazioni.cant_cc = None
            if self.user.livello == self.user.UAZIENDA:
                ditta = self.user.ditte[0]
                user_info.ditta = ditta.info
                user_info.ditta.preferenze = ditta.get_all_prefs()
                user_info.ditta.impostazioni = ditta.get_all_impostazioni()
                infoDitta = self._q(Ditta,
                        Ditta.codice == ditta.codice).first()
                try:
                    if infoDitta:
                        user_info.ditta.impostazioni.cant_cc = {'1':'cc', '2':'cc',
                                '5':'ct'}.get(infoDitta.gest_cc_ca, None)
                    else:
                        user_info.ditta.impostazioni.cant_cc = None
                except AttributeError:
                    user_info.ditta.impostazioni.cant_cc = None

            user_info.is_mutante = len(self.session['prev_user']) > 0
            if self.user.studio:
                user_info.studio = self.user.studio.info
            self.risposta.contenuto = objutils.toJSON(user_info)
        else:
            self.contenuto = "Non ho nessun utente attivo"

    @WaAuth()
    def get_ditta(self, ditta_id=WaAuth.INT):
        ditta = self.user.ditta_by_id(ditta_id)
        ditta_info = ditta.info
        ditta_info.preferenze = ditta.get_all_prefs()
        ditta_info.impostazioni = ditta.get_all_impostazioni()
        self.risposta.contenuto = objutils.toJSON(ditta_info)

    @WaAuth()
    def getUserPref(self, id_utente=WaAuth.INT, id_genitore=WaAuth.INT0):
        info = []
        if self.user:
            user = self.user.child_by_id(id_utente) or self.user
            user.genitore_id = id_genitore or self.user.id
            info = objutils.getObjInfo(user.preferenze, includi=['id', 'nome',
                'valore', 'utente_id', 'genitore_id'])
        if not info:
            info = []
        self.risposta.contenuto = objutils.toJSON(info)

    @WaAuth()
    def getUserOptions(self, user_id=WaAuth.INT0):
        """ Opzioni per l'utente """
        user = self.user.child_by_id(user_id) or self.user
        self.risposta.contenuto = objutils.toJSON(user.impostazioni)

    @sessions.require_lock
    @WaAuth()
    def aggiornaPreferenze(self, id_utente=WaAuth.INT, id_genitore=WaAuth.INT,
            dati=WaAuth.JEDO):
        """ Aggiorna le preferenze """
        user = self.user.child_by_id(id_utente) or self.user
        if not user:
            self._addErr("Utente non valido")
            return
        genitore = user.get_genitore(id_genitore) or user
        if dati.opzioni:
            dati.opzioni.ordinamento = dati.opzioni.ordinamento or None
            user.update(**dati.opzioni)
######  Preferenze
        newskin = newtema = None
        if dati.preferenze:
            for preferenza in dati.preferenze:
                if preferenza.id:
                    record_preferenza = user._q(NewUtentePref).get(
                            preferenza.id)
                    record_preferenza.nome = preferenza.nome
                    record_preferenza.valore = preferenza.valore
                else:
                    record_preferenza = NewUtentePref(preferenza.nome,
                            preferenza.valore)
                    record_preferenza.genitore_id = genitore.id
                    record_preferenza.utente_id = user.id
                    user.preferenze_rel.append(record_preferenza)
                if preferenza.nome == 'skin':
                    newskin = preferenza.valore
                if preferenza.nome == 'tema':
                    newtema = preferenza.valore
        if newskin:
            self.session['skin'] = newskin
        else:
            self.session['skin'] = self.request.SKIN_NAME
        if newtema:
            self.session['tema'] = newtema
        else:
            self.session['tema'] = self.request.DEFAULT_TEMPLATE
        try:
            user.commit()
            self._addStato('Preferenze salvate.')
        except Exception, e:
            self._addErr('Errore: %s' % e)

    @WaAuth()
    def eliminaPreferenza(self, dati=WaAuth.JEDO):
        """ elimina preferenze """
        id_opzione = dati['id_opzione']
        self.user._q(NewUtentePref, NewUtentePref.id == id_opzione).delete()
        self.user.commit()

    @WaAuth()
    def invia_segnalazione(self, email, testo, telefono='',
            debug_log={"default":None, "forcelist": True}):
        """ Invia una mail di segnalazione """
        log = ''
        for l in debug_log:
            log += "%s\n%s\n" % ("-"*80,l)
        From = To = "waonline@essepaghe.it"
        testo = """
Utente: %(username)s - %(udesc)s
Ditta: %(codice)s - %(ddesc)s
Email: %(email)s
Telefono: %(telefono)s
Data-ora: %(data)s
Segnalazione:
    %(testo)s

Log:
%(log)s""" % dict(
        username=self.user.username, udesc=self.user.descrizione,
        codice=self._ditta.codice, ddesc=self._ditta.descrizione, email=email,
        data=str(datetime.now()), testo=testo, telefono=telefono, log=log)
        msg = MIMEText(testo)
        msg['Subject'] = "Segnalazione per waonline inviata da %s" % \
                self.user.descrizione
        msg['From'] = From
        msg['To'] = To
        s = smtplib.SMTP("zcs.essepaghe.it")
        try:
            s.sendmail(From, [To], msg.as_string())
            self._addMess("Segnalazione inviata correttamente")
        except Exception, err:
            self._addErr("Errore inviando l'email %s" % err)
        s.quit()

    def invia_logo(self):
        self.info(self._files)
        self._repHead("Content-Type", "text/plain")
        #self.risposta.contenuto = '<html><body><script type="text/javascript">'
        try:
            ilfile = self._files['Logo'][0]
            fpath = os.path.join(conf.DEFAULT_DATA_PATH, self.user.studio_codice,
                    "LogoStudio.bmp")
            with open(fpath, 'wb') as f:
                f.write(ilfile['body'])
                #self._addMess("Logo salvato con successo")
                self.risposta.contenuto = 'mostraFinestraMessaggi([{msg:"Logo salvato con successo<br/><img src=\\\"/img/LogoStudio.png?%s\\\"/>", coords:null, tipo:"messaggio"}]);'\
                    'wndInviaLogo.destroy();'\
                    'try{'\
                    'var l = document.getElementById("logo_studio");'\
                    'l.src = "/img/LogoStudio.png?" + new Date().getTime();'\
                    '}catch(err){}' % time.time()

        except Exception, err:
            self.errore(err)
            #self._addErr(err)
            self.risposta.contenuto += 'mostraFinestraMessaggi([{msg:"%s", coords:null, tipo:"errore"}]);' % err
        #self.risposta.contenuto += '</script></body></html>'
