#! /usr/bin/python
# coding: latin-1
"""
Gestisce la connessione ai database in relazione a delle chiavi.
"""
__fine_name__ = "new_main.py"
__author__ = "luca"
__version__ = "1.0.0"
__date__ = "2011-11-07"


import time
from sqlalchemy                 import create_engine
from sqlalchemy.orm             import sessionmaker
from sqlalchemy.interfaces      import ConnectionProxy
from sqlalchemy.pool            import QueuePool
from tornado.options import options

from waonline.conf              import DEFAULT_DB_STR
import waonline.core.waologgin as wl


class TimerProxy(ConnectionProxy):
    """ Classe per il timeproxy """
    def cursor_execute(self, execute, cursor, statement, parameters,
            context, executemany):
        """ execute del cursore"""
        now = time.time()
        try:
            return execute(cursor, statement, parameters, context)
        finally:
            total = (time.time() - now) * 1000 # ms
            wl.info("Query: %s (%2.3f ms)" % (statement, total))


class SchemaProxy(ConnectionProxy):
    """ Classe per lo SchemaProxy """
    def execute(self, conn, execute, clauseelement, *multiparams, **params):
        """ execute del cursore"""
        schema = None
        now = time.time()
        try:
            salta = params.get('salta', True)
            if salta:
                database, schema = conn.execute("select current_database(), "\
                    "pg_catalog.current_setting('search_path')",
                    salta=False).fetchone()
            return execute(clauseelement, *multiparams, **params)
        finally:
            if schema:
                total = (time.time() - now) * 1000 # ms
                wl.info("=== DB: %s SCHEMA : %s (%2.3f ms)" % (
                    database, schema, total))
class DatabaseManager(object):
    """ Classe per il database manager """

    db_dict = {}
    """Dizionario !!!:
    ``key``: codice studio, ``value``: engine per quello studio
    """

    def __init__(self, db_url=DEFAULT_DB_STR, db_base="WA",
                 poolclass=QueuePool, timeproxy=0, pool_size=200):
        """
        :param db_url: database URL (rfc1738) - defaults to conf.DEFAULT_DB_STR
        :param db_base: default db name = defaults to ``WA``
        """

        self.db_url = db_url
        self.db_base = db_base
        self.timeproxy = timeproxy
        self.poolclass = poolclass
        self.pool_size = pool_size
        if timeproxy == 1:
            self.proxy = TimerProxy()
        elif timeproxy == 2:
            self.proxy = SchemaProxy()
        else:
            self.proxy = None
        self.Session = sessionmaker()
        #: creiamo un oggetto comodo per l'engine generico a WA
        #: (utenti e sessioni)
        self.sdb = self.get_engine(None)

    def get_dburl(self, cod_studio=None):
        if cod_studio:
            url = "%s/%s_%s" % (self.db_url, self.db_base, cod_studio)
        else:
            url = "%s/%s" % (self.db_url, self.db_base)
        # test case
        return url if options.nose == 0 else "%s_TEST" % url

    def get_db_name(self, cod_studio):
        name =  "%s_%s" % (self.db_base, cod_studio)
        return name if options.nose == 0 else "%s_TEST" % name

    def get_engine(self, cod_studio=None):
        "Ritorna un engine"
        try:
            return self.db_dict[cod_studio]
        except:
            self.db_dict[cod_studio] = create_engine(self.get_dburl(
                cod_studio), pool_size=self.pool_size, proxy=self.proxy)
            return self.db_dict[cod_studio]

    get_db = get_engine

    def db_exists(self, cod_studio):
        """ Verifica l'esistenza di un database """
        return self.db_dict.values()[0].execute(
                "SELECT EXISTS(SELECT datname "\
                "FROM pg_catalog.pg_database WHERE datname = '%s_%s');" % (
                self.db_base, cod_studio)).scalar()

    def db_create(self, cod_studio):
        """ Crea un database dal template """
        ss = self.get_sessiondb(None)
        ss.connection().connection.set_isolation_level(0)
        ss.execute('CREATE DATABASE "%s" WITH TEMPLATE "%s_BASE";' % (
                    self.get_db_name(cod_studio), self.db_base))
        ss.connection().connection.set_isolation_level(1)

    def get_sessiondb(self, cod_studio, sid=None):
        """ Ritorna una session db """
        return self.Session(bind=self.get_engine(cod_studio))

    def get_module(self, module_name):
        """ ricava il modulo """
        try:
            return __import__('waonline.gestione.%s' % module_name, {}, {},
                ['waonline.gestione'])
        except ImportError:
            return __import__('waonline.apps.%s' % module_name, {}, {},
                ['waonline.apps'])


    def get_method(self, mod_name, meth):
        """Ritorna il metodo ``meth`` in ``mod_name``. Suppone che
        esista una sola classe principale nel modulo chiamata ``Main``"""

        if not mod_name or not meth:
            return None
        mod_chiamato = self.get_module(mod_name)
        try:
            class_ = mod_chiamato.Main
            return getattr(class_, meth)
        except AttributeError:
            return None

    def get_function(self, cod_studio, modulo, session, request,
            login=False):
        """ chiama la funzione """
        sessione_db = self.get_sessiondb(cod_studio)
        mod_chiamato = self.get_module(modulo)
        content = mod_chiamato.dogentask(sessione_db, session, request)
        return content

    def call_app(self, cod_studio, app_name, method_name, session, request):
        """ Calls method in module """
        sessione_db = self.get_sessiondb(cod_studio)
        mod_chiamato = self.get_module(app_name)
        content = mod_chiamato.Main(sessione_db, session,
                request).doTask(method=method_name)
        return content
