#! /usr/bin/python
# coding: latin-1
"""
:File: qdict.py
:Author: luca
:Version: 1.0.0
:Date: 12/01/2011

Modulo python per dizionario campi etc.
"""
import re
import types
from datetime import datetime

from waonline.utils.utility import formatData

CAMPI_DICT = {
    "ACODDT01065" : "cod_fiscale",
    "ACODDP01019" : "nominativo",
    "ACODDP01040" : "comune_nascita",
    "ACODDP01035" : "data_nascita",
    "ACODDP01073" : "comune_resid",
    "ACODDP01055" : "indirizzo_resid",
    "ACODDP01050" : "cod_fiscale",
    "ACODDP01155" : "data_ass",
    "ACODDP17046" : "",
    "ADESDP17015" : "qualifica.codice",
    "ACODDP17045" : "qualifica.livello",
    "AD0FASS0001" : "",
    "ACODDP17050" : "qualifica.mansione",
    "ACODDT01090" : "",
    "ACODDT01085" : "",
    "GDATA012" : lambda x: datetime.today().strftime("%d/%m/%Y")

}

def con_data(self, data):
    """ Stupida funzione di conversione data """
    return formatData(data)

def get_comune(self, cod_comune):
    """ Recupera il comune """
    from waonline.tabelle.tabelle_l import Comune
    try:
        return self.base_instance._q(Comune.descrizione).filter(
                Comune.codice==cod_comune).scalar()
    except:
        return ''

#: Funzioni di base per alcuni campi noti
BASE_FUNCTS = {
    'data_nascita' : con_data,
    'data_ass' : con_data,
    'comune_resid': get_comune,
    'comune_nascita': get_comune,
}


class QDict(dict):
    """ Classe per dizionario con query set """
    def __init__(self, query_set=None, functs=BASE_FUNCTS, base_instance=None, *args, **kwargs):
        super(QDict, self).__init__(*args, **kwargs)
        self.campi_dict = CAMPI_DICT
        self.query_set = query_set
        self.base_instance = base_instance
        self.functs = functs
        self.rec = re.compile("(.+)(\w{2})(.)(\d{3})")

    def __getitem__(self, key):
        if key in self.campi_dict:
            value = self.campi_dict[key]
            if type(value) == types.FunctionType:
                value = value(self)
            elif '.' in value:
                try:
                    value = eval("self.query_set." + value)
                except:
                    value = ''
            elif value in self.functs:
                if hasattr(self.query_set, value):
                    value = self.functs[value](self, getattr(self.query_set,
                        value))
                else:
                    value = self.functs[value](self, None)
            else:
                if hasattr(self.query_set, value):
                    value = getattr(self.query_set, value)
        elif dict.__contains__(self, key) :
            value = dict.__getitem__(self, key)
        else:
            value = ''
        if type(value) == types.FunctionType:
            value = value(self)
        return unicode(value)

    def replace_file(self, file_path):
        """ Ritorna una stringa come risultato del replace sul file in
        file_path
        """
        return (open(file_path).read() % self).encode("latin-1")

    def replace(self, stringa):
        """ Ritorna una stringa come risultato del replace nella stringa
        stringa
        """
        return (stringa % self).encode("latin-1")
