# coding: latin-1

import os
import time
import base64
import cPickle as pickle
from datetime import datetime, timedelta

from sqlalchemy import sql, exc
from tornado.options import options

from waonline import conf
from waonline.core import waologgin as wl
from waonline.core.sessions.sessions import Session
from waonline.core.sessions.tabelle.web_session import session_table

CURRENT_SESSION = None
#sel = sql.select([session_table])
sel = sql.text("""SELECT data FROM web_session
             WHERE expire_date >= CURRENT_TIMESTAMP and key = :key""")
sel_lock = sql.text("""SELECT data FROM web_session
             WHERE expire_date >= CURRENT_TIMESTAMP and key = :key FOR UPDATE""")
upd = sql.text("""UPDATE web_session set data = :data, port = :port, cod_studio=:cod_studio,
               expire_date= :expire_date WHERE key = :key""")
ins = sql.text("""INSERT INTO web_session (key, data, expire_date, cod_studio, port)
                VALUES (:key, :data, :expire_date, :cod_studio, :port)""")
delete = sql.text("""DELETE FROM web_session WHERE key = :key""")


class SessionManager(object):
    """
    Session manager class, used to manage the various session objects stored in db
    """

    def __init__(self, session_expire_time=conf.SESSION_EXPIRE_TIME, db_manager=None):
        """
        :param db_manager: an instance of
        """
        self._session_expire_time = timedelta(seconds=session_expire_time)

        if not db_manager:
            # scorciatoia per debug interattivo
            from waonline.core.new_main import DatabaseManager
            db_manager = DatabaseManager()

        self.db_manager = db_manager

    def get_session(self, cookie=None, lock=False):
        """recupera la sessione dal db

        :param cookie: il cokie usato per identificare la session
        :param lock: se necessario usa ``FOR UPDATE``
        """
        global CURRENT_SESSION

        engine = self.db_manager.get_engine()

        if lock:
            engine.execute('BEGIN')

        res = engine.execute(sel_lock if lock else sel, key=cookie)

        row = res.fetchone()

        session = Session(self._session_expire_time, session_manager=self)
        try:
            from waonline.apps.presenze import calendario
            data = base64.decodestring(row['data'])
            session.data = pickle.loads(data)
            session.renew()
            session.sid = cookie
            session.to_save = False
            if lock:
                session.connection = res.connection
            wl.debug("Sess da DB: %s calendario: %s" % (session.sid, session.calendario))
        except TypeError, e:  # nessuna risposta row['data']: NoneType is unsubscriptable
            session.sid = cookie or session.generate_sid()
            wl.debug("Sess NUOVA: %s %s" % (session.sid, session.calendario))

        CURRENT_SESSION = session
        return session

    def save_session(self, session):
        """salva la sessione corrente"""
        session_conn = session.connection
        session.connection = None # A connection is not serializable
        wl.debug("SAVE SESSION: to_save: %s %s calendario: %s" % (session.to_save, session.sid, session.calendario))

        if session.to_save:
            pickled = pickle.dumps(session.data)
            data = base64.encodestring(pickled)
            conn = session_conn or self.db_manager.get_engine()
            args = {'key' : session.sid,
                    'data' : data,
                    'expire_date' : datetime.now() + self._session_expire_time,
                    'cod_studio' : '' if not 'cod_studio' in session else session['cod_studio'],
                    'port' : options.port
                    }
            # if session_conn:
            #     session_conn.execute("SAVEPOINT pre_insert")
            if not session_conn:  # try first INSERT
                try:
                    res = conn.execute(ins, **args)
                except exc.IntegrityError:
                    res = conn.execute(upd, **args)
                # if session_conn:
                #     session_conn.execute("RELEASE SAVEPOINT pre_insert")
            else:
                # if session_conn:
                #     session_conn.execute("ROLLBACK TO SAVEPOINT pre_insert")

                res = conn.execute(upd, **args)
        else:
            if session_conn:
                session_conn.execute('COMMIT')

        return

    def delete_session(self, session):
        """elimina la sessione"""
        global CURRENT_SESSION
        conn = session.connection or self.db_manager.get_engine()
        conn.execute(delete, key=session.sid)
        CURRENT_SESSION = None
        return



