# coding: latin-1
"""
File: __init__.py
Author: Marco Dalla Stella
Date: 28/05/2010
"""
__author__ = "Marco Dalla Stella"

import redis
import cPickle as pickle
from datetime import datetime, timedelta
import os
import logging
import inspect
from multiprocessing import current_process
import time

from waonline.gestione import calendario
from waonline.core.sessions.sessions import Session

settings = {
    "SESSION_EXPIRE_TIME": 7200, # sessions are valid for 7200 seconds (2 hours)
    "REDIS_URL": {'ip': 'redis_server', 'port': 6379, 'db': 0, }
}

DEBUG_STR = "\x1b[35m[%s %s:%s]\x1b[0m:\n\x1b[1;35m%s\x1b[0m"
DEBUG_INLINE = "\x1b[35m[%s %s:%s]\x1b[0m:\x1b[1;35m%s\x1b[0m"
def debug_inspect(messaggio, deept=2):
    """ scrive sul log """
    p = current_process()
    frame = inspect.getframeinfo(inspect.stack()[deept][0])
    if frame.function == 'calendario':
        frame = inspect.getframeinfo(inspect.stack()[deept - 1][0])
    fname = os.path.basename(frame.filename)
    logging.info(DEBUG_INLINE % (
        fname, frame.function, frame.lineno,
        "SessionMenager: [%s] %s" % (p.pid, messaggio)))

#lock = threading.Lock()

class SessionManager(object):
    """
    Session manager class, used to manage the various session objects and talk with Redis.
    """

    class _instance_():
        """ classe per l'istanza """
        def __init__(self, session_expire_time=settings["SESSION_EXPIRE_TIME"],
                redis_url=settings["REDIS_URL"], on_delete=None, db_manager=None):
            #db_manager
            self._rd = redis.Redis(host=redis_url['ip'],
                    port=redis_url['port'], db=redis_url['db'])
            self._on_delete = on_delete
            self._session_expire_time = session_expire_time
            self._sessioni = {}

        def get_session(self, cookie=None):
            """ recupera la sessione """
            from waonline.core.sessions.session import Session
            if cookie in self._sessioni:
                return self._sessioni[cookie]
            try:
                data = self._rd.get(cookie)
                session = pickle.loads(data)
                if session.is_expired:
                    self.delete_session(session)
                else:
                    session.renew()
            except Exception, err:
                print str(err)
                session = Session(self._session_expire_time)
                self.save_session(session)
            self._sessioni[session.sid] = session
            return self._sessioni[session.sid]

        def has_session(self, sid):
            """verifica se esiste la sessione"""
            try:
                session = self._rd.get(sid)
                if not session:
                    return False
                else:
                    return True
            except:
                return False

        def get_all_sessions(self):
            """ritorna tutte le sessioni"""
            for sid in self._rd.keys('*'):
                session = self._rd.get(sid)
                if session != None:
                    session = pickle.loads(session)
                    yield session

        def save_session(self, session):
            """salva la sessione corrente"""
            try:
                data = pickle.dumps(session)
                res =  self._rd.set(session.sid, data)
            except:
                res = None
            return res

        def delete_session(self, session):
            """elimina la sessione"""
            if self._on_delete:
                self._on_delete(session.sid)
            try:
                del self._sessioni[session.sid]
            except:
                pass
            return  self._rd.delete(session.sid)

    instance = {}
    def __init__(self, *args):
        if self.__class__ not in SessionManager.instance:
            SessionManager.instance[self.__class__] = \
                    SessionManager._instance_(*args)
        else:
            return None

    def __getattr__(self, name):
        return getattr(self.instance[self.__class__], name)

    def __setattr__(self, name, value):
        return setattr(self.instance[self.__class__], name, value)

    def __dir__(self):
        return dir(SessionManager.instance[self.__class__])


