# coding: latin-1
"""
"""
# Codice precedentemente inserito in __init__


import os
import time
from datetime import datetime, timedelta

from waonline.core import waologgin as wl


class Session(object):
    """
    Session object, works like a dict.
    """

    connection = None
    """If not ``None`` punta alla connessione al db usata nel SELECT FOR UPDATE
    Connessione che eve essere usata per chiudere la transtazione.
    """

    def __init__(self, session_expire, session_manager=None):
        self.session_manager = session_manager
        #self.sid = self._generate_sid()
        self._expires_in = session_expire
        self._last_update = datetime.now()
        self.data = dict()
        self.connection = None
        self.to_save = False
    
    def generate_sid(self):
        """genera il ... sid/cookie"""
        return os.urandom(32).encode('hex') # 256 bits of entropy

    @property
    def is_expired(self):
        """controlla se la sessine  spirata"""
        td = datetime.now() - self._last_update
        return td > self._expires_in

    def renew(self):
        """rinnova la sessione"""
        self._last_update = datetime.now()

    def has_key(self, keyname):
        """verifica se la sessione contiene la chiave"""
        return self.__contains__(keyname)

    def save(self):
        """Salva la sessione"""
        return self.session_manager.save_session(self)

    def delete(self):
        """Elimina la sessione"""

        return self.session_manager.delete_session(self)

    def __delitem__(self, key):
        del self.data[key]
        self.to_save = True
        return True

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, val):
        if key in self.data and self.data[key] == val:
            return
        self.data[key] = val
        self.to_save = True
        wl.debug("set key %s in session %s" % (key, self.sid))
        return True

    def __len__(self):
        return len(self.data)

    def __contains__(self, key):
        return key in self.data

    def __iter__(self):
        for key in self.data:
            yield key

    def __str__(self):
        return u"sid: %s, {%s}" % (self.sid, ', '.join(['"%s" = "%s"' % (k,
            self.data[k]) for k in self.data]))

    def __repr__(self):
        return '<Sessione: %s %s>' % (self.sid, self.data.keys())

    @property
    def calendario(self):
        try:
            return self.data['calendario']
        except KeyError:
            return None
