#!/usr/bin/env python
# coding: latin-1

#-----------------------------------------------------------------------------
# Name:        tracciati.py
# Purpose:
#
# Author:      Luca Zaccaria
#
# Created:     2009/12/28
# RCS-ID:      $Id: tracciati.py $
# Copyright:   (c) 2008
# Licence:     <your licence>
#-----------------------------------------------------------------------------

import re
import os
from cStringIO import StringIO
import logging

# costatnti per l'estrazione delle informazioni dei campi del tracciato
RE_CAMPO = r'^\s*(?P<livello>\d{2})\s+(?P<nome>[a-z|A-Z|0-9|\-]+)\s*(?P<val>.*)\.'
RE_TIPO = [
    r'pic (?P<tipo>\w+)(?P<virgola>[,|v])(?P<tipo2>\w+)',
    r'pic (?P<tipo>\w)\((?P<lung1>\d+)\)(?P<virgola>[,|v])(?P<tipo2>\w)\((?P<lung2>\d)\)',
    r'pic (?P<tipo>\w)\((?P<lung1>\d+)\)',
    r'pic (?P<tipo>\w+)'
    ]

def parseValue(value):
    val = StringIO(value)

class Campo(object):
    """Classe per la gestione dei singoli campi del tracciato"""

    def __init__(self,nome,livello,typestring='',**kwargs):
        """
        nome: nome del campo
        livello: livello del campo
        typestring: stringa per la definiziode del tipo campo, compresa di PIC
        """

        self.campi= {}
        self._campi = []
        self.nome = nome.lower()
        self.livello = int(livello)
        self.lunghezza = 0
        self._tipo = 'x'
        self.lunghezza2 = 0
        self._tipo2 = ''
        self.virgola = ''
        self._value = ''
        self.parent = None
        self._typestring = typestring
        self.pos = 0
        for k,v in kwargs.items():
            setattr(self,k,v)

        self._inittype()

    def len(self):
        return len(self.force_format())

    def force_format(self):
        """Forza la formattazione del valore in base al tipo di campo e la
        stringa di formattazione (come verranno scritti sul file insomma)
        """

        # se il campo ha dei figli utilizzo le loro formattazioni per creare
        # quella del campo stesso
        if self._campi:
            fvalue = ''
            for v in self._campi:
                fvalue += v.force_format()
        # altrimenti formatto partendo dal valore del campo
        else:
            fvalue = self.value
            if fvalue == None:
                fvalue = ''
            try:
                if self.virgola == 'v':
                    fvalue = self.format % fvalue * 100
                elif self.virgola == ',':
                    fvalue = (self.format % fvalue).replace('.',',')
                else:
                    if self.tipo == '9':
                        fvalue = self.format % fvalue
                    else:
                        fvalue = self.format % str(fvalue.encode("utf-8"))
            except Exception,why:
               pass
        return fvalue

    @property
    def format(self):
        """Genera la stringa di formattazione del campo in base al tipo di
        campo e alle psue proprieta'
        """
        st  = ''
        nl = self.lunghezza + self.lunghezza2
        if self.virgola == ',':
            st = "%0" + str(nl +1) + ".0" + str(self.lunghezza2) + "f"
        else:
            if self.tipo == "9":
                st = "%0" + str(nl) + "d"
            else:
                st = "%-" + str(nl) + "s"
        return st

    @property
    def value(self):
        """Ritorna il valore in base al tipo di campo e alle sue proprieta'"""

        # la virgola e' virtuale quindi divido il valore nella variabile per
        # cento prima di trasformarlo in float.
        if self.virgola.lower() == "v":
            return round(float(self._value)/100,self.lunghezza2)
        elif self.virgola == ',':
            return round(float(self._value.replace(',','.') if self._value else 0),self.lunghezza2)
        else:
            if self.tipo == '9':
                return int(self._value if self._value else 0)
            else:
                return self._value
    @value.setter
    def value(self,value):
        """Imposta il valore creando una stringa in base al tipo campo e alle
        proprieta'"""
        try:
            if self.virgola == 'v':
                self._value = self.format % value * 100
            elif self.virgola == ',':
                self._value = (self.format % value).replace('.',',')
            else:
                if self.tipo == '9':
                    self._value = self.format % value
                else:
                    self._value = self.format % str(value.encode("utf-8"))
        except Exception, why:
            print why
            self._value = value

    @value.deleter
    def value(self):
        del self._value

    @property
    def tipo(self):
        return self._tipo

    @tipo.setter
    def tipo(self,tipo):
        self._tipo = tipo[0]
        if not self.lunghezza:
            self.lunghezza = len(tipo)

    @tipo.deleter
    def tipo(self):
        del self._tipo

    @property
    def tipo2(self):
        return self._tipo2

    @tipo2.setter
    def tipo2(self,tipo):
        if tipo:
            self._tipo2 = tipo[0]
            if not self.lunghezza2:
                self.lunghezza2 = len(tipo)

    @tipo2.deleter
    def tipo2(self):
        del self._tipo2

    def _inittype(self):
        """Imposta tipo e proprieta' del campo in base alla stringa _typestring"""
        self._typestring = self._typestring.lower()
        for xs in RE_TIPO:
            m = re.match(xs,self._typestring,re.IGNORECASE)
            if m:
                self.tipo = m.groupdict().get('tipo','x').lower()
                self.tipo2 = m.groupdict().get('tipo2','').lower()
                self.lunghezza = int(m.groupdict().get('lung1',self.lunghezza))
                self.lunghezza2 = int(m.groupdict().get('lung2',self.lunghezza2))
                self.virgola = m.groupdict().get('virgola','')
                break

    def addcampo(self,campo):
        """Aggiunge un campo alle liste dei sottocampi e rinimina eventuali
        campi FILLER in caso ce ne fosse piu' d'uno"""
        campo.parent = self

        if self.campi.has_key(campo.nome):
            if campo.nome[:6].lower() == 'filler':
                campo.nome = 'filler-%02d' % len([i for i in self.campi.keys() if i[:6].lower() == 'filler'])
            else:
                raise Exception("Nome del campo duplicato")
        # aggiungo al dizionario dei campi
        self.campi[campo.nome]=campo
        # aggiungo alla lista dei campi (questo sopratutto per mantenere
        # l'ordine di inserimento dei campi)
        self._campi.append(campo)
        # qui aggiungo come oggetto sostituendo eventuali - con _ nel nome
        setattr(self,campo.nome.replace('-','_'),campo)

class Record(Campo):

    def __init__(self,nome,tracciato,**kwargs):
        Campo.__init__(self,nome,1,'',**kwargs)
        self.tracciato = tracciato

    def write(self):
        self.tracciato._file.write(self.force_format() + "\r\n")


class Tracciato:
    def __init__(self,nome_tracciato=None,nome_file=None):
        self._cleart()
        if nome_tracciato:
            self.apri_tracciato(nome_tracciato)

        if nome_file:
            self.apri_file(nome_file)
        else:
            self._file = StringIO()


    def _cleart(self):
        self.campi = {}
        self._campi = []
        self.tracciati = {}
        self._tracciati = []
        for c in dir(self):
            if type(getattr(self,c)) == Campo:
                delattr(self,c)


    def apri_file(self,nomefile):
        self.nome_file = nomefile
        #if os.path.exists(self.nome_file):
        #    self._file = StringIO(open(self.nome_file).read())
        #else:
        self._file = StringIO()

    def close(self):
        open(self.nome_file,'wb').write(self._file.getvalue())
        self._file.close()

    def apri_tracciato(self,nomefile):

        self.nome_tracciato = nomefile
        if not isinstance(nomefile, str):
            f = nomefile
            f.seek(0)
        else:
            f = open(self.nome_tracciato)
        mre = re.compile(RE_CAMPO)
        self._cleart()
        lastCampo = None
        while 1:
            riga = f.readline()
            if not riga:
                break
            riga = re.sub(r'\*.*','',riga,re.MULTILINE)
            riga = re.sub(r'\|.*','',riga,re.MULTILINE)
            m = mre.match(riga)
            if m:
                if int(m.group('livello')) == 1:
                    campo = Record(m.group('nome'),self)
                else:
                    campo = Campo(m.group('nome'),m.group('livello'),m.group('val'))
                if campo:
                    if campo.livello > 1:
                        if lastCampo :
                            if campo.livello > lastCampo.livello:
                                lastCampo.addcampo(campo)
                            elif campo.livello == lastCampo.livello:
                                lastCampo.parent.addcampo(campo)
                            else:
                                ncampo = lastCampo.parent
                                while (ncampo.livello >= campo.livello ) and (ncampo.parent != None):
                                    ncampo = ncampo.parent
                                ncampo.addcampo(campo)
                    else:
                        if campo.livello == 1:
                            self.tracciati[campo.nome]=campo
                            self._tracciati.append(campo)
                            record = campo
                    campo.record = record
                    self.addcampo(campo)
                    lastCampo = campo
        f.close()
        self._findpos()

    def _findpos(self):
        for t in self._tracciati:
            lastpos = 0
            for campo in t._campi:
                campo.pos = lastpos
                lastpos = campo.len()


    def addcampo(self,campo):
        if not campo.parent:
            campo.parent = self
        if self.campi.has_key(campo.nome):
            if campo.nome[:6].lower() == 'filler':
                campo.nome = 'filler-%02d' % len([i for i in self.campi.keys() if i[:6].lower() == 'filler'])
            else:
                raise Exception("Nome del campo duplicato")
        self.campi[campo.nome]=campo
        self._campi.append(campo)
        setattr(self,campo.nome.replace('-','_'),campo)




