#! /usr/bin/python
# -*- coding: Latin-1 -*-
"""
"""
__fine_name__ = "red_utenti.py"
__author__ =    "luca"
__version__ =   "1.0.0"
__date__ =      "2011-10-28"


import os
import redis
import re
import hashlib
import cPickle                              as pickle

from waonline.core.ete_dumbobj              import EteDumbObj
from waonline.core.users import NewUtente
from waonline.conf                          import DEFAULT_WORKING

redis_users = EteDumbObj(HOST='localhost', PORT=6379, DB=3)

class UtentiManager(object):
    class _instance_(object):
        ADMIN = 100; SERVICE = 80; USTUDIO = 60; UAZIENDA = 40; AZIENDA = 0
        def __init__(self, **kwargs):
            self._rd = redis.Redis(host=redis_users.HOST,
                    port=redis_users.PORT, db=redis_users.DB)
            self.reload()

        def get_user(self, username):
            try:
                u = pickle.loads(self._rd.get(username.upper()))
                return u
            except Exception, err:
                return None

        def set_user(self, user, reload=True):
            try:
                data = pickle.dumps(user)
                self._rd.set(user.username, data)
                if reload:
                    self.reload()
                    self.sreload()
            except:
                pass

        def del_user(self, username, reload=True):
            self._rd.delete(username)
            if reload:
                self.reload()
                self.sreload()

        def sreload(self):
            os.system("%s/bin/reload_users.sh&" % DEFAULT_WORKING)

        def _del_all_users(self, reload=True):
            self._rd.flushdb()
            if reload:
                self.reload()
                self.sreload()

        def reload(self):
            users = []
            def ordina(oggetto):
                try:
                    return oggetto.livello
                except:
                    return 0
            for k in self._rd.keys("*"):
                users.append(self.get_user(k))
            self.users = sorted(users, key=ordina, reverse=True)
            self.rusers = self.users[::-1]

        def login(self, username, passwd):
            user = self.get_user(username)
            if user and user.check_password(passwd):
                return user
            else:
                return None

    instance = None
    def __init__(self, *args):
        if UtentiManager.instance == None:
            UtentiManager.instance = UtentiManager._instance_(*args)
        else:
            return None

    def __getattr__(self, name):
        return getattr(self.instance, name)

    def __setattr__(self, name, value):
        return setattr(self.instance, name, value)

    def __getitem__(self, idx):
        return self.instance._menu[idx]

    def __setitem__(self, idx, value):
        self.instance._menu[idx] = value

    def __dir__(self):
        return dir(self.instance)

class RedUser(object):

    ADMIN = 100; SERVICE = 80; USTUDIO = 60; UAZIENDA = 40; AZIENDA = 0
    _username = ""
    _password = ""
    livello = AZIENDA
    codice = ""
    descrizione = ""
    studio_username = None
    studio_codice = ''
    _genitore_username = []
    db_ref = ''
    db_user_id = 0
    #: Da NON salvare
    _q = None
    _dbu = None
    _genitore_att = None
    _figli_rel = None
    _children = None
    _id_children = None
    _genitori_rel = None
    _antenati = None
    _id_antenati = None
    _ditte = None
    _id_ditte = None
    _codici_ditte = None

    def __init__(self, **kwargs):
        #: Da salvare nel pickle
        self._username = ""
        self._password = ""
        self.livello = self.AZIENDA
        self.codice = ""
        self.descrizione = ""
        self.studio_username = None
        self.studio_codice = ''
        self._genitore_username = []
        self.db_ref = ''
        self.db_user_id = 0
        #: Da NON salvare nel pickle
        self._q = None
        self._dbu = None
        self._genitore_att = None
        self._figli_rel = None
        self._children = None
        self._id_children = None
        self._genitori_rel = None
        self._antenati = None
        self._id_antenati = None
        self._ditte = None
        self._id_ditte = None
        self._codici_ditte = None
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __repr__(self):
        return u"<%s: %s>" % (self.__class__.__name__, self.username)

    @property
    def dbu(self):
        if not self._dbu:
            self._dbu = self._q(Utente).filter(
                    Utente.username == self.username).first()
        return self._dbu

    def get_genitore_db(self, id_genitore):
        return self._q(Utente).get(id_genitore)

    def check_password(self, passwd):
        cript = self._password == hashlib.sha1(passwd.upper()).hexdigest()
        if cript:
            self._password = passwd.upper()
            return True
        else:
            return self._password == passwd.upper()

    def clean(self):
        self._figli_rel = self._children = self._id_children = \
               self._genitori_rel = self._antenati = self._id_antenati = \
               self._ditte = self._id_ditte = self._codici_ditte = \
               self._genitore_att = None

    @property
    def genitore_att(self):
        return self._genitore_att or self.genitore_username[0]

    @genitore_att.setter
    def genitore_att(self, username):
        self._antenati = None
        self._id_antenati = None
        self._genitore_att = username

    @property
    def genitore(self):
        try:
            return UtentiManager().get_user(self.genitore_att)
        except:
            return None

    @property
    def ditte(self):
        if self._ditte == None:
            self._ditte = []
            for d in self.children:
                if d.livello == 0:
                    self._ditte.append(d)
        return self._ditte

    @property
    def id_ditte(self):
        if self._id_ditte == None:
            self._id_ditte = [uu.db_user_id for uu in self.ditte]
        return self._id_ditte

    @property
    def codici_ditte(self):
        if self._codici_ditte == None:
            self._codici_ditte = [uu.codice for uu in self.ditte]
        return self._codici_ditte

    @property
    def figli_rel(self):
        if self._figli_rel == None:
            self._figli_rel = []
            for u in UtentiManager().users:
                if self.username in u.genitore_username:
                    u.genitore_att = self.username
                    self._figli_rel.append(u)
        else:
            for f in self._figli_rel:
                f.genitore_att = self.username
        return self._figli_rel

    @property
    def genitori_rel(self):
        if self._genitori_rel == None:
            self._genitori_rel = []
            for gn in self._genitore_username:
                g = UtentiManager().get_user(gn)
                if g:
                    self._genitori_rel.append(g)
        return self._genitori_rel

    @property
    def children(self):
        if self._children == None:
            self._children = [] + self.figli_rel
            for u in UtentiManager().users:
                if [g for g in u.genitore_username if g in \
                        [c.username for c in self._children]]:
                    u.genitore_att = g
                    self._children.append(u)
        else:
            for f in self._figli_rel:
                f.genitore_att = self.username
        return self._children

    @property
    def id_children(self):
        if self._id_children == None:
            self._id_children = [uu.db_user_id for uu in self.children]
        return self._id_children

    @property
    def antenati(self):
        if self._antenati == None:
            self._antenati = []
            a = self
            while True:
                a = a.genitore
                if not a:
                    break
                self._antenati.append(a)
        return self._antenati

    @property
    def id_antenati(self):
        if self._id_antenati == None:
            self._id_antenati = [uu.db_user_id for uu in self.antenati]
        return self._id_antenati


    @property
    def genitore_username(self):
        return self._genitore_username

    @genitore_username.setter
    def genitore_username(self, username):
        if not username in self._genitore_username:
            self._genitore_username.append(username)

    def del_genitore(self, username):
        try:
            self._genitore_username.remove(username)
        except:
            pass

    @property
    def username(self):
        return self._username.upper()

    @username.setter
    def username(self, name):
        self._username = name.upper()

    @property
    def password(self):
        return self._password

    @password.setter
    def password(self, passwd):
        self._password = passwd.upper()

    def __getstate__(self):
        d = self.__dict__.copy()
        for k in self.__dict__.keys():
            if k in ["_username", "_password", "livello", "codice",
                    "descrizione", "studio_username", "studio_codice",
                    "_genitore_username", "db_ref", "db_user_id"]:
                d[k] = self.__dict__[k]
        return d

    def get_ditta(self, codice_username):
        try:
            return [d for d in self.ditte if d.codice == codice_username or\
                    d.username == codice_username][0]
        except:
            return None

    def has_ditta(self, codici):
        """ Ritorna True se il codice della ditta appartiene ad una delle ditte
        dell'utente, altrimenti ritorna il primo codice non valido.
        *codici* pu essere sia una stringa che una lista di stringhe.
        """
        if not isinstance(codici, list):
            codici = [codici]
        for codice in codici:
            if not codice.upper() in [d.codice.upper() for d in self.ditte]:
                return codice.upper()
        return True

    def get_all_prefs(self, cls, sdb, toLista=False):
        """ Ritorna tutte le preferenze partendo da quelle del :class:`Utente`
        di livello pi alto gerarchicamente. In caso di preferenze con lo stesso
        nome verranno sostituite con quelle dell' :class:`Utente` del livello
        pi basso.

        In caso di **filtri** (*FILTRO_NOME_FILTRO* ,
        *FILTRO_NO_NOME_FILTRO*) i valori verranno messi in una lista.
        Un eventuale *FILTRO_NO_NOME_FILTRO* al livello pi basso andr
        ad invalidare il relativo *FILTRO_NOME_FILTRO* con il medesimo valore
        del livello pi alto e viceversa.

        Ritorna un oggetto :class:`wadecohelpers.EteDumbObj` con le preferenze
        oppure una lista di tuplet (*id*, *nome*, *valore*) se il parametro
        *toLista*  True.
        """
        pat = re.compile(r'^FILTRO_(?P<tipo>NO_){0,1}(?P<nome>\w+)')
        prefs = EteDumbObj()
        prefs_id = EteDumbObj()
        prefs_lv = EteDumbObj()
        count_ante = 0
        for utente in sdb.query(cls).filter(cls.id.in_(
            self.id_antenati[::-1] + [self.db_user_id])).order_by(
                    cls.livello.desc()):
            prefs_idx = {}
            lpref = None
            for pref in utente.preferenze:
                m = pat.match(pref.nome.upper())
                if m:
                    d = m.groupdict()
                    if not d['nome'] in prefs:
                        prefs[d['nome']] = {"si": [[]], "no": [[]]}
                        prefs_id[d['nome']] = pref.id
                    ntipo = "no" if d['tipo'] == "NO_" else "si"
                    try:
                        lpref = prefs[d["nome"]][ntipo][count_ante]
                    except:
                        prefs[d["nome"]][ntipo].append([])
                        lpref = prefs[d["nome"]][ntipo][count_ante]
                    if not pref.valore in lpref:
                        lpref.append(pref.valore)
                else:
                    nnome = pref.nome
                    if nnome.startswith('+'):
                        isLista = True
                        nnome = nnome[1:]
                    elif nnome.startswith('-'):
                        nnome = nnome[1:]
                        if nnome in prefs:
                            if isinstance(prefs[nnome], list):
                                if pref.valore in prefs[nnome]:
                                    prefs[nnome].remove(pref.valore)
                            else:
                                del prefs[nnome]
                        continue
                    else:
                        isLista = False
                    if not nnome in prefs:
                        if isLista:
                            prefs[nnome] = [pref.valore]
                        else:
                            prefs[nnome] = pref.valore
                        prefs_id[nnome] = pref.id
                        prefs_lv[nnome] = utente.livello
                        prefs_idx[nnome] = 0
                    else:
                        if not nnome in prefs_idx:
                            prefs_idx[nnome] = 0
                        if isLista:
                            if not isinstance(prefs[nnome], list):
                                prefs[nnome] = [prefs[nnome]]
                            if not pref.valore in prefs[nnome]:
                                prefs[nnome].append(pref.valore)
                        else:
                            if isinstance(prefs[nnome], list):
                                prefs[nnome][prefs_idx[nnome]] = pref.valore
                            else:
                                prefs[nnome] = pref.valore
                        prefs_idx[nnome] += 1
            if lpref:
                count_ante += 1
        if toLista:
            return zip(prefs_id.values(), prefs.keys(), prefs.values())
        else:
            return prefs

    def get_all_impostazioni(self, cls, sdb):
        """ Ritorna un dizionario con le impostazioni per il client partendo da
        quelle dell' :class:`Utente` di livello pi alto gerarchicamente.
        Sostituendole di volta in volta con quelle di livello pi basso se
        presenti.
        """
        impostazioni = EteDumbObj({
            'seleziona_tutti': True,
            'calendario_verticale': True,
            'ordinamento': 8,
            'tipo_ordinamento': True,
            'is_sessantesimi': False,
            'auto_anagrafiche': False,
            'auto_presenze': False
        })
        for utente in sdb.query(cls).filter(cls.id.in_(
            self.id_antenati[::-1] + [self.db_user_id])):
            for chiave in impostazioni.keys():
                if hasattr(utente, chiave):
                    valore = getattr(utente, chiave)
                    if not valore == None:
                        impostazioni[chiave] = valore
        return impostazioni

    def save(self, reload=True):
        UtentiManager().set_user(self, reload)

if __name__ == '__main__':
    um = UtentiManager()
