#! /usr/bin/python
# coding: latin-1
"""
:File: waologgin.py
:Author: luca
:Version: 1.0.0
:Date: 06/07/2011

"""

import inspect
import logging
import time
import os
import codecs
from functools import wraps

from tornado.options import options

from waonline import conf


ERROR_STR_C = "\x1b[31m[%s %s:%s]\x1b[0m:\n\t\x1b[1;31m%s\x1b[0m"
INFO_STR_C = "\x1b[32m[%s %s:%s]\x1b[0m:\n\t\x1b[1;32m%s\x1b[0m"
DEBUG_STR_C = "\x1b[34m[%s %s:%s]\x1b[0m:\n\t\x1b[1;34m%s\x1b[0m"
WARN_STR_C = "\x1b[33m[%s %s:%s]\x1b[0m:\n\t\x1b[1;33m%s\x1b[0m"
ERROR_INLINE_C = "\x1b[31m[%s %s:%s]\x1b[0m:\x1b[1;31m%s\x1b[0m"
INFO_INLINE_C = "\x1b[32m[%s %s:%s]\x1b[0m:\x1b[1;32m%s\x1b[0m"
DEBUG_INLINE_C = "\x1b[34m[%s %s:%s]\x1b[0m:\x1b[1;34m%s\x1b[0m"
WARN_INLINE_C = "\x1b[33m[%s %s:%s]\x1b[0m:\x1b[1;33m%s\x1b[0m"
STRS_C = [0, 1]
STRS_C[0]=range(4)
STRS_C[1]=range(4)
INFO, DEBUG, WARN, ERROR = range(4)
STRS_C[0][INFO] = INFO_STR_C
STRS_C[0][DEBUG] = DEBUG_STR_C
STRS_C[0][WARN] = WARN_STR_C
STRS_C[0][ERROR] = ERROR_STR_C
STRS_C[1][INFO] = INFO_INLINE_C
STRS_C[1][DEBUG] = DEBUG_INLINE_C
STRS_C[1][WARN] = WARN_INLINE_C
STRS_C[1][ERROR] = ERROR_INLINE_C

ERROR_STR = "[%s %s:%s]\n\t%s"
INFO_STR = "[%s %s:%s]\n\t%s"
DEBUG_STR = "[%s %s:%s]\n\t%s"
WARN_STR = "[%s %s:%s]\n\t%s"
ERROR_INLINE = "[%s %s:%s]%s"
INFO_INLINE = "[%s %s:%s]%s"
DEBUG_INLINE = "[%s %s:%s]%s"
WARN_INLINE = "[%s %s:%s]%s"
STRS = [0, 1]
STRS[0]=range(4)
STRS[1]=range(4)
STRS[0][INFO] = INFO_STR
STRS[0][DEBUG] = DEBUG_STR
STRS[0][WARN] = WARN_STR
STRS[0][ERROR] = ERROR_STR
STRS[1][INFO] = INFO_INLINE
STRS[1][DEBUG] = DEBUG_INLINE
STRS[1][WARN] = WARN_INLINE
STRS[1][ERROR] = ERROR_INLINE

USA_COLORI = True

class MultiLineFormatter(logging.Formatter):
    def format(self, record):
        """A formatter that log also the session information
        """
        from waonline.core.sessions import db_manager

        msg = logging.Formatter.format(self, record)
        header, footer = msg.split(record.message)
        msg = msg.replace('\n', '\n' + '  '*len(header))
        mark = db_manager.CURRENT_SESSION and db_manager.CURRENT_SESSION.sid[:5] or '-SID-'
        try:
            return "%s %s" % (mark, msg)
        except:
            return msg

my_logger = None
my_file = None

def init_log(filename=None,color=True):
    global my_logger, my_file
    my_logger = logging.getLogger()
    for lo in my_logger.handlers:
        my_logger.handlers.remove(lo)
    syshandler = logging.handlers.SysLogHandler(address=(conf.LOG_ADDRESS))
    syshandler.setFormatter(
        MultiLineFormatter("%s %%(message)s" % options.port))
    my_logger.addHandler(syshandler)
    if filename:
        my_file = codecs.open(filename, 'a+', 'utf-8')
        fh = logging.StreamHandler(my_file)
        my_logger.addHandler(fh)


def _log(messaggio, tipo=INFO, inline=1, dept=1):
    """ Scrive nel log """
    frame = inspect.getframeinfo(inspect.stack()[dept+1][0])
    while frame.function in ['timelog', 'wrapper', 'timeinfo', 'timedebug',
            'timeerror', 'timewarn']:
        dept += 1
        frame = inspect.getframeinfo(inspect.stack()[dept+1][0])
    fname = os.path.basename(frame.filename)
    if USA_COLORI:
        log_str = STRS_C[inline][tipo]
    else:
        log_str = STRS[inline][tipo]
    if tipo == ERROR:
        chelog = logging.exception
    elif tipo == INFO:
        chelog = logging.info
    elif tipo == DEBUG:
        chelog = logging.debug
    elif tipo == WARN:
        chelog = logging.warn
    else:
        chelog = logging.warn
    if isinstance(messaggio, float):
        messaggio = "%2.3f" % messaggio
    chelog(log_str % (fname, frame.function, frame.lineno, messaggio))
    del frame


def errore(messaggio, inline=True, dept=1):
    """ log per l'errore """
    _log(messaggio, ERROR, inline, dept)

def info(messaggio, inline=True, dept=1):
    """ log info"""
    _log(messaggio, INFO, inline, dept)

def debug(messaggio, inline=True, dept=1):
    """ log debug """
    _log(messaggio, DEBUG, inline, dept)

def warn(messaggio, inline=True, dept=1):
    """ log warn """
    _log(messaggio, WARN, inline, dept)

def timeinfo(fn):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        return timelog(fn, INFO, *args, **kwargs)
    return wrapper

def timedebug(fn, *args, **kwargs):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        return timelog(fn, DEBUG, *args, **kwargs)
    return wrapper

def timewarn(fn, *args, **kwargs):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        return timelog(fn, WARN, *args, **kwargs)
    return wrapper

def timeerror(fn, *args, **kwargs):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        return timelog(fn, ERROR, *args, **kwargs)
    return wrapper

def timelog(fn, tipo, *args, **kwargs):
    """ Log del time """
    chelog = [info, debug, warn, errore][tipo]
    chelog("Inizio la funzione: %s" % fn.__name__, dept=2)
    t = time.time()
    r = fn(*args, **kwargs)
    chelog("Finito la funzione %s (%2.3f)" % (fn.__name__, time.time() - t))
    return r
