# -*- coding: utf-8 -*-
"""
Alcune funzioni per da usare con gli oggetti
"""
__fine_name__ = "objutils.py"
__author__ =    "luca"
__version__ =   "1.0.0"
__date__ =      "2012-01-10"


import inspect
import json
import types

from datetime           import datetime
from decimal            import Decimal

from waonline.core.ete_dumbobj import EteDumbObj

def fromJSON(obj):
    def fjson(obj):
        """ vede di convertire i dizionari in EteDumbObj """
        if isinstance(obj, dict):
            return EteDumbObj(obj)
        return obj
    return json.loads(obj, object_hook=fjson)

def toJSON(obj):
    """ fa il dumps di un oggetto in JSON """
    def jtodate(obj):
        """ se è possibile e se è una data la converte in stringa """
        if isinstance(obj, datetime):
            return obj.strftime('%d/%m/%Y')
        elif isinstance(obj, Decimal):
            return str(obj)
        return None
    return json.dumps(obj, default=jtodate)

def getJsonObj(obj, includi=[], escludi=[], custom={}):
    """ Ritorna il valore di getObjInfo già in JSON """
    return toJSON(getObjInfo(obj, includi, escludi, custom))

def getObjInfo(obj, includi=[], escludi=[], custom={}):
    """ ritorna un EteDumbObj contenente i membri di un oggetto """
    if obj:
        if isinstance(obj, list):
            lobj = []
            for nobj in obj:
                lobj.append(getObjInfo(nobj, includi, escludi, custom))
            return lobj
        else:
            if includi:
                info = EteDumbObj()
                for k in includi:
                    try:
                        k, n = k.split(":")
                    except:
                        n = k
                    try:
                        info[n] = getattr(obj, k)
                    except:
                        pass
            else:
                info = EteDumbObj(inspect.getmembers(obj))
            for k, v in info.items():
                if k[0] == '_' or k in escludi:
                    del info[k]
                elif not isinstance(k, (str, int, float)):
                    info[k] = getObjInfo(v)
            for k, v in custom.items():
                try:
                    k, n = k.split(":")
                except:
                    n = k
                if hasattr(obj, k):
                    if type(v) == types.FunctionType:
                        info[n] = v(*[obj, info, getattr(obj, k)][:len(
                            inspect.getargspec(v).args)])
                    else:
                        info[n] = getattr(obj, k)
                else:
                    if type(v) == types.FunctionType:
                        info[n] = v(*[obj, info][:len(
                            inspect.getargspec(v).args)])
                    else:
                        info[n] = v
            return info
    else:
        return None

def to_dict(campi, lista=None):
    """ ritorna una lista di ``EteDumbObj`` aventi i nomi in campi partendo da
        una lista di list o tuple, oppure partendo da una lista di tuple con
        nome e valore ( es. dict.items() ) se lista e' a None
    """
    if lista == None:
        return map(dict, campi)
    elif lista and hasattr(lista[0], '__tablename__'):
        return getObjInfo(lista, includi=campi)
    n = []
    for e in lista:
        d = EteDumbObj
        for i in xrange(len(campi)):
            d[campi[i]] = e[i]
        n.append(d)
    return n
