# coding: utf-8

import time
import base64
import json
import urllib

from client import BaseHTTPClient

from waonline.utils.xml2dict import XML2Dict

x2d = XML2Dict()

class WaoClient(BaseHTTPClient):

    get_url = '/get'
    app_url = '/app/%s/%s'

    def req(self, page, params=None):
        self.unparsed_reponse = None
        ret = BaseHTTPClient.req(self, page, params)
        self.sid = get_secure_cookie(self.cookie_val)
        return ret

    def waoget(self, gtype, gtask, params=None, get=None):
        """ esegue il get sul sito """
        url = self.get_url + "?" + urllib.urlencode(dict(
            type=gtype, task=gtask))
        if get:
            url += "&" + urllib.urlencode(get)
        return self.req(url, params)

    def login(self, username, passwd, page='/app/utenti/login'):
        """Esegui il login sul waonline """
        #import ipdb; ipdb.set_trace()
        params = {'username': username, 'passwd': passwd}
        self.username = username
        self.passwd = passwd
        self.req(page, params)

    def get_user(self):
        self.user = self.get_appc('utenti', 'get_user')
        return self.user

    def logout(self):
        """ eseque il logout """
        self.get("/logout")

    def get_appc(self, app_name, method, params=None, get=None):
        """ esegue l'app sul sito con conversione"""
        self.parsed_response = None
        preq = self.get_app(app_name, method, params, get)
        if preq.code != 200:
            return None
        self.unparsed_reponse = preq.read()
        self.parsed_response = conv(self.unparsed_reponse)
        return self.parsed_response

    def get_app(self, app_name, method, params=None, get=None):
        """ esegue l'app sul sito """
        url = self.app_url % (app_name, method)
        if get:
            url += "&" + urllib.urlencode(get)
        return self.req(url, params)

    def getc(self, gtype, gtask, params=None, get=None):
        """ esegue il get sul sito con conversione"""
        self.parsed_response = None
        preq = self.waoget(gtype, gtask, params, get)
        if preq.code != 200:
            return None
        self.unparsed_reponse = preq.read()
        self.parsed_response = conv(self.unparsed_reponse)
        return self.parsed_response

    @property
    def last_msg(self):
        try:
            return self.parsed_response['response']['messaggi'][0]['msg'].encode('utf-8')
        except:
            return None

    @last_msg.setter
    def last_msg(self):
         raise AttributeError("Proprieta in sola lettura")

    def messages(self, index=None):
        try:
            if index != None:
                return u"[%(tipo)s]:%(msg)s" % self.parsed_response['response']['messaggi'][index]
            else:
                s = ''
                for m in self.parsed_response['response']['messaggi']:
                    s += u"[%(tipo)s]:%(msg)s\n" % m
                return s
        except KeyError:
            return None




def get_secure_cookie(value):
        """Returns the given signed cookie if it validates, or None."""
        parts = value.split("|")
        if len(parts) != 3: return None
        #if _cookie_signature(parts[0], parts[1]) != parts[2]:
            #logging.warning("Invalid cookie signature %r", value)
            #return None
        timestamp = int(parts[1])
        if timestamp < time.time() - 31 * 86400:
            #logging.warning("Expired cookie %r", value)
            return None
        try:
            return base64.b64decode(parts[0])
        except:
            return None

def conv(req):
    """ tenta di convertire una stringa in dizionario python """
    try:
        d = json.loads(req)
    except:
        try:
            d = x2d.fromstring(req)
        except:
            d = req
    return d


