# coding: latin-1
"""
:File: Utility
:Author: Marco Dalla Stella
:Date: 03/06/2010
"""
import json
import os
import re
import socket
import types
from datetime           import datetime
from decimal            import Decimal
from inspect            import getmembers, getargspec
from inspect            import stack as _stack
from stat               import *

from xml.etree          import cElementTree as ET
from xml.sax.saxutils   import escape

try:
    import pwd
except:
    pwd = None

HOST = socket.gethostname()
GG_MESI = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]

def getFuncName():
    """ Ritorna in nome della funzione """
    return _stack()[1][3]

def getCaller():
    """ Ritorna il chiamante """
    return _stack()[2][3]

def toJSONResponse(risposta, status=0, invalidateCache=False,
        isDSResponse=True):
    """ trasforma la risposta in oggetto jeson con messaggi """
    response = {"response":{
        "status": status,
        "isDSResponse":isDSResponse,
        "invalidateCache": invalidateCache,
        "messaggi": risposta.messaggi,
        "data": risposta.contenuto,
        "tipoDati": risposta.tipoDati
    }}
    risposta.contenuto = json.dumps(response, separators=(", ", ":"),
            indent=4, default=lambda x: None)
    return risposta

def toXMLResponse(risposta, status=0, invalidateCache=False,
        isDSResponse=True):
    """ trasforma la risposta in xml con messaggi """
    response = ET.Element('response')
    ET.SubElement(response, 'status').text = str(status)
    ET.SubElement(response, 'isDSResponse').text = str(isDSResponse)
    ET.SubElement(response, 'invalidateCache').text = str(invalidateCache)
    ET.SubElement(response, 'tipoDati').text = str(risposta.tipoDati)
    if risposta.errori:
        terrori = ET.SubElement(response, "errori")
        for errore in risposta.errori:
            terrore = ET.SubElement(terrori, "errore")
            ET.SubElement(terrore, "tipo").text = "errore"
            ET.SubElement(terrore, "msg").text = str(errore)
    if risposta.messaggi:
        tmessaggi = ET.SubElement(response, "messaggi")
        for messaggio in risposta.messaggi:
            tmessaggio = ET.SubElement(tmessaggi, "messaggio")
            ET.SubElement(tmessaggio, "tipo").text = "messaggio"
            ET.SubElement(tmessaggio, "msg").text =str( messaggio)
    data = ET.SubElement(response, "data")
    if isinstance(risposta.contenuto, str):
        data.text = str(risposta.contenuto)
    else:
        ncampo, contenuto = risposta.contenuto.items()[0]
        for nrecord in contenuto:
            record = ET.SubElement(data, ncampo)
            for campo, valore in nrecord.items():
                ET.SubElement(record, campo).text = str(valore)
    risposta.contenuto = ET.tostring(response)
    return risposta

def chk_make_dir(dir_path):
    """ controlla l'esistenza del path, lo creae e se puo' cambia l'owner """
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)
        if pwd:
            u = pwd.getpwnam("waonline")
            os.chown(dir_path, u.pw_uid, u.pw_gid)
            os.chmod(dir_path, S_IRUSR + S_IWUSR + S_IXUSR + S_IRGRP + S_IWGRP + S_IXGRP + S_IROTH + S_IXOTH)

def random_password(lunghezza=8):
    """ Crea una password a caso """
    from random import choice
    from string import ascii_letters, digits
    chars = ascii_letters + digits
    return "".join(choice(chars) for x in range(lunghezza)).upper()

def accentate2entity(stringa):
    """ converte i caratteri accentati in entit xml/html valide """
    if not stringa:
        return ''
    caratteri = [(u'\xe0', '&agrave;'),
            (u'\xe8', '&egrave;'),
            (u'\xec', '&igrave;'),
            (u'\xf2', '&ograve;'),
            (u'\xf9', '&ugrave;')]
    for uni, ent in caratteri:
        stringa = stringa.replace(uni, ent)
    return stringa

def formatData(data, tipo='str', formato=None):
    """ Formatta una data in vari modi """
    rm = re.compile("^(\d{4})\D(\d{2})\D(\d{2}).*")
    if not data:
        return None
    formati = [("%Y%m%d", "%d/%m/%Y"), ("%Y%m", "%m/%Y"), ("%Y", "%Y")]
    data = str(data)
    data = rm.sub(r'\1\2\3', data)
    if data.count("/")==1:
        data = "01/" + data
    elif len(data) == 6:
        data += "01"
    for i in range(2):
        for t in formati:
            try:
                nd = datetime.strptime(data, t[i])
                if tipo == 'int':
                    if formato:
                        return int(nd.strftime(formato))
                    else:
                        return int(nd.strftime(t[(1 if i == 0 else 0)]))
                elif tipo == 'date':
                    return nd
                else:
                    if formato:
                        return nd.strftime(formato)
                    else:
                        return nd.strftime(t[(1 if i == 0 else 0)])
            except:
                pass
    return data


def framData(data, isString=False):
    data = str(data)
    if len(data) == 6:
        data += "00"
    if isString:
        return [data[0:4], data[4:6], data[6:8]]
    else:
        return [int(data[0:4]), int(data[4:6]), int(data[6:8])]


def getGiorniMese(data):
    """ ritorna il numero di giorni del mese"""
    anno = int(str(data)[0:4])
    mese = int(str(data)[4:6])
    if mese == 2:
        if (anno % 4) == 0:
            return 29
    return GG_MESI[mese - 1]


def isDataOK(data, isPeriodo=False):
    """ verifica la validit della data """
    if len(str(data)) != 6 and len(str(data)) != 8: # 6:periodo, 8:data
        return False
    if framData(data)[0] < 1000:
        return False
    if framData(data)[1] < 1 or framData(data)[1] > 12:
        return False
    if framData(data)[2] < 1 or framData(data)[2] > getGiorniMese(data):
        return False
    return True


def isOrarioOK(orario):
    """ verifica la validit dell'orario """
    orario = str(orario)
    (ore, min) = orario.split(".")
    if int(ore) > 23 or int(min) > 59:
        return False
    else:
        return True


class Risposta():
    """ Classe per la risposta """
    def __init__(self):
        self.contenuto = None
        self.errori = []
        self.messaggi = []
        self.header = []
        self.sorgente = None
        self.tipoDati = "xml"
        self.chunked = None
        self.cookies = []
        self.status = None

    def __repr__(self):
        return str(self.__dict__)


def getObjAttrXML(obj, nome_elemento, **kwargs):
    """ Ritorna un xml per l'oggetto passato """
    escludi = kwargs.get('escludi', [])
    includi = kwargs.get('includi', None)
    tabs = kwargs.get('tabs', 0)
    custom = kwargs.get('custom', None)

    xml = "\t" * tabs + "<%s>\n" % nome_elemento

    if includi :
        for attribute in includi:
            value = getattr(obj, attribute)
            if (type(value) in [str, int, bool, unicode, float]) and (
                    attribute not in escludi):
                if type(value) in [str, unicode]:
                    value = escape(value)
                try:
                    value = unicode(value)
                except:
                    value = '__valore non valido__'
                if value == "False":
                    value = 'false'
                if value == "True":
                    value = 'true'
                xml += "\t" * (tabs+1) + "<%s>%s</%s>\n" % (attribute, value,
                        attribute)
    else:
        for attribute in [a for a in dir(obj) if a[0] != "_"]:
            value = getattr(obj, attribute)
            if (type(value) in [str, int, bool, unicode, float]) and (
                    attribute not in escludi):
                if type(value) in [str, unicode]:
                    value = escape(value)
                try:
                    value = unicode(value)
                except:
                    value = '__valore non valido::'
                if value == "False":
                    value = 'false'
                if value == "True":
                    value = 'true'
                xml += "\t" * (tabs+1) + "<%s>%s</%s>\n" % (attribute, value,
                        attribute)
    if custom:
        for attribute, value in custom.items():
            if type(value) in [str, unicode]:
                value = escape(value)
            try:
                value = unicode(value)
            except:
                value = '__valore non valido__'
            if value == "False":
                value = 'false'
            if value == "True":
                value = 'true'
            xml += "\t" * (tabs+1) + "<%s>%s</%s>\n" % (attribute, value,
                    attribute)

    xml += "\t" * tabs + "</%s>\n" % nome_elemento

    return xml


def doppieInLista(lista):
    """ verifica se ci sono doppie in una lista """
    if type(lista) == types.ListType:
        return len(lista) - len(set(lista)) > 0
    if type(lista) == types.DictionaryType:
        return len(lista.values()) - len(set(lista.values())) > 0
    return None


def converti_ore(ore, is_60, is_stringa=False):
    """ converte le ore """
    hh = str(ore).split(".")[0]
    mm = ("%4.2f" % ore).split(".")[1]
    if is_60:
        mm = str(round(float(mm) * float(60) / float(100), 0)).split('.')[0]
    else:
        mm = str(round(float(mm) * float(100) / float(60), 0)).split('.')[0]
    if is_stringa:
        return "%s.%s" %(hh, mm)
    else:
        return Decimal("%s.%s" %(hh, mm))


def debug(rbase, txt=False, *args):
    dbg = ">>>>>>>>>>>>>>>> "
    if txt:
        dbg += str(txt) + ": "
    if args:
        sw_primo = True
        for arg in args:
            if sw_primo:
                sw_primo = False
            else:
                dbg += "  _  "
            dbg += str(arg)
    try:
        rbase.debug(dbg)
    except:pass


class ObjectToXML():
    """ Classe ObjectToXML per trasformare da struttura Python a XML """
    def __init__(self):
        self.adapt = {
            dict: self.getXML_dict,
            list: self.getXML_list,
            tuple: self.getXML_list,
        }

    def getXML(self, obj, objname=None):
        """ recupera l'xml """
        if obj == None:
            return ""
        if not objname:
            objname = "root"
        if self.adapt.has_key(obj.__class__):
            return self.adapt[obj.__class__](obj, objname)
        else:
            val = str(obj)
            if val == "False":
                val = 'false'
            if val == "True":
                val = 'true'
            return "<%(n)s>%(o)s</%(n)s>" % {'n':objname, 'o':val}

    def getXML_dict(self, indict, objname=None):
        """ dal dizionario """
        h = "<%s>" % objname
        for k, v in indict.items():
            h += self.getXML(v, k)
        h += "</%s>" % objname
        return h

    def getXML_list(self, inlist, objname=None):
        """ dalla lista """
        h = ""
        for i in inlist:
            h += self.getXML(i, objname)
        return h

def toJSON(obj):
    """ fa il dumps di un oggetto in JSON """
    def jtodate(obj):
        """ se  possibile e se  una data la converte in stringa """
        if isinstance(obj, datetime):
            return obj.strftime('%d/%m/%Y')
        elif isinstance(obj, Decimal):
            return str(obj)
        return None
    return json.dumps(obj, default=jtodate)

def getJsonObj(obj, includi=[], escludi=[], custom={}):
    """ Ritorna il valore di getObjInfo gi in JSON """
    return toJSON(getObjInfo(obj, includi, escludi, custom))

def getObjInfo(obj, includi=[], escludi=[], custom={}):
    """ ritorna un dizionario contenente i membri di un oggetto """
    if obj:
        if isinstance(obj, list):
            lobj = []
            for nobj in obj:
                lobj.append(getObjInfo(nobj, includi, escludi, custom))
            return lobj
        else:
            if includi:
                info = {}
                for k in includi:
                    try:
                        k, n = k.split(":")
                    except:
                        n = k
                    try:
                        info[n] = getattr(obj, k)
                    except:
                        pass
            else:
                info = dict(getmembers(obj))
            for k, v in info.items():
                if k[0] == '_' or k in escludi:
                    del info[k]
                elif not isinstance(k, (str, int, float)):
                    info[k] = getObjInfo(v)
            for k, v in custom.items():
                try:
                    k, n = k.split(":")
                except:
                    n = k
                if hasattr(obj, k):
                    if type(v) == types.FunctionType:
                        info[n] = v(*[obj, info, getattr(obj, k)][:len(
                            getargspec(v).args)])
                    else:
                        info[n] = getattr(obj, k)
                else:
                    if type(v) == types.FunctionType:
                        info[n] = v(*[obj, info][:len(getargspec(v).args)])
                    else:
                        info[n] = v
            return info
    else:
        return None

def to_dict(campi, lista=None):
    """ ritorna una lista di dizionari aventi i nomi in campi partendo da
        una lista di list o tuple, oppure partendo da una lista di tuple con
        nome e valore ( es. dict.items() ) se lista e' a None
    """
    if lista == None:
        return map(dict, campi)
    elif lista and hasattr(lista[0], '__tablename__'):
        return getObjInfo(lista, includi=campi)
    n = []
    for e in lista:
        d = {}
        for i in xrange(len(campi)):
            d[campi[i]] = e[i]
        n.append(d)
    return n

def efill(result, total, fill=None):
    """ prende la lista in result e la restituisce modificata in base a total
        e fill
    """
    s = result[:]
    for x in xrange(len(result), total):
        s.append(fill)
    return s[:total]


